// CIF mapping amendments
// $Id: CIF.h,v 1.7 2002/07/24 22:56:35 alexy Exp $

#ifndef _CIF_H_
#define _CIF_H_

#include <vector>
#include <string>
#include <map>

class Item;

using namespace std;
typedef vector<string>      Parents;
typedef map<string, string> Suffixes;

class CIF {
 public:
  CIF();

  bool isVector(const Item*)    const;
  bool isCartesian(const Item*) const;
  bool isMatrix(const Item*)    const;

  string vectorName(const Item*) const;
  string cartesianName(const Item*) const;
  string matrixName(const Item*) const;

  bool isVectorHead(const Item*) const;
  bool isVectorTail(const Item*, const string) const;

  bool isCartesianHead(const Item*) const;
  bool isCartesianTail(const Item*, const string) const;

  bool isMatrixHead(const Item*) const;
  bool isMatrixTail(const Item*, const string) const;

  bool   isAtomIndexHead(const Item*) const;
  string atomIndexName(const   Item*) const;
  string isAtomIndexTail(const Item*, const string) const;

  bool   isSeqIndexHead(const Item*) const;
  string seqIndexName(const   Item*) const;
  string isSeqIndexTail(const Item*, const string) const;

  string indexName(const Item*) const;

  static const string atom;
  static const string seq;
  static const string comp;
  static const string asym;
  static const string alt;

 private:
  Parents  atomIndexParents;
  Suffixes atomIndexSuffixes;

  Parents  seqIndexParents;
  Suffixes seqIndexSuffixes;
};

#endif
