// Category
// $Id: Category.h,v 1.15 2002/08/17 03:46:15 alexy Exp $

#ifndef _CATEGORY_H_
#define _CATEGORY_H_

#include "Item.h"

#include <string>
#include <vector>

#include "CifFileObj.h"
#include "Flags.h"
#include "CategoryGroup.h"

class Category {
public:
  // there should be a way to ensure it's called before
  // any Item object is constructed, at least by throwing
  // an exception otherwise
  static void setCifBuilder(CifBuilder *cb);

  Category();
  Category(string name);

  virtual ~Category();

  bool isMandatory() const;
  bool isOptional() const;

  void addItem(Item *ip);

  Strings itemNames() const;
  Item* getItem(const string) const;
  void deleteItem(const string);

  string name() const;
  string nameIDL() const;

  string nameDB() const; // accessor
  void   nameDB(const string);       // modifier
  
  string IDL(const short tab =0);

  string DB2(const short tab =0);

  string client(const short tab =0);

  void numberFlags(int& n, const bool split);
  string optFlags(const short tab =0);
  void setFlags(Flags& flags, const string& id);

  string entryCalls(const short tab =0);

  const long nItems() const;

  void transform();

  IDL::CategoryGroup groupIDL() const;

private:
  static CifBuilder* _cb; // CifBuilder's own Get methods are not const

  bool   _mandatory;
  bool   _hasOptionalItems;

  string _name;
  string _nameDB;
  string _nameIDL;
  
  string _optFlagName;
  int    _optFlagNumber;

  Items  _items;

  bool _isStruct;

  void checkOptional();
  void index();
  void aggregate();

  void aggregateBlocks();
  void aggregateTensors();

  int itemPosByName(const string) const;

  bool _dbChecked;
  void dbCheck();
  bool _dbAbsent;

  IDL::CategoryGroup _groupIDL; // MMS_CLASS, XRAY_CLASS, etc.
  string scopedIDL(const string) const;
};
#endif
