// CorbaCif body
// $Id: CorbaCif.C,v 1.16 2002/08/17 03:46:15 alexy Exp $

#include "CorbaCif.h"
#include <iostream>
#include <algorithm>
#include "util.h"
#include <assert.h>

namespace ccif {

  CorbaCif::CorbaCif()
  {}


  CorbaCif::CorbaCif(const char *cifName, const char *dataBlock)
  {
    CifFileObj * cifObj = new CifFileObj("", WRITE_MODE, 80, "?", 0);
    char * diags = NULL;

    cifObj->Read((char*)cifName, diags); 
    assert(!diags);

    tables.resize(numberTables);

    tables[blockTable]   = cifObj->GetTable((char*)dataBlock,     tableNames[blockTable]);
    assert(tables[blockTable]);
    tables[blockItemTable] = cifObj->GetTable((char*)dataBlock,   tableNames[blockItemTable]);
    assert(tables[blockItemTable]);

    tables[tensorTable]   = cifObj->GetTable((char*)dataBlock,    tableNames[tensorTable]);
    assert(tables[tensorTable]);
    tables[tensorIndexTable] = cifObj->GetTable((char*)dataBlock, tableNames[tensorIndexTable]);
    assert(tables[tensorIndexTable]);
    tables[tensorItemTable] = cifObj->GetTable((char*)dataBlock,  tableNames[tensorItemTable]);
    assert(tables[tensorItemTable]);

    tables[categorySplitTable] = cifObj->GetTable((char*)dataBlock, tableNames[categorySplitTable]);
    assert(tables[categorySplitTable]);
    tables[categoryItemTable]   = cifObj->GetTable((char*)dataBlock, tableNames[categoryItemTable]);
    assert(tables[categoryItemTable]);

    tables[categoryStructTable] = cifObj->GetTable((char*)dataBlock, tableNames[categoryStructTable]);
    assert(tables[categoryStructTable]);

    tables[categoryGroupTable] = cifObj->GetTable((char*)dataBlock, tableNames[categoryGroupTable]);
    assert(tables[categoryGroupTable]);
  }


  CorbaCif::~CorbaCif()
  {
    // Apparently the tables are within cifObj, breaks:
    // for (int i = 0; i < tables.size(); ++i)
    //   delete tables[i];

    delete cifObj;
  }


  Blocks CorbaCif::blocks(const string category, int& errCode)
  {

    ReVarPCifArray<int>      blockColumnList;
    ReVarCifArray<CifString> blockSearchList;
    ReVarPCifArray<int>      *blockList;

    blockColumnList.Add(blockCategoryColumn);
    blockSearchList.Add(category.c_str());

    Blocks v;

    blockList = tables[blockTable]->Search(blockSearchList, blockColumnList, errCode);
    if (errCode) return v;

    CifString s;

    for (int i = 0; i < blockList->Length(); ++i) {

      Block r;

      tables[blockTable]->GetCell(s, blockNameColumn, (*blockList)[i]);
      r.name     = string(s.Text());

      tables[blockTable]->GetCell(s, blockTypeColumn, (*blockList)[i]);
      r.type     = string(s.Text());

      v.push_back(r);
    }

    delete blockList;
    return v;
  }


  BlockItems CorbaCif::blockItems(const string category, const string block, int& errCode)
  {
    ReVarPCifArray<int>      blockItemColumnList;
    ReVarCifArray<CifString> itemSearchList;
    ReVarPCifArray<int>      *itemList;

    blockItemColumnList.Add(blockItemCategoryColumn);
    blockItemColumnList.Add(blockItemBlockColumn);

    itemSearchList.Add(category.c_str());
    itemSearchList.Add(block.c_str());

    BlockItems v;

    itemList = tables[blockItemTable]->Search(itemSearchList, blockItemColumnList, errCode);
    if (errCode) return v;

    CifString s;

    for (int i = 0; i < itemList->Length(); ++i) {

      BlockItem r;

      tables[blockItemTable]->GetCell(s, blockItemNameColumn, (*itemList)[i]);
      r.name     = string(s.Text());

      tables[blockItemTable]->GetCell(s, blockItemFieldColumn, (*itemList)[i]);
      r.field    = string(s.Text());

      v.push_back(r);
    }

    delete itemList;
    return v;
  }


  Strings CorbaCif::blockItemNames(const string category, const string block, int& errCode)
  {
    Strings names;

    BlockItems items = blockItems(category, block, errCode);
    if (errCode) return names;

    for(BlockItems::const_iterator 
	  it = items.begin(); it != items.end(); ++it)
      names.push_back(it->name);

    return names;
  }


  Tensors       CorbaCif::tensors(const string category, int& errCode)
  {
    ReVarPCifArray<int>      tensorColumnList;
    ReVarCifArray<CifString> tensorSearchList;
    ReVarPCifArray<int>      *tensorList;

    Tensors v;

    tensorColumnList.Add(tensorCategoryColumn);
    tensorSearchList.Add(category.c_str());

    tensorList = tables[tensorTable]->Search(tensorSearchList, tensorColumnList, errCode);
    if (errCode) return v;

    CifString s;

    for (int i = 0; i < tensorList->Length(); ++i) {

      Tensor r;

      tables[tensorTable]->GetCell(s, tensorNameColumn, (*tensorList)[i]);
      r.name     = string(s.Text());

      tables[tensorTable]->GetCell(s, tensorTypeColumn, (*tensorList)[i]);
      r.type     = string(s.Text());

      tables[tensorTable]->GetCell(s, tensorRowOrderColumn, (*tensorList)[i]);
      r.row_major = s == tensorRowMajor;
      
      tables[tensorTable]->GetCell(s, tensorKindColumn, (*tensorList)[i]);
      r.kind     = string(s.Text());

      v.push_back(r);
    }

    delete tensorList;
    return v;
  }


  TensorIndexes CorbaCif::tensorIndexes(const string category, const string tensor, int& errCode)
  {
    ReVarPCifArray<int>      tensorIndexColumnList;    
    ReVarCifArray<CifString> indexSearchList;
    ReVarPCifArray<int>      *indexList;

    TensorIndexes v;

    tensorIndexColumnList.Add(tensorIndexCategoryColumn);
    tensorIndexColumnList.Add(tensorIndexTensorColumn);

    indexSearchList.Add(category.c_str());
    indexSearchList.Add(tensor.c_str());

    indexList = tables[tensorIndexTable]->Search(indexSearchList, tensorIndexColumnList, errCode);
    if (errCode) return v;

    CifString s;

    for (int i = 0; i < indexList->Length(); ++i) {

      TensorIndex r;

      tables[tensorIndexTable]->GetCell(s, tensorIndexNameColumn, (*indexList)[i]);
      r.name     = string(s.Text());

      tables[tensorIndexTable]->GetCell(s, tensorIndexLengthColumn, (*indexList)[i]);
      r.length     = atoi(s.Text()); // NB: validate

      v.push_back(r);
    }

    delete indexList;
    return v;
  }


  struct TensorElement {
    string name;
    int index;
    int position;
  };
  typedef vector<TensorElement> TensorElements;


  TensorItems   CorbaCif::tensorItems(const string category, const string tensor, int& errCode)
  {
    ReVarPCifArray<int>      tensorItemColumnList;
    ReVarCifArray<CifString> itemSearchList;
    ReVarPCifArray<int>      *itemList;

    TensorItems v;

    tensorItemColumnList.Add(tensorItemCategoryColumn);
    tensorItemColumnList.Add(tensorItemTensorColumn);

    itemSearchList.Add(category.c_str());
    itemSearchList.Add(tensor.c_str());

    itemList = tables[tensorItemTable]->Search(itemSearchList, tensorItemColumnList, errCode);
    if (errCode) return v;

    CifString s;

    TensorElements e;

    for (int i = 0; i < itemList->Length(); ++i) {

      TensorElement r;

      tables[tensorItemTable]->GetCell(s, tensorItemNameColumn, (*itemList)[i]);
      r.name     = string(s.Text());

      tables[tensorItemTable]->GetCell(s, tensorItemIndexColumn, (*itemList)[i]);
      r.index     = atoi(s.Text()); // NB: validate

      tables[tensorItemTable]->GetCell(s, tensorItemPositionColumn, (*itemList)[i]);
      r.position     = atoi(s.Text()); // NB: validate

      e.push_back(r);
    }
    delete itemList;

    TensorIndexes indexes = tensorIndexes(category, tensor, errCode);
    if (errCode) return v;

    while (e.size()) {
      TensorItem r;
      r.pos.resize(indexes.size());
      // we don't use index lengths?

      vector<TensorElements::iterator> remove;
      r.name = e[0].name;
      for (TensorElements::iterator
	     it = e.begin(); it != e.end(); ++it)
	if (it->name == r.name) {
	  r.pos[it->index - tensorIndexOneBased] = it->position;
	  remove.push_back(it);
	}

      sort(remove.begin(), remove.end());
      while (!remove.empty()) {
	e.erase(remove.back());
	remove.pop_back();
      }
      v.push_back(r);
    }
    return v;
  }

  Strings       CorbaCif::tensorItemNames(const string category, const string tensor, int& errCode)
  {
    Strings v;

    // could do a table search and fetch just names, then make unique,
    // or reuse the above:
    TensorItems items = tensorItems(category, tensor, errCode);
    if (errCode) return v;

    for (int i = 0; i < items.size(); ++i) 
      v.push_back(items[i].name);

    return v;
  }


  CategorySplits CorbaCif::categorySplits()
  {
    CategorySplits v;
    
    int numRows = tables[categorySplitTable]->GetNumRows();

    CifString s;

    for (int i = 0; i < numRows; ++i) {
      CategorySplit r;

      tables[categorySplitTable]->GetCell(s, categorySplitParentColumn, i);
      r.parent = string(s.Text());

      tables[categorySplitTable]->GetCell(s, categorySplitChildColumn, i);
      r.child = string(s.Text());

      v.push_back(r);
    }

    return v;
  }


  Strings CorbaCif::categoryItems(const string category, int& errCode)
  {
    ReVarPCifArray<int>      itemColumnList;
    ReVarCifArray<CifString> itemSearchList;
    ReVarPCifArray<int>      *itemList;

    Strings v;

    itemColumnList.Add(categoryItemCategoryColumn);
    itemSearchList.Add(category.c_str());

    itemList = tables[categoryItemTable]->Search(itemSearchList, itemColumnList, errCode);
    if (errCode) return v;

    CifString s;

    for (int i = 0; i < itemList->Length(); ++i) {

      string r;

      tables[categoryItemTable]->GetCell(s, categoryItemNameColumn, (*itemList)[i]);
      r = string(s.Text());

      v.push_back(r);
    }
    
    return v;
  }
  

  bool CorbaCif::categoryStruct(const string name)
  {
    int numRows = tables[categoryStructTable]->GetNumRows();

    CifString s;

    // or could use a search on category name
    for (int i = 0; i < numRows; ++i) {

      tables[categoryStructTable]->GetCell(s, categoryStructColumn, i);
      if (name == string(s.Text())) return true;

    }

    return false;
  }


  IDL::CategoryGroup CorbaCif::categoryGroup(const string name)
  {
    int numRows = tables[categoryGroupTable]->GetNumRows();

    CifString s; // string className;

    // or could use a search on category name
    for (int i = 0; i < numRows; ++i) {

      tables[categoryGroupTable]->GetCell(s, categoryGroupCategoryColumn, i);
      if (name == string(s.Text())) {
	tables[categoryGroupTable]->GetCell(s, categoryGroupColumn, i);
	for (int i = 1; categoryGroupNames[i]; ++i) // i < NUMBER_OF_CLASSES
	  if (strcmp(s.Text(), categoryGroupNames[i]) == 0) return (IDL::CategoryGroup)i;
	return IDL::UNKNOWN_CLASS; // 0
      }
    }

    return IDL::UNKNOWN_CLASS;
  }
  
} // namespace ccif
