// Common for Item and CorbaCif
// $Id$

#ifndef _CORBA_CIF_STRUCT_H_
#define _CORBA_CIF_STRUCT_H_

#include <vector>
#include <string>
using namespace std;

#ifndef NULL
#define NULL ""
#endif

class Item;

namespace ccif {

  enum {blockTable,  blockItemTable, 
	tensorTable, tensorIndexTable, tensorItemTable, 
	categorySplitTable, categoryItemTable,
	categoryStructTable, categoryGroupTable,
	numberTables};

  extern const char *tableNames[];

  enum {blockCategoryColumn, blockNameColumn, blockTypeColumn}; 
  // , blockTableNumberColumns -- would equal it if the last enum above
  extern const char *blockTableColumnNames[]; 


  enum {blockItemCategoryColumn, blockItemBlockColumn, 
	blockItemNameColumn, blockItemFieldColumn};
  extern const char *blockItemTableColumnNames[];


  enum {tensorCategoryColumn, tensorNameColumn, tensorTypeColumn, 
	tensorRowOrderColumn, tensorKindColumn};
  extern const char *tensorTableColumnNames[];
  extern const char tensorRowMajor[];
  extern const char tensorKindVector[];
  extern const char tensorKindSquare[];
  extern const char tensorKindTriangular[];
  extern const char tensorKindStrange[];


  enum {tensorIndexCategoryColumn, tensorIndexTensorColumn, 
	tensorIndexNameColumn, tensorIndexLengthColumn};
  extern const char *tensorIndexTableColumnNames[];
  extern const int  tensorIndexOneBased;


  enum {tensorItemCategoryColumn, tensorItemTensorColumn, tensorItemNameColumn, 
	tensorItemIndexColumn, tensorItemPositionColumn};
  extern const char *tensorItemTableColumnNames[];


  enum {categorySplitParentColumn,  categorySplitChildColumn};
  // no names needed here

  enum {categoryItemCategoryColumn, categoryItemNameColumn};
  // no names needed here

  enum {categoryStructColumn};
  // no names needed here

  extern const char *categoryGroupNames[];
  
  enum {categoryGroupCategoryColumn, categoryGroupColumn};
  // no names needed here

  extern const char **tableColumnNames[];


  // structures corresponding to row results, only cells ranges which differ

  struct Block {
    string name;
    string type;
  };
  typedef vector<Block> Blocks;

  struct BlockItem {
    string name;
    string field;
    Item   *ip;
  };
  typedef vector<BlockItem> BlockItems;


  struct Tensor {
    string name;
    string type;
    bool   row_major;
    string kind;
  };
  typedef vector<Tensor> Tensors;  

  struct TensorIndex {
    string name;
    int    length;
  };
  typedef vector<TensorIndex> TensorIndexes; // sic -- for easy replace

  struct TensorItem {
    string name;
    vector<int> pos;
    Item   *ip;
  };
  typedef vector<TensorItem> TensorItems;


  struct CategorySplit {
    string parent;
    string child;
  };
  typedef vector<CategorySplit> CategorySplits;
 
} // namespace ccif

#endif
