// Bitarray flags body
// $Id: Flags.C,v 1.3 2002/07/26 21:29:49 alexy Exp $

#include "Flags.h"
#include <iostream>

Flags::Flags()
  : _bytes(0), _bit_size(0), _num_bits(0)
{
  cerr << "Flags(): "
       << "_bit_size = " << _bit_size
       << ", _num_bits = " << _num_bits
       << endl;
}


Flags::~Flags()
{}


Flags::Flags(const long unsigned bit_size)
  : _bytes((bit_size+7)/8), _bit_size(bit_size), _num_bits(0)

{
  // why didn't Flags() do _num_bits(0) itself,
  // so I had to do it below explicitly?!  
  // Flags(); 
  // _bytes.resize((bit_size+7)/8);
  cerr << "Flags(" << bit_size << "): " 
       << "_bit_size = " << _bit_size
       << ", _num_bits = " << _num_bits
       << endl;
}


Flags::Flags(const unsigned char *buffer, const long unsigned size)
{
  load(buffer, size);
}


void Flags::resize(const long unsigned number)
{
  if (_num_bits > _bit_size) _num_bits = _bit_size;
  _bit_size = number;
  _bytes.resize((number+7)/8);
}


void Flags::set(const long unsigned number)
{
  if (_num_bits <= number) _num_bits = number + 1;

  const unsigned byte = number/8, bit = number%8;

  if (byte >= _bytes.size()) _bytes.resize(byte);
  _bytes[byte] |= 1 << bit;
}


bool Flags::get(const long unsigned number)
{
  const unsigned byte = number/8, bit = number%8;

  if (byte >= _bytes.size()) return false;

  return _bytes[byte] & (1 << bit);
}


namespace flags {
  vector<string> name;
  bool name_inited = false;

#include "auto/flags.C"

  void name_init()
  {
    if (name_inited) return;
    name.resize(MAX_FLAG+1);
#include "auto/flagNames.C"
    name_inited = true;
  }
} // namespace flags


void Flags::show()
{
  flags::name_init();
  cerr << "_num_bits = " << _num_bits << endl;
  
  for (long unsigned i = 0; i < flags::MAX_FLAG; ++i)
    cout << get(i) << '\t' << flags::name[i] << endl;
}


long unsigned Flags::bit_size() const
{
  return _bit_size;
}

long unsigned Flags::byte_size() const
{
  return _bytes.size();
}


long unsigned Flags::num_bits() const
{
  return _num_bits;
}


void Flags::num_bits(const long unsigned num_bits)
{
  _num_bits = num_bits;
}


vector<unsigned char>& Flags::vBuffer()
{
  return _bytes;
}


unsigned char *Flags::buffer()
{
  return &_bytes[0];
}


void Flags::load(const unsigned char *buffer, const long unsigned size)
{
  _bytes.resize(size);
  memcpy(&_bytes[0], buffer, size);
  _bit_size = size * 8;
  _num_bits = _bit_size;
}
