// IDL body
// $Id: IDL.C,v 1.9 2003/05/04 17:53:06 alexy Exp $


#include "IDL.h"
#include "options.h"
#include "util.h"
#include <iostream>
#include <fstream>
#include <assert.h>

namespace IDL {
  const string tab(const short n)
  {
    string one = string(tab_width, ' ');
    
    string r;
    
    for (int i = 0; i < n; ++i)
      r += one;
    
    return r;
  }
  
  
  int type(const string intype)
  {
    
    if (intype == ""
	|| intype == "code"
	|| intype == "text"
	|| intype == "atcode"
	|| intype == "ucode"
	|| intype == "line"
	|| intype == "uline"
	|| intype == "name"
	|| intype == "idname"
	|| intype == "yyyy-mm-dd"
	|| intype == "any"
	|| intype == "uchar3"
	|| intype == "uchar1"
	|| intype == "symop"
	|| intype == "yyyy-mm-dd:hh:mm"		  // pdbx
	|| intype == "email"			  // pdbx
	|| intype == "phone"			  // pdbx
	|| intype == "fax"			  // pdbx
	|| intype == "code30"			  // pdbx
	) return STRING;
    else if (intype == "float"
	     || intype == "float-range"		  // pdbx
	     ) return FLOAT;
    else if (intype == "int"
	     || intype == "int-range"		  // pdbx
	     ) return LONG;
    else {
      cerr << "CIF type " << intype << " unrecognized" << endl;
      exit(1); // impossible, or return intype
    }
  }

  
  string typeName(const int type)
  {
    switch (type) {
    case LONG:   return "long";
    case FLOAT:  return "float";
    case STRING: return "string";
    default: assert(0); // impossible
    }
  }


  string maxFlag(const unsigned n)
  {
    return flagType + " MAX_FLAG = " + util::str(n);
  }
  
  
  const short tab_width = 4;
  
  const string vectorType    = "Vector3";
  const string cartesianType = "VectorXYZ";
  const string matrixType    = "Matrix3";
  const string indexType     = "IndexId";
  const string atomIndexType = "AtomIndex";
  const string seqIndexType  = "SeqIndex";
  const string flagType      = "const short";

  const string scopeMMS      = "MMS";

  const char *prolog_mms    = IDL_DIR "prolog_mms.idl";
  const char *entry_head_mms= IDL_DIR "entry_head_mms.idl";
  const char *epilog_mms    = IDL_DIR "epilog_mms.idl";

  const char *prolog_all    = IDL_DIR "prolog_all.idl";
  const char *entry_head_all= IDL_DIR "entry_head_all.idl";
  const char *epilog_all    = IDL_DIR "epilog_all.idl";
  

  // can templatize all of the below prolog/entry_head/epilog...
  
  const char * prolog_filename()
  {
    if (opt::mms)
      return prolog_mms;
    else // if (opt::full)
      return prolog_all;
  }
  
  
  const char * entry_head_filename()
  {
    if (opt::mms)
      return entry_head_mms;
    else // if (opt::full)
      return entry_head_all;
  }
  
  
  const char * epilog_filename()
  {
    if (opt::mms)
      return epilog_mms;
    else // if (opt::full)
      return epilog_all;
  }
  
  

  string prolog()
  {
    return util::readFile(prolog_filename());
  }

  
  string entryHead()
  {
    return util::readFile(entry_head_filename());
  }


  string epilog()
  {
    return util::readFile(epilog_filename());
  }
} // namespace IDL
