// Item
// $Id: Item.C,v 1.20 2003/05/04 17:53:06 alexy Exp $

#include "Item.h"
#include "IDL.h"
#include "options.h"
#include "util.h"
#include "Category.h"
#include "global.h"

#include <algorithm>
#include <cctype>
#include "db.h"
#include "cxx.h"
#include <assert.h>

// static members

CifBuilder *Item::_cb = 0;


Item::Item() 
{}


Item::Item(char *fullName)
  :  _dbAbsent(false)
{ 
  if (fullName) _fullName = string(fullName);
  // cerr << _fullName << endl;
  
  char *type = _cb->GetItemType(fullName);
  if (type) _type = string(type);
  free(type);

  _typeIDL = IDL::type(_type);

  assert(fullName[0] == '_');
  char *dotpos = strchr(fullName, '.');
  assert(dotpos);
  assert(dotpos + 1); // non-empty item name

  *dotpos = '\0';     // ends category prefix
    
  _category = string(fullName + 1); // gets category without leading '_'
  _name = string(dotpos + 1);

  // get the abbrev now, while case of _name is preserved:
  // (however, I now use case-insensitive ISTable search...)

  _nameDB = util::lower(global::scif->abbrev(_category, _name));

  // convert _name to lowercase
  for (int i = 0; i < _name.size(); ++i)
    if (isupper(_name[i])) _name[i] = tolower(_name[i]);

  if (_nameDB == "") _nameDB = _name; // lowercase

  _nameIDL = _nameDB; // lowercase
  
  _nameDB = util::upper(_nameDB); // just happens so with our DB

  // cerr << "DB: " << _fullName << " => " << _nameDB << endl;

  *dotpos = '.'; // restored fullName

  char * mandcode = _cb->GetItemMandatoryCode(fullName);
  _mandatory = mandcode && strcmp(mandcode, "yes") == 0; // NB: make case insensitive
  free(mandcode);

  uWord subcategoriesNum;
  assert(_cb);
  char ** subcategories = _cb->GetItemSubcategories(fullName, subcategoriesNum);
  for (int i = 0; i < subcategoriesNum; ++i) {
    string subcategory(subcategories[i]);
    _subcategories.push_back(subcategory);
    free(subcategories[i]);
  }
  if (subcategories) free(subcategories);

  uWord parentsNum;
  assert(_cb);
  char ** parents = _cb->GetItemLinkedParents(fullName, parentsNum);
  for (int i = 0; i < parentsNum; ++i) {
    string parent(parents[i]);
    _parents.push_back(parent);
    free(parents[i]);
  }
  if (parents) free(parents);
  
  uWord childrenNum;
  assert(_cb);
  char ** children = _cb->GetItemLinkedChildren(fullName, childrenNum);
  for (int i = 0; i < childrenNum; ++i) {
    string child(children[i]);
    _children.push_back(child);
    free(children[i]);
  }
  if (children) free(children);
}


void Item::setCifBuilder(CifBuilder *cb)
{
  _cb = cb;
}


string Item::fullName() const
{
  return _fullName;
}


string Item::type() const
{
  return _type;
}


string Item::category() const
{
  return _category;
}


string Item::name() const
{
  return _name;
}


//  vector<string> Item::subcategories()
//  {
//    return _subcategories;
//  }


void Item::setCategory(Category *cat)
{
  _theCategory = cat;
}


ItemIndex::ItemIndex(Item *ip, const string name, const string original)
  : Item(*ip), _originalName(original)
{
  _name = name;
  _originalType = _type;
  _type = IDL::indexType;
  _nameIDL = _name;
}


string ItemIndex::IDL()
{
  return _type + ' ' + _nameIDL;
}


bool Item::isSubcategory(const string subcategory) const
{
  return find(_subcategories.begin(), _subcategories.end(), subcategory)
    != _subcategories.end();
}


bool Item::hasParent(const string parent) const
{
  return find(_parents.begin(), _parents.end(), parent)
    != _parents.end();
}


bool Item::hasParents() const
{
  return _parents.size();
}


bool Item::hasChildren() const
{
  return _children.size();
}


bool Item::isIndex() const
{
  return hasParents() || hasChildren();
}


string Item::IDL()
{
  string r;

  if (opt::keep_types)
    r = _type + ' ' + _nameIDL;
  else
    r = IDL::typeName(_typeIDL) + ' ' + _nameIDL;

  return r;
}


bool Item::isMandatory() const
{
  return _mandatory;
}


bool Item::isOptional() const
{
  return !_mandatory;
}


string Item::moreInfo() const
{
  string r;

  if (opt::mandatory)
    r += _mandatory? " [+]" : " [ ]";

  if (opt::parents && hasParents()) {
    r += " [parents:";
    for (int i = 0; i < _parents.size(); ++i)
      r += ' ' + _parents[i];
    r += ']';
  }

  if (opt::children && hasChildren()) {
    r += " [children:";
    for (int i = 0; i < _children.size(); ++i)
      r += ' ' + _children[i];
    r += ']';
  }
  
  return r;
}


void Item::numberFlags(int& n, const bool split)
{
  if (_mandatory) return;
  
  _optFlagName = "F_" + util::upper(_theCategory->name()) + '_' + util::upper(_nameIDL);

  _optFlagNumber = n++;
}


string Item::optFlags(const short t)
{
  if (_mandatory) return "";

  return IDL::tab(t) + IDL::flagType + ' ' + _optFlagName + " = " + util::str(_optFlagNumber) + ";\n";
}


ItemAggregate::ItemAggregate(Category *cp, 
	    const string category,
	    const string name,
	    const string type)
{
  _theCategory = cp;
  // _category = category;
  _name = name;
  _type = type;
  _mandatory = false;
  _nameIDL = name;
}


ItemAggregate::~ItemAggregate()
{
  for (Items::reverse_iterator
	 it = _subitems.rbegin(); it != _subitems.rend(); --it)
    delete (*it); // isn't it automatic?
}


void ItemAggregate::addChild(Item *ip)
{
  _subitems.push_back(ip);
}


string ItemAggregate::IDL()
{
  return _type + ' ' + _nameIDL;
}


void ItemAggregate::numberFlags(int& n, const bool split)
{
  if (_mandatory) return;

  _optFlagsSplit = split;
  
  if (split)
    for (Items::const_iterator
	   it = _subitems.begin(); it != _subitems.end(); ++it)
      (*it)->numberFlags(n, split);
  else
    Item::numberFlags(n, split);
}


string ItemAggregate::optFlags(const short t)
{
  string r;

  // cerr << "block " << _name << " has " << _subitems.size() << " children" << endl;
  if (_mandatory) return r;

  if (_optFlagsSplit)
    for (Items::const_iterator
	   it = _subitems.begin(); it != _subitems.end(); ++it)
      r += (*it)->optFlags(t);
  else 
    r = Item::optFlags(t);
  
  return r;
}


void ItemAggregate::checkOptional()
{
  // _mandatory is false originally

  for (Items::const_iterator
	 it = _subitems.begin(); it != _subitems.end(); ++it)
    if ((*it)->isMandatory()) {
      _mandatory = true;
      // cerr << _name << " is mandatory" << endl;
      return;
    }
  // _mandatory stays false
}


string ItemAggregate::moreInfo() const
{
  return Item::moreInfo();
}


ItemBlock::ItemBlock(Category *cp, 
	    const string category,
	    const string name,
	    const string type)
  : ItemAggregate(cp, category, name, type)
{}

ItemBlock::~ItemBlock() {}


ccif::BlockItems& ItemBlock::getMembers()
{
  return _members;
}


void ItemBlock::setMembers(ccif::BlockItems members)
{
  _members = members;
}


ItemTensor::ItemTensor(Category *cp, 
		       const string category,
		       const string name,
		       const string type,
		       const bool row_major,
		       const string kind)
  : ItemAggregate(cp, category, name, type),
    _row_major(row_major), _kind(kind)
{}

ItemTensor::~ItemTensor() {}


ccif::TensorItems& ItemTensor::getMembers()
{
  return _members;
}


void ItemTensor::setMembers(ccif::TensorItems members) {
  _members = members;
}

bool ItemTensor::row_major() const { return _row_major; }
string ItemTensor::kind()    const { return _kind; }



string ItemBlock::moreInfo() const
{
  string r = ItemAggregate::moreInfo(); 
  for (int i = 0; i < _members.size(); ++i)
    r += ' ' + _members[i].name + ':' + _members[i].field;

  return r;
}


string ItemTensor::moreInfo() const
{
  string r = ItemAggregate::moreInfo(); 

    for (int j = 0; j < _members.size(); ++j) {
      r += ' ' + _members[j].name + ':'; 
      for (int k = 0; k < _members[j].pos.size(); ++k)
	r += '(' + util::str(_members[j].pos[k]) + ')';
    }

  return r;
}


void ItemBlock::numberFlags(int& n, const bool split)
{
  if (isCartesian()) ItemAggregate::numberFlags(n, false);
  else ItemAggregate::numberFlags(n, split);
}


void ItemTensor::numberFlags(int& n, const bool split)
{
  ItemAggregate::numberFlags(n, false);
}


bool ItemBlock::isCartesian() const
{
  if (_subitems.empty()) return false;

  return global::cif->isCartesian(_subitems[0]);
}


db::Columns Item::dbColumns() const
{
  db::Columns r;

  if (_dbAbsent) return r;

  r.resize(1);
  
  r[0].dbname = _nameDB;
  r[0].ip     = (Item *)this;

  return r;
}


db::Columns ItemAggregate::dbColumns() const
{
  db::Columns r;

  for (int i = 0; i < _subitems.size(); ++i) {

    if (_subitems[i]->dbAbsent()) continue;

    db::Column c;

    // why _subitems[i]->_nameDB is protected `in this context'?

    c.dbname = _subitems[i]->nameDB();
    c.ip = _subitems[i];

    r.push_back(c);
  }

  return r;
}

  
string Item::nameDB() const
{
  return _nameDB;
}


 string Item::DB2member(const string prefix)
{
  // cerr << "Item::DB2member called for " << prefix << " idl: " << _typeIDL << " sql: " << _typeSQL << endl;

  return prefix + " = " + global::db->corbaCast(_nameDB, _typeIDL, _typeSQL) + ";\n";
}


string Item::DB2(const short t)
{
  if (_dbAbsent) return "";

  return DB2member(cxx::list_prefix(t) + _name);
}


string ItemIndex::DB2member(const string prefix)
{
  // cerr << "ItemIndex::DB2member called for " << prefix << endl;
  
  return prefix + ".id" +    " = " + global::db->corbaCast(_nameDB, IDL::STRING, _typeSQL) + ";\n"
    +    prefix + ".index" + " = " + global::db->corbaCast(_nameDB, IDL::LONG,   _typeSQL) + ";\n";
}

    
// string ItemIndex::DB2(const short t)
// -- same as Item::DB2(), but calls ItemIndex::DB2member()!


string ItemBlock::DB2(const short t)
{
  string r;

  // cerr << "ItemBlock::DB2 called for " << _name << endl;

  for (int i = 0; i < _members.size(); ++i) {
    // NB can this be?:
    if (_members[i].ip->dbAbsent()) {
      // cerr << "   ItemBlock member ABSENT: " <<  _members[i].ip->name() + '.' + _members[i].field << endl;
      continue; // _dbAbsent protected?
    }
    
    // cerr << _members[i].ip->DB2member(cxx::list_prefix(t) + _name + '.' + _members[i].field) << endl;
    r += _members[i].ip->DB2member(cxx::list_prefix(t) + _name + '.' + _members[i].field);
    
  }
  return r;
}


string ItemTensor::DB2(const short t)
{
#define ARRAY_BASE 1
  string r;

  for (int i = 0; i < _members.size(); ++i) {
    // NB can this be?:
    if (_members[i].ip->dbAbsent()) continue; // _dbAbsent protected?

    r += cxx::list_prefix(t)  + _name;
    
    for (vector<int>::const_iterator
	   jt = _members[i].pos.begin(); jt != _members[i].pos.end(); ++jt)
      r += '[' + util::str(*jt - ARRAY_BASE) + ']';
  
    // cast unnecessary for numerics
    r += " = " + _members[i].ip->nameDB() + ".val;\n";
  }

  return r;
}


void ItemBlock::linkMember(Item *ip)
{
  // cerr << "number of members: " << _members.size() << endl;
  for (ccif::BlockItems::iterator
	 kt = _members.begin(); kt != _members.end(); ++kt)
    if ((*kt).name == ip->nameMember()) { (*kt).ip = ip; return; } // point back
  // else cerr << "...nope: " << (*kt).name << endl;

  cerr << "ERROR: couldn't link " << ip->nameMember()
       << " (DB name: " << ip->nameDB() << ')' << endl;
  exit(1);
}


void ItemTensor::linkMember(Item *ip)
{
  for (ccif::TensorItems::iterator
	 kt = _members.begin(); kt != _members.end(); ++kt)
    if ((*kt).name == ip->name()) { (*kt).ip = ip; return; } // point back

  cerr << "ERROR: couldn't link " << ip->nameMember()
       << " (DB name: " << ip->nameDB() << ')' << endl;
  exit(1);
}


void Item::setSqlType(const int type)
{
  _typeSQL = type;
}


void Item::setSqlSize(const int size)
{
  _sizeSQL = size;
}


bool Item::dbAbsent() const
{
  return _dbAbsent;
}


void Item::dbCheck(const Strings& columnNamesPresent)
{
  if (!util::present(columnNamesPresent, _nameDB))
    _dbAbsent = true;
}


void ItemAggregate::dbCheck(const Strings& columnNamesPresent)
{
  // cerr << "ItemAggregate DB Check: " << _name << endl;
  
  _dbAbsent = true;
  for (Items::iterator
	 it = _subitems.begin(); it != _subitems.end(); ++it) {
    (*it)->dbCheck(columnNamesPresent); // this could return bool:
    // cerr << '\t' << (*it)->name() << ' ' << ((*it)->dbAbsent() ? "ABSENT" : "present") << endl;
    _dbAbsent &= (*it)->dbAbsent(); // once false, stays false
  }
}


void Item::setDbAbsent()
{
  _dbAbsent = true;
}


void ItemAggregate::setDbAbsent()
{
 for (Items::iterator
	 it = _subitems.begin(); it != _subitems.end(); ++it)
   (*it)->setDbAbsent();  
}


string Item::nameMember() const
{
  return _name;
}


string ItemIndex::nameMember() const
{
  return _originalName;
}



void Item::setFlags(Flags& flags, const string& id)
{
  if (_dbAbsent || _mandatory) return;

  if (global::db->itemPresent(_theCategory->nameDB(), _nameDB, id)) {
    flags.set(_optFlagNumber);
    cerr << "Setting " << _optFlagName << '(' << _optFlagNumber << ')' << endl;
  }
  //  else
  //    cerr << "ABSENT--" << _optFlagName << '(' << _optFlagNumber << ')' << endl;
}


void ItemAggregate::setFlags(Flags& flags, const string& id)
{
  if (_optFlagsSplit)
    for (Items::iterator
	   it = _subitems.begin(); it != _subitems.end(); ++it)
      (*it)->setFlags(flags, id);  
  else {
    if (_dbAbsent || _mandatory) return;
    
    bool hasSubitem = false;
    for (Items::iterator
	   it = _subitems.begin(); it != _subitems.end() && !hasSubitem; ++it) {
      if ((*it)->dbAbsent()) continue;
      hasSubitem = global::db->itemPresent(_theCategory->nameDB(), (*it)->nameDB(), id);
    }

    if (!hasSubitem) return;

    flags.set(_optFlagNumber);
    cerr << "Setting " << _optFlagName << '(' << _optFlagNumber << ')' << " [Aggregated]" << endl;
  }
}


void ItemBlock::printMembers() const
{
  cerr << "BLOCK " << _name << " members:" << endl;
  for (int i = 0; i < _members.size(); ++i) {
    cerr << '\t' << i << ' '<< _members[i].field << ' ';
    if (_members[i].ip)
      cerr << _members[i].ip->name()
	   << ' ' << "db name: " << _members[i].ip->nameDB();
    else
      cerr << "MISSING";
    cerr << endl;
  }
}
