// Item
// $Id: Item.h,v 1.19 2003/05/04 17:53:06 alexy Exp $

#ifndef _ITEM_H_
#define _ITEM_H_

#include "CifBuilder.h"

#include <string>
#include <vector>

#include "CIF.h"
#include "CorbaCifStruct.h"

#include "util.h"
#include "IDL.h" // for IDL::tab -- move to util?
#include "Column.h" // for dbColumns()
#include "Flags.h"

class Item;
typedef vector<Item*> Items;

class Category;

class Item {
  //  friend class CIF;
public:
  // there should be a way to ensure it's called before
  // any Item object is constructed, at least by throwing
  // an exception otherwise
  static void setCifBuilder(CifBuilder *cb);

  Item();
  Item(char *fullName);

  void setCategory(Category *);

  virtual string IDL();
  virtual void numberFlags(int&, const bool split);
  virtual string optFlags(const short tab);
  virtual void setFlags(Flags& flags, const string& id);

  string fullName() const;
  string type() const;
  string category() const;
  string name() const;
  virtual string nameMember() const;
  string nameDB() const;

  bool isSubcategory(const string) const;
  bool hasParent(const string) const;
  bool hasParents() const;
  bool hasChildren() const;
  bool isIndex() const;

  bool isMandatory() const;
  bool isOptional() const;

  virtual void dbCheck(const Strings&); // against columns present
  bool dbAbsent() const;
  virtual void setDbAbsent();

  virtual string moreInfo() const;

  // vector<string> subcategories();

  virtual db::Columns dbColumns() const;
  virtual string DB2(const short tab);
  virtual string DB2member(const string prefix);

  void setSqlType(const int);
  void setSqlSize(const int);


 protected:
  // Is there a way to make a reference static?
  // Then, how do we initialize it for the whole Item class?
  static CifBuilder* _cb; // CifBuilder's own Get methods are not const,
  // hence no const above

  Category *_theCategory;

  string _fullName;
  string _category;
  string _name;
  string _type;

  bool _mandatory;

  string _optFlagName;
  int    _optFlagNumber;
  
  vector<string> _subcategories;
  vector<string> _parents;
  vector<string> _children;

  string _nameIDL;
  string _nameDB;
  bool   _dbAbsent;

  int _typeIDL; // _typeIDL

  int _typeSQL; // _typeSQL
  int _sizeSQL; // _sizeSQL

};


class ItemIndex : public Item
{
 public:
  ItemIndex(Item *ip, const string name, const string originalName);
  virtual string nameMember() const; // or call it nameCIF
  virtual string IDL();
  // virtual string DB2(const short t);
  virtual string DB2member(const string prefix);
  
 private:
  string _originalName;
  string _originalType;
  int    _key;
};


class ItemAggregate : public Item
{
 public:
  ItemAggregate(Category *, 
	    const string category,
	    const string name,
	    const string type);


  virtual ~ItemAggregate();

  void addChild(Item *);
  void checkOptional();

  virtual void dbCheck(const Strings&); // against columns present
  virtual void setDbAbsent();

  virtual string IDL();
  virtual void numberFlags(int&, const bool split);
  virtual string optFlags(const short tab);
  virtual void setFlags(Flags& flags, const string& id);

  virtual string moreInfo() const;

  virtual db::Columns dbColumns() const;
 protected:
  Items  _subitems;
  
  bool   _optFlagsSplit;
};


class ItemBlock : public ItemAggregate
{
 public:
  ItemBlock(Category *, 
	    const string category,
	    const string name,
	    const string type);

  virtual ~ItemBlock();

  ccif::BlockItems& getMembers();
  void setMembers(ccif::BlockItems);

  bool isCartesian() const;

  virtual void numberFlags(int&, const bool split);

  virtual string moreInfo() const;

  virtual string DB2(const short tab);

  void linkMember(Item *);

  void printMembers() const;
  
 private:
  ccif::BlockItems _members;
};


class ItemTensor : public ItemAggregate
{
 public:
  ItemTensor(Category *, 
	     const string category,
	     const string name,
	     const string type,
	     const bool row_major,
	     const string kind);

  virtual ~ItemTensor();

  ccif::TensorItems& getMembers();
  void setMembers(ccif::TensorItems);

  bool row_major() const;
  string kind() const;

  virtual void numberFlags(int&, const bool split);

  virtual string moreInfo() const;

  virtual string DB2(const short tab);

  void linkMember(Item *);

 private:
  ccif::TensorItems _members;
  bool _row_major;
  string _kind;
};

#endif
