// SchemaCif body
// $Id: SchemaCif.C,v 1.3 2002/08/17 03:46:15 alexy Exp $

#include "SchemaCif.h"
#include <iostream>
#include <algorithm>
#include "util.h"
#include <assert.h>

namespace scif {

  SchemaCif::SchemaCif()
  {}


  SchemaCif::SchemaCif(const char *cifName, const char *dataBlock)
  {
    CifFileObj * cifObj = new CifFileObj("", WRITE_MODE, 80, "?", 0);
    char * diags = NULL;

    cifObj->Read((char*)cifName, diags); 
    // assert(!diags); -- schema CIF is a work in progress

    tables.resize(numberTables);

    tables[abbrevTable]   = cifObj->GetTable((char*)dataBlock,     tableNames[abbrevTable]);
    assert(tables[abbrevTable]);
  }


  SchemaCif::~SchemaCif()
  {
    // Apparently the tables are within cifObj, breaks:
    // for (int i = 0; i < tables.size(); ++i)
    //   delete tables[i];

    delete cifObj;
  }


  string SchemaCif::abbrev(const string category, const string item)
  {
    ReVarPCifArray<int>      columnList;
    ReVarCifArray<CifString> searchList;
    ReVarPCifArray<int>      *resultList;

    columnList.Add(abbrevTableColumn);
    columnList.Add(abbrevItemColumn);

    searchList.Add(category.c_str());
    searchList.Add(item.c_str());

    tables[abbrevTable]->SetFlags(ISTable::DT_STRING | ISTable::CASE_INSENSE, abbrevTableColumn);
    tables[abbrevTable]->SetFlags(ISTable::DT_STRING | ISTable::CASE_INSENSE, abbrevItemColumn);
    
    string r = "";

    int errCode;
    resultList = tables[abbrevTable]->Search(searchList, columnList, errCode);
    if (errCode || !resultList->Length()) return r;

    CifString s;

    tables[abbrevTable]->GetCell(s, abbrevColumn, (*resultList)[0]);
    r = string(s.Text());

    delete resultList;

    // cerr << "abbrev: " << r;
    return r;
  }


  bool SchemaCif::has_abbrev(const string category, const string item)
  {
    string r = abbrev(category, item);
    return r != "";
  }

} // namespace scif
