// Corba CIF translation body
// $Id: Trans.C,v 1.11 2003/05/04 17:53:06 alexy Exp $

#include "Trans.h"

#include "options.h"
#include "global.h"
#include "Item.h"
#include "IDL.h"
#include "util.h"
#include "Flags.h"
#include "output.h"


Trans::Trans(const char *cifdb, const char *context)
{
  cerr << "Reading binary CIF dictionary " 
       << opt::cifDB << ", context " << opt::cifDBContext << endl;

  _cb = new CifBuilder(opt::cifDB);

  _cb->BuildDictionaryList();
  _cb->SetContext(opt::cifDBContext);

  Category::setCifBuilder(_cb);
  Item::setCifBuilder(_cb);
  
  uWord itemsNum;

  char ** items = _cb->GetDictionaryItems(itemsNum);

  cerr << "we got " << itemsNum << " items" << endl;
  
  for (int i = 0; i < itemsNum; i++) {

    Item *ip = new Item(items[i]);

    const string& catName = ip->category();
    if (_categories.find(catName) == _categories.end())
      _categories[catName] = Category(catName);

    Category& cat = _categories[catName];
    ip->setCategory(&cat);
    cat.addItem(ip);

    free(items[i]);
  }

  if (items) free(items);
}


void      Trans::setCategory(const string name, Category& category)
{
  _categories[name] = category;
}


Category& Trans::getCategory(const string name)
{
  return _categories[name];
}


bool      Trans::hasCategory(const string name)
{
  return _categories.find(name) != _categories.end();
}


void      Trans::eraseCategory(const string name)
{
  _categories.erase(name);
}


long      Trans::numCategories() const
{
  return _categories.size();
}


vector<string> Trans::categoryNames() const
{
  vector<string> v;

  for (Categories::const_iterator
	 it = _categories.begin(); it != _categories.end(); ++it)
    v.push_back(it->first);

  return v;
}


void Trans::transform()
{
  for (Categories::iterator
	 it = _categories.begin(); it != _categories.end(); ++it)
    it->second.transform();

  split();
}


void Trans::numberFlags(int& base, const bool split)
{
  for (Categories::iterator
	 it = _categories.begin(); it != _categories.end(); ++it)
    it->second.numberFlags(base, split);
}


void Trans::numberFlags(int& base, const Strings& catNames, const bool split)
{
  for (Strings::const_iterator
	 it = catNames.begin(); it != catNames.end(); ++it)
    if (!categoryPresent(*it)) continue;
    else _categories[*it].numberFlags(base, /*split=>*/true);
}


void Trans::split()
{
  int errCode;

  ccif::CategorySplits cs = global::ccif->categorySplits();

  for (int i = 0; i < cs.size(); ++i) {
    Category & parent = _categories[cs[i].parent];

    // cerr << "parent: " << cs[i].parent << " child: " << cs[i].child << endl;

    _categories[cs[i].child] = Category(cs[i].child);

    Category & child = _categories[cs[i].child];

    // currently, the child comes from the same database table as the parent:
    child.nameDB(parent.nameDB());

    Strings itemsOld = parent.itemNames();
    Strings itemsNew = global::ccif->categoryItems(cs[i].parent, errCode);
    if (errCode) {
      cerr << "categoryItems() error" << endl;
      exit(1);
    }


    for (int j = 0; j < itemsOld.size(); ++j) {

      const string& name = itemsOld[j];

      if (util::present(itemsNew, name)) continue;

      // cerr << "moving " << name << endl;

      // move this child to the new category

      Item* ip = parent.getItem(name);

      child.addItem(ip);
      parent.deleteItem(name);
    }
  }
}

string Trans::IDL(const Strings& catNames)
{
  string r;
  const short t = 1; // indentation, # tabs, respective to prolog

  r += IDL::prolog();				  // could take t

  int flagNumber = 0;
  
  for (int i = 0; i < catNames.size(); ++i) {
    if (!categoryPresent(catNames[i])) {
      cerr << "Trans::IDL: category " << catNames[i] << " not found" << endl;
      continue;
    }
    r += _categories[catNames[i]].IDL(t);
    r += '\n';
  }

  r += IDL::entryHead();			  // could take t

  // somehow this should not be done twice in setFlags()!
  int n = 0;
  numberFlags(n, catNames, /*split=>*/true);
  
  for (Strings::const_iterator
	 it = catNames.begin(); it != catNames.end(); ++it)
    if (!categoryPresent(*it)) continue;
    else r += _categories[*it].optFlags(t+1);

  r += IDL::tab(t+1) + IDL::maxFlag(n-1) + ";\n\n"; 

  for (Strings::const_iterator
	 it = catNames.begin(); it != catNames.end(); ++it)
    if (!categoryPresent(*it)) continue;
    else r += _categories[*it].entryCalls(t+1);

  r += IDL::epilog();				  // could take t

  return r;
}


bool Trans::categoryPresent(const string& name) const
{
  return _categories.find(name) != _categories.end();
}


void Trans::setFlags(const Strings& catNames, const string& id)
{

  _id = id;
  
  // somehow this should not be done twice in IDL()!
  int n = 0;
  numberFlags(n, catNames, /*split=>*/true);
  _flags.resize(n);
  
  for (Strings::const_iterator
	 it = catNames.begin(); it != catNames.end(); ++it)
    if (!categoryPresent(*it)) continue;
    else _categories[*it].setFlags(_flags, id);
}


void Trans::showFlags()
{
  _flags.show();
}


void Trans::writeFlags()
{
  global::db->writeFlags(_flags.vBuffer(), _id);
}


// include from the same place here and in options.C
#define BRAVER_OUT_DIR "/home/alexy/cs/out/"

const char *prolog_client = BRAVER_OUT_DIR "client_prologue.C";
const char *epilog_client = BRAVER_OUT_DIR "client_epilogue.C";

void Trans::clientPrologue(const short n)
{
  out::client << util::readFile(prolog_client);
}


void Trans::clientEpilogue(const short n)
{
  out::client << util::readFile(epilog_client);
}
