#include <time.h>
#include <iostream.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <malloc.h>
#include "CifBuilder.h"

#include <vector>
#include <map>
#include <string>
#include <algorithm>

#include "options.h"
#include "global.h"
#include "Category.h"
#include "IDL.h"
#include "Trans.h"
#include "output.h"
#include "db.h" // db::DBException
#include "CategoryGroup.h" // MMS_CLASS, XRAY_CLASS...
#include <assert.h>

void doBitFlags(Trans& tr, const Strings& catNames, const string& id);

int main(int argc, char ** argv) {

  bool useDB;
  
  try {
  opt::parse(argc, argv);

  opt::all |= (opt::mms || opt::xray || opt::full);

  useDB = opt::db2 || opt::bitflags;
  
  assert(opt::cifDB);
  assert(opt::cifDBContext);
  assert(opt::corbaCifFile);
  assert(opt::corbaDataBlock);

  global::create_ccif();
  global::create_scif();
  if (useDB) global::db_open();
  if (opt::headers) out::h.open(opt::headers);
  if (opt::client)  out::client.open(opt::clientFile);

  Trans tr(opt::cifDB, opt::cifDBContext);

  cerr << "We got " << tr.numCategories() << " categories" << endl;

  if (!opt::keep_types) tr.transform(); 
  
  vector<string> catNames;

  if (opt::all) {
    vector<string> allCatNames = tr.categoryNames();

    for (int i = 0; i < allCatNames.size(); ++i)
      if (opt::mms
	  // && global::ccif->categoryClass(allCatNames[i]) != ccif::MMS_CLASS
	  && tr.getCategory(allCatNames[i]).groupIDL() != IDL::MMS_CLASS)
	continue;
      else if (opt::xray
	       // && global::ccif->categoryClass(allCatNames[i]) != ccif::XRAY_CLASS
	       && tr.getCategory(allCatNames[i]).groupIDL() != IDL::XRAY_CLASS)
	continue;
      else
	catNames.push_back(allCatNames[i]);
    
  }
  else if (opt::category)
    catNames.push_back(opt::category);
  else {    
//      catNames.push_back("chem_comp_angle");
//      catNames.push_back("struct_biol");
//      catNames.push_back("struct_biol_view");
      catNames.push_back("struct_ncs_oper");
//      catNames.push_back("chem_comp_atom");
//      catNames.push_back("atom_site_anisotrop");
//      catNames.push_back("struct_conf");
//      catNames.push_back("geom_angle");
//      catNames.push_back("atom_site");
//      catNames.push_back("atom_site_ext");
      catNames.push_back("entity");  
  }
 
  int n = opt::n;

  if (opt::client) tr.clientPrologue();

  if (opt::full) 
    cout << tr.IDL(catNames);
  else if (opt::bitflags) {
    string id;
    if (opt::id) {
      id = opt::id; // string(opt::id); // or have opt::id a string already?
      doBitFlags(tr, catNames, id);
    }
    else if (opt::file) {
      ifstream in(opt::file);
      while (!in.eof()) {
	in >> id;
	// should I put it in a try block to in.close()?
	// but then we need to re-raise to db_close()...
	doBitFlags(tr, catNames, id); 
      }
      in.close();
    }
  } else {
    for (int i = 0; i < catNames.size(); ++i) {

      string& categoryName = catNames[i];
      
      Category & category = tr.getCategory(categoryName);
      
      //      cout << "The category " << categoryName
      //  	 << " has " << category.nItems() 
      //  	 << " items..." << endl << endl;
      
      if (opt::idl)   cout << category.IDL() << endl;
      if (opt::flags) {
	category.numberFlags(n, /*split=>*/true);
	cout << category.optFlags(0) << endl;
      }
      if (opt::entry) cout << category.entryCalls() << endl;
      if (opt::db2)   cout << category.DB2() << endl;
      if (opt::client) out::client << category.client(3) << endl;
    } // for some
  } // full

  
  if (opt::flags) cout << IDL::maxFlag(n-1) << ';' << endl;

  if (opt::client) tr.clientEpilogue();

  global::shutdown();
  
  exit(0);
  
  }
  catch (db::DBException &e) {
    cerr << "DBException: " << e.method_name << ", " << e.description << endl;
    global::shutdown();
  }
  catch (...) {
    cerr << "unexpected exception" << endl;
    global::shutdown();
  }
}


void doBitFlags(Trans& tr, const Strings& catNames, const string& id)
{
  cerr << "Showing the binary flags for entry " << id << endl;
  tr.setFlags(catNames, id);
  tr.showFlags();
  if (opt::db2) tr.writeFlags();
}
