// db2 connection body
// $Id: db.C,v 1.10 2003/05/04 17:53:06 alexy Exp $

#include "db.h"
#include "util.h"
#include <iostream>
#include "IDL.h"
#include "bad.h"

#define NEWHANDLE

const string db::DB::schema = "DB2INST1";


db::DBException::DBException(const string method_name, const string description)
{
  this->method_name = method_name;
  this->description = description;
}


db::DB::DB()
{
  // these things should go into options
  // for now get them from bad.h:
  
  strcpy(dbAlias,	DB_NAME);
  strcpy(user, 		DB_USER);
  strcpy(pswd,		DB_PSWD);
  
  /* initialize the CLI application */
  sqlrc = SQLAllocHandle( SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::DB()", "SQLAllocHandle henv");

  sqlrc = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc);
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::DB()", "SQLAllocHandle hdbc");

  cerr << "Connecting to " << dbAlias << "... ";
  sqlrc = SQLConnect( hdbc,
		      (SQLCHAR *)dbAlias, SQL_NTS,
		      (SQLCHAR *)user, SQL_NTS,
		      (SQLCHAR *)pswd, SQL_NTS
		      ) ;
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::DB()", "SQLConnect");
  
  cerr << "done." << endl;

#ifndef NEWHANDLE
  sqlrc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::DB()", "SQLAllocHandle hstmt");
#endif

}


db::DB::~DB()
{
#ifndef NEWHANDLE
  sqlrc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt) ;
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::~DB()", "SQLFreeHandle hstmt");
#endif
  
  cerr << "Disconnecting from " << dbAlias << "... ";
  sqlrc = SQLDisconnect(hdbc) ;
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::~DB()", "SQLDisconnect");

  cerr << "done." << endl;

  sqlrc = SQLFreeHandle( SQL_HANDLE_DBC, hdbc) ;
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::~DB()", "SQLFreeHandle hdbc");

  sqlrc = SQLFreeHandle( SQL_HANDLE_ENV, henv) ;
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::~DB()", "SQLFreeHandle henv");
}


string db::DB::cType(const int sqlType)
{
  switch (sqlType) {
  case _SQL_UNKNOWN_TYPE: return "UNKNOWN";
  case _SQL_INTEGER:      return "SQLINTEGER";
  case _SQL_SMALLINT:     return "SQLSMALLINT";
  case _SQL_FLOAT:
  case _SQL_REAL:
  case _SQL_NUMERIC:
  case _SQL_DECIMAL:
  case _SQL_DOUBLE:       return "SQLDOUBLE";
  case _SQL_CLOB:
  case _SQL_CHAR:
  case _SQL_VARCHAR:      return "SQLCHAR";
  default:                return "UNIMPLEMENTED_" + util::str(sqlType); 
  }
}


string db::DB::bType(int sqlType)
{
  switch (sqlType) {
    
  case _SQL_UNKNOWN_TYPE: return "UNKNOWN";
  case _SQL_INTEGER:      return "SQL_C_LONG";
  case _SQL_SMALLINT:     return "SQL_C_SHORT";
  case _SQL_FLOAT:
  case _SQL_REAL:
  case _SQL_NUMERIC:
  case _SQL_DECIMAL:
  case _SQL_DOUBLE:       return "SQL_C_DOUBLE";
  case _SQL_CLOB:
  case _SQL_CHAR:
  case _SQL_VARCHAR:      return "SQL_C_CHAR";
  default:                return "UNIMPLEMENTED_" + util::str(sqlType); 
  }
}


int db::DB::bSize(int sqlType, int size)
{
  switch (sqlType) {
  case _SQL_CLOB:
  case _SQL_CHAR:
  case _SQL_VARCHAR: return size;
  default:           return 0;
  }
}


string db::DB::corbaCast(const string name, const int typeIDL, const int sqlType)
{
  string r = name + ".val";

  if (typeIDL == IDL::STRING) {
    switch (sqlType) {
    case _SQL_INTEGER: return "itoa(" + r + ')';
    case _SQL_CHAR:
    case _SQL_VARCHAR: return "CORBA::string_dup((char*)" + r + ')';
    default:           return r;
    }
  }
  else if (typeIDL == IDL::LONG) {
    switch (sqlType) {
    case _SQL_CHAR:
    case _SQL_VARCHAR: return "atoi((char*)" + r + ')';
    default:           return r;
    }
  }
  else
    return r;
} 


bool db::DB::tablePresent(const string& table)
{

  SQLRETURN   result;

#ifdef NEWHANDLE
  sqlrc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::DB::tablePresent()", "SQLAllocHandle hstmt");
#endif

  string tableUpper = util::upper(table);
  sqlrc = SQLTables(hstmt, NULL, 0,
		    (SQLCHAR*) schema.c_str(), SQL_NTS,
		    (SQLCHAR*) tableUpper.c_str(),   SQL_NTS,
		    NULL, 0);
  if (sqlrc != SQL_SUCCESS) {
    cerr << "db::DB::tablePresent(): sqlrc = " << sqlrc << ", table = " << table << ", schema = " << schema << endl;
    throw DBException("db::DB::tablePresent()", "SQLTables");
  }
  
  result = SQLFetch( hstmt);
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::DB::tablePresent()", "SQLFetch");


#ifdef NEWHANDLE
  sqlrc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt) ;
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::DB::tablePresent()", "SQLFreeHandle");
#else
  //  sqlrc = SQLCloseCursor(hstmt);
  sqlrc = SQLFreeStmt(hstmt, SQL_CLOSE);
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::DB::tablePresent()", "SQLFreeStmt");
#endif

  
  return result != SQL_NO_DATA_FOUND;
}


vector<string> db::DB::columns(const string& table)
{
    vector<string> vs;

#ifdef NEWHANDLE
  sqlrc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::DB::columns()", "SQLAllocHandle hstmt");
#endif

  SQLCHAR colNamePattern[] = "%";  
  
  string tableUpper = util::upper(table);
  sqlrc = SQLColumns( hstmt, NULL, 0,
		      (SQLCHAR*) schema.c_str(), SQL_NTS,
		      (SQLCHAR*) tableUpper.c_str(),   SQL_NTS,
		      colNamePattern, SQL_NTS); 
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::DB::columns()", "SQLColumns");
    
  struct
    {   SQLINTEGER ind ;
        SQLCHAR    val[129] ;
    } colName, colType;   

  // we don't really need other variables than the name,
  // neither in structs nor do we reaqlly need to bind them...
    
    struct 
    {   SQLINTEGER ind ;
        SQLCHAR val[255] ;
    } colRemarks;         

    struct 
    {   SQLINTEGER ind ;
        SQLINTEGER val; 
    } colLength;             
    
    struct 
    {   SQLINTEGER ind ;
        SQLSMALLINT val; 
    } colScale, colNullable;       
    
  
    // bind columns to variables

    sqlrc = SQLBindCol( hstmt, 4, SQL_C_CHAR, colName.val, 129,
                        &colName.ind ) ;
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::DB::columns()", "SQLBindCol 4");

    sqlrc = SQLBindCol( hstmt, 6, SQL_C_CHAR, colType.val, 129,
                        &colType.ind ) ;
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::DB::columns()", "SQLBindCol 6");

    sqlrc = SQLBindCol( hstmt, 7, SQL_C_LONG, (SQLPOINTER)&colLength.val,
                        sizeof(colLength.val), &colLength.ind ) ;
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::DB::columns()", "SQLBindCol 7");

    sqlrc = SQLBindCol( hstmt, 9, SQL_C_SHORT, (SQLPOINTER)&colScale.val,
                        sizeof(colScale.val), &colScale.ind ) ;
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::DB::columns()", "SQLBindCol 9");

    sqlrc = SQLBindCol( hstmt, 11, SQL_C_SHORT, (SQLPOINTER)&colNullable.val,
                        sizeof(colNullable.val), &colNullable.ind ) ;
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::DB::columns()", "SQLBindCol 11");

    sqlrc = SQLBindCol( hstmt, 12, SQL_C_CHAR, colRemarks.val, 255, 
                        &colRemarks.ind ) ;
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::DB::columns()", "SQLBindCol 12");


    // fetch each row
    sqlrc = SQLFetch( hstmt );
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::DB::columns()", "SQLFetch first");

    while( sqlrc != SQL_NO_DATA_FOUND ) {
      vs.push_back(string((char*)colName.val));
      
      sqlrc = SQLFetch( hstmt );
    if (sqlrc != SQL_SUCCESS
	// && sqlrc != SQL_SUCCESS_WITH_INFO
	&& sqlrc != SQL_NO_DATA_FOUND)
      throw DBException("db::DB::columns()", "SQLFetch loop");

    } // endwhile

#ifdef NEWHANDLE
  sqlrc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt) ;
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::DB::columns()", "SQLFreeHandle");
#else
  //  sqlrc = SQLCloseCursor(hstmt);
  sqlrc = SQLFreeStmt(hstmt, SQL_CLOSE);
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::DB::columns()", "SQLFreeStmt");
#endif
  
    return vs;
}


bool db::DB::categoryPresent(const string& name, const string& id)
{
  string item = "*";
  return itemPresent(name, item, id);
}


bool db::DB::itemPresent(const string& category, const string& item, const string& id)
{

    string sstmt = "SELECT count(" + item + ") FROM "
      + schema + '.' + category + " WHERE structure_id = '" + id + '\'';
    SQLCHAR * stmt = ( SQLCHAR * ) sstmt.c_str();
    // cerr << "SQL statement: " << stmt << endl;

#ifdef NEWHANDLE
    sqlrc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::itemPresent()", "SQLAllocHandle");
#endif

    sqlrc = SQLExecDirect(hstmt, stmt, SQL_NTS);
    if (sqlrc != SQL_SUCCESS)
        throw DBException("db::itemPresent()", "SQLExecDirect: " + sstmt);

    struct {
        SQLINTEGER ind;
        SQLINTEGER val;
    } total;

    sqlrc = SQLBindCol(hstmt, 1, SQL_C_LONG, &total.val, 0, &total.ind);
    if (sqlrc != SQL_SUCCESS)
        throw DBException("db::itemPresent()", "SQLBindCol");

    sqlrc = SQLFetch(hstmt);

    if (sqlrc != SQL_SUCCESS
	&& sqlrc != SQL_SUCCESS_WITH_INFO
	&& sqlrc != SQL_NO_DATA)
        throw DBException("db::itemPresent()", "SQLFetch: " + sstmt);

    if (sqlrc == SQL_NO_DATA_FOUND)
        throw DBException("db::itemPresent()", "no data: " + sstmt);

#ifdef NEWHANDLE
    sqlrc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt) ;
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::itemPresent()", "SQLFreeHandle");
#else
    sqlrc = SQLFreeStmt(hstmt, SQL_CLOSE);
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::itemPresent()", "SQLFreeStmt");
#endif
     
    return total.val;
}


db::ColumnInfo::ColumnInfo() : bufferLength(COLUMN_NAME_SIZE) {}
db::ColumnInfo::~ColumnInfo() {}


void db::DB::prepareSQL(const string& stmt)
{
#ifdef NEWHANDLE
    sqlrc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::prepareSQL()", "SQLAllocHandle");
#endif

  sqlrc = SQLPrepare(hstmt, (SQLCHAR*)stmt.c_str(), SQL_NTS);
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::prepareSQL()", "SQLPrepare " + stmt);
}


void db::DB::finishSQL()
{
#ifdef NEWHANDLE
    sqlrc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt) ;
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::finishSQL()", "SQLFreeHandle");
#else
    sqlrc = SQLFreeStmt(hstmt, SQL_CLOSE);
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::finishSQL()", "SQLFreeStmt");
#endif
}


void db::DB::describeColumn(const unsigned num, ColumnInfo& ci)
{
  // hstmt must be prepared by a call to prepareSQL
  
  sqlrc = SQLDescribeCol(hstmt,
			 num,     
			 ci.columnName,       
			 ci.bufferLength,     
			 &ci.nameLength,   
			 &ci.dataType,     
			 &ci.columnSize,   
			 &ci.decimalDigits,
			 &ci.nullable);
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::describeColumn()", "SQLDescribeCol " + util::str(num));
}


void db::DB::writeFlags(const vector<unsigned char>& vBuffer, const string &id)
{
  string sstmt;
  SQLCHAR * stmt;
  
#ifdef NEWHANDLE
  sqlrc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::writeFlags()", "SQLAllocHandle");
#endif

  sstmt = "DELETE FROM " + schema + ".FLAGS WHERE structure_id = '" + id + '\'';
  stmt = (SQLCHAR *)sstmt.c_str();

  sqlrc = SQLExecDirect(hstmt, stmt, SQL_NTS);
  if (sqlrc != SQL_SUCCESS && sqlrc != SQL_NO_DATA)
    cerr << id << " is created fir the first time (" << sqlrc << ')' << endl;
      // throw DBException("db::itemPresent()", "SQLExecDirect: sqlrc = " + util::str(sqlrc) + ": " + sstmt);
  
  sstmt = "INSERT INTO " + schema + ".FLAGS VALUES ('" + id + "', ?)";
  stmt = (SQLCHAR *)sstmt.c_str();

  SQLCHAR         inputParam[] = "Flags Data";    
  SQLINTEGER      blobInd ;
  SQLPOINTER      valuePtr;    
  sqlrc = SQLPrepare( hstmt, stmt, SQL_NTS ) ;
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::writeFlags()", "SQLPrepare: " + sstmt);

  /*
   * This paramter will use SQLPutData
   */
  blobInd = SQL_DATA_AT_EXEC;
  sqlrc = SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_BINARY, SQL_BLOB,
			   vBuffer.size(), 0, (SQLPOINTER)inputParam,
			   vBuffer.size(), &blobInd);  

  sqlrc = SQLExecute(hstmt);
  if (sqlrc != SQL_NEED_DATA)
    throw DBException("db::writeFlags()", "SQLExecute needs no data: " + sstmt);
  
  sqlrc = SQLParamData( hstmt, (SQLPOINTER *) &valuePtr);
  if (sqlrc != SQL_NEED_DATA)
    throw DBException("db::writeFlags()", "SQLParamData 1: " + sstmt);

    // if more than 1 parms used DATA_AT_EXEC then valuePtr would
    // have to be checked to determine which param needed data
      
  sqlrc = SQLPutData(hstmt,(unsigned char*) &vBuffer[0], vBuffer.size());
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::writeFlags()", "SQLPutData: " + sstmt);
  
  cout << "BLOB: Wrote " << vBuffer.size() << " bytes for " << id << endl;
  
  // need this to avoid Function sequence error:
  sqlrc = SQLParamData( hstmt, (SQLPOINTER *) &valuePtr);
  if (sqlrc != SQL_SUCCESS) {
    cerr << "sqlrc = " << sqlrc << endl;
    throw DBException("db::writeFlags()", "SQLParamData 2: " + sstmt);
  }
  
#ifdef NEWHANDLE
  sqlrc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt) ;
  if (sqlrc != SQL_SUCCESS) {
    cerr << "sqlrc = " << sqlrc << endl;    
    throw DBException("db::writeFlags()", "SQLFreeHandle");
  }
#else
  sqlrc = SQLFreeStmt(hstmt, SQL_CLOSE);
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::writeFlags()", "SQLFreeStmt");
#endif
}


void db::DB::readFlags(vector<unsigned char>& vBuffer, const string &id)
{
  string sstmt = "SELECT flags FROM " + schema + ".FLAGS WHERE structure_id = \'" + id +'\'';
  SQLCHAR * stmt = (SQLCHAR *)sstmt.c_str();

  // can be read from the database as BLOB(size)
  // const unsigned long BUFSIZ = 1000; // fine in g++ 2.95.3, error in 3.3
  const unsigned long bufSize = 1000;
    
  SQLCHAR         buffer[bufSize];     
  SQLINTEGER      bufInd;

#ifdef NEWHANDLE
  sqlrc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::readFlags()", "SQLAllocHandle");
#endif

  sqlrc = SQLPrepare( hstmt, stmt, SQL_NTS ) ;
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::readFlags()", "SQLPrepare: " + sstmt);

  sqlrc = SQLExecute(hstmt);
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::readFlags()", "SQLExecute: " + sstmt);
  
  sqlrc = SQLFetch(hstmt);
  if (sqlrc == SQL_NO_DATA)
    throw DBException("db::readFlags()", "SQLFetch no data: " + sstmt);
  else if (sqlrc != SQL_SUCCESS)
    throw DBException("db::readFlags()", "SQLFetch");
  
  sqlrc = SQLGetData(hstmt, 1, SQL_C_BINARY,
		     (SQLPOINTER) buffer, bufSize, &bufInd);
  if (sqlrc == SQL_NO_DATA)
    throw DBException("db::readFlags()", "SQLFetch no data: " + sstmt);
  else if (sqlrc != SQL_SUCCESS)
    throw DBException("db::readFlags()", "SQLGetData");

  vBuffer.resize(bufInd);
  memcpy(&vBuffer[0], buffer, bufInd);

#ifdef NEWHANDLE
  sqlrc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt) ;
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::readFlags()", "SQLFreeHandle");
#else
  sqlrc = SQLFreeStmt(hstmt, SQL_CLOSE);
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::readFlags()", "SQLFreeStmt");
#endif
}
