// Simple DB2 connection for C++ code generation
// $Id: db.h,v 1.6 2002/07/24 22:56:35 alexy Exp $

#ifndef _DB_H_
#define _DB_H_

#include <sqlcli1.h>

#include <string>
#include <vector>
using namespace std;

namespace db {
  
  struct DBException
  {
    DBException(const string method_name, const string description);
    
    string method_name;
    string description;
  };

  
  enum {
    _SQL_CLOB		=SQL_CLOB,		//  -99
    _SQL_UNKNOWN_TYPE	=SQL_UNKNOWN_TYPE,	//    0
    _SQL_CHAR		=SQL_CHAR,		//    1
    _SQL_NUMERIC	=SQL_NUMERIC,		//    2
    _SQL_DECIMAL	=SQL_DECIMAL,		//    3
    _SQL_INTEGER	=SQL_INTEGER,		//    4
    _SQL_SMALLINT	=SQL_SMALLINT,		//    5
    _SQL_FLOAT		=SQL_FLOAT,		//    6
    _SQL_REAL		=SQL_REAL,		//    7
    _SQL_DOUBLE		=SQL_DOUBLE,		//    8
    _SQL_DATETIME	=SQL_DATETIME,		//    9
    _SQL_VARCHAR	=SQL_VARCHAR,		//   12
    _SQL_TYPE_DATE	=SQL_TYPE_DATE,		//   91
    _SQL_TYPE_TIME	=SQL_TYPE_TIME,		//   92
    _SQL_TYPE_TIMESTAMP	=SQL_TYPE_TIMESTAMP	//   93
  };


  const short COLUMN_NAME_SIZE = 255;

  struct ColumnInfo {
    ColumnInfo();
    virtual ~ColumnInfo();
    
    SQLCHAR           columnName[COLUMN_NAME_SIZE];
    SQLSMALLINT       bufferLength;
    SQLSMALLINT       nameLength;
    SQLSMALLINT       dataType;
    SQLUINTEGER       columnSize;
    SQLSMALLINT       decimalDigits;
    SQLSMALLINT       nullable;
  };
  
  
  class DB {
  public:
    DB();
    virtual ~DB();

    static const string schema;

    string cType(int typeSQL);
    string bType(int typeSQL);
    int    bSize(int typeSQL, int size);

    string corbaCast(const string name, const int typeIDL, const int typeSQL);

    bool   tablePresent(const string& table);     // in the schema
    vector<string> columns(const string& table); // -"-

    void prepareSQL(const string&);
    void finishSQL();
    void describeColumn(const unsigned num, ColumnInfo&);
    
    bool categoryPresent(const string& name, const string& id);
    bool itemPresent(const string& category, const string& item, const string& id);

    void writeFlags(const vector<unsigned char>& vBuffer, const string &id);
    void readFlags(vector<unsigned char>& vBuffer, const string &id);
    
  private:
    SQLRETURN   sqlrc;
    SQLHANDLE   henv;  // environment handle
    SQLHANDLE   hdbc;  // connection handle
    SQLHANDLE   hstmt; // statement handle
    
    // can come from opt::
#define MAX_UID_LENGTH 18
#define MAX_PWD_LENGTH 30
    char       dbAlias [SQL_MAX_DSN_LENGTH + 1] ;
    char       user[MAX_UID_LENGTH + 1] ;
    char       pswd[MAX_PWD_LENGTH + 1] ;

  }; // class DB

}

#endif
