// Relevance Feedback
// Options

const char *vc_rf_options =
"$Name:  $ $RCSfile: options.C,v $ $Revision: 1.13 $";

#include "options.h"

#include <Options.h> // Brad Appleton's
#include <iostream>
#include <cstdlib> // -- needed for exit(1)?!
#include <cstring> // atoi
using namespace std;

namespace opt {

static const char * optv[] = {
  "?|help",
  "B|client",
  "C|children",
  "D|cifdb",
  "F|bitflags",
  "I|idl",
  "M|mms",
  "N|corbacif",
  "O|schemacif",
  "P|parents",
  "S|cifdbdata",
  "X|xray",
  "2|db2",
  "a|all",
  "c:category",
  "d|debug",
  "e|entry",
  "f|flags",
  "h:headers",
  "i:id",
  "k|keep_types",
  "m|mandatory",
  "n:number",
  "o|catonly",
  "s|split_flags",
  "u|full",
  "v|valuetypes",
  NULL
} ;
  
  bool debug    = false;

  
  // #define PDBX_DIC
  char *cifDB   = 0;
  const char cifDB_default[] =
#ifndef PDBX_DIC 
  ODB_DIR "cif_mm"		ODB_EXT
#else
  ODB_DIR "mmcif_pdbx_2"	ODB_EXT
#endif
  ;
  char *cifDBContext = 0;

  const char cifDBContext_default[] = 
#ifndef PDBX_DIC 
  "cif_mm.dic"
#else
  "cif_pdbx.dic"
#endif
  ;
  char *category = 0;
  char *id       = 0;
  char *file     = 0;
  char *headers  = 0;

  bool parents = false;
  bool children = false;
  bool mandatory = false;
  bool more_info;

  bool keep_types = false;
  bool all = false;
  bool idl = false;
  bool flags = 0;
  int  n = 0;
  bool bitflags = false;
  bool split_flags = false;
  bool catonly = false;
  bool mms = false;
  bool xray = false;
  bool entry = false;
  bool full = false;
  bool valuetypes = true;
  bool db2 = false;
  bool client = false;

  char  *corbaCifFile = 0;
  const char corbaCifFile_default[] = CIF_DIR "corba.cif";
  char  *corbaDataBlock = 0;
  const char corbaDataBlock_default[] = "cif_corba_mapping";

  char  *schemaCifFile = 0;
  const char schemaCifFile_default[] = CIF_DIR "rcsb_schema.cif";
  char  *schemaDataBlock = 0;
  const char schemaDataBlock_default[] = "rcsb_schema";

  char *clientFile = 0;
  const char clientFile_default[] = OUT_DIR "client.C";

void parse(int argc, char **argv, const bool need_files)
{
  int  optchar;
  const char * optarg;

  int  errors = 0;

  Options  opts(*argv, optv);
  OptArgvIter  iter(--argc, ++argv);

  opts.ctrls(Options::PARSE_POS);
  while( (optchar = opts(iter, optarg)) ) {
    switch (optchar) {
    case '?' :
      opts.usage(cout, (const char*)"files ...");
      exit(0);
      break;
    case 'B':
      client = !client;
      clientFile = (char*) optarg;
    case 'C':
      children = !children;
      break;
    case 'D':
      cifDB = (char*) optarg;
      break;
    case 'F':
      bitflags = !bitflags;
      break;
    case 'I':
      idl = !idl;
      break;
    case 'M':
      mms = !mms;
      break;
    case 'N':
      corbaCifFile = (char*) optarg;
      break;
    case 'O':
      schemaCifFile = (char*) optarg;
      break;
    case 'P' :
      parents = !parents;
      break;
    case 'S':
      cifDBContext = (char*) optarg;
      break;
    case 'X':
      xray = !xray;
      break;
    case '2':
      db2 = !db2;
      break;
    case 'a':
      all = !all;
      break;
    case 'c':
      if (optarg) category = (char*) optarg;
      break;
    case 'd' :
      debug = !debug;
      break;
    case 'e':
      entry = !entry;
      break;
    case 'f':
      flags = !flags;
      break;
    case 'h':
      if (optarg) headers = (char*) optarg;
      else ++errors;
      break;
    case 'i':
      if (optarg) id = (char*) optarg;
      break;
    case 'k':
      keep_types = !keep_types;
      break;
    case 'm':
      mandatory = !mandatory;
      break;
    case 'n':
      if (optarg) n = atoi(optarg);
      break;
    case 'o':
      catonly = !catonly;
      break;
    case 's':
      split_flags = !split_flags;
      break;
    case 'u':
      full = !full;
      break;
    case 'v':
      valuetypes = !valuetypes;
      break;
    case Options::POSITIONAL:
      file = (char*) optarg;
      break;
    default :  ++errors; break;
    } //switch
  }

  if (errors || need_files && (iter.index() == argc)) {
    if (! errors) 
      cerr << opts.name() << ": no filenames given." << endl ;
    opts.usage(cerr, (const char*)"files ...");
    exit(1);
  }

  if (! cifDB)           cifDB           = (char*) cifDB_default;
  if (! cifDBContext)    cifDBContext    = (char*) cifDBContext_default;
  if (! corbaCifFile)    corbaCifFile    = (char*) corbaCifFile_default;
  if (! corbaDataBlock)  corbaDataBlock  = (char*) corbaDataBlock_default;
  if (! schemaCifFile)   schemaCifFile   = (char*) schemaCifFile_default;
  if (! schemaDataBlock) schemaDataBlock = (char*) schemaDataBlock_default;
  if (client && !clientFile) clientFile  = (char*) clientFile_default;
  
  more_info = parents || children || mandatory;
} // parse

} // namespace opt
