/*

COPYRIGHT 1998 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.

*/

/* 
  FILE:         ndb_cifparse.h

  This file is part of the NDBQUERY application,
  a program component of the Nucleic Acid Database Project.

  H. M. Berman, W. K. Olson, D. L. Beveridge, J. K. Westbrook, A. Gelbin,
  T. Demeny, S. H. Shieh, A. R. Srinivasan, and B. Schneider. (1992).
  The Nucleic Acid Database: A Comprehensive Relational Database of 
  Three-Dimensional Structures of Nucleic Acids.  Biophys J., 63, 751-
  759.

  Questions about this software should be directed to:

              ndbadmin@ndbserver.rutgers.edu

  PURPOSE:   Header file for cifparselib.a, the collection of routines 
             provides a simple interface to the DDL 2.1 compliant CIF files.
*/

/*$$LICENSE-NDB$$*/

#ifndef __NDB_CIFPARSE_H__
#define __NDB_CIFPARSE_H__
#if 0
#ifdef __cplusplus
extern "C" {
#endif
#endif

/*
 *                     Prototypes ... 
 */

#include "ndb_misclib.h"

/*
 *   MxNameLen applies to block, category, item and keyword names...
 *   
 */
#define MxNameLen    		 80 
/*
 *   This is the size of the buffer length allocated to individual
 *   values.
 */

#define CifDefaultSpace           2

#undef   YYLMAX
#define  YYLMAX       256

#if 0
#undef   YYMAXDEPTH  
#define  YYMAXDEPTH   10000
#undef   YYINITDEPTH 
#define  YYINITDEPTH   1000
#endif

#define  YYPRINT(file, type, value) yyprint(file, type, value)


#define TRUE                   1
#define FALSE                  0

typedef struct NdbCifRowFormat {
  char **columns;
} NdbCifRowFormat;

typedef struct NdbCifCategoryFormat {
  int numCol;
  int allCol;
  int curCol;
  int allRow;
  int numRow;
  int curRow;
  char categoryName[MxNameLen];
  char **colNames;
  NdbCifRowFormat *rows;
} NdbCifCategoryFormat;

typedef struct NdbCifDatablockFormat {
  int numCategory; /* Number of categories in this datablock */
  int allCategory; /* Allocated space */
  int curCategory; /* index of the current category */
  char datablockName[MxNameLen]; 
  NdbCifCategoryFormat *categories;
} NdbCifDatablockFormat;  

typedef struct NdbCifDatablocksFormat {
  int numDatablock; /* Number of datablocks in this structure */
  int curDatablock; /* index of the current datablock */
  int allDatablock; /* Allocated space */
  NdbCifDatablockFormat *datablocks;
} NdbCifDatablocksFormat;


/**PROTO*******************************************************************
 *       Prototypes for file:  ndb_cifparse_iolib.c
 */
int ndb_cif_read_file(FILE *fp);
int ndb_cif_write_file(FILE *fp);

/**PROTO*******************************************************************
 *       Prototypes for file:  ndb_cifparse_util.c
 */
int ndb_cif_init(void);
int ndb_cif_close(void);
int ndb_cif_new_datablock(const char *datablockName);
int ndb_cif_put_datablock_name(const char *datablockName);
int ndb_cif_new_category(const char *categoryName);
int ndb_cif_new_row(void);
int ndb_cif_insert_new_row(const int rowNo);
int ndb_cif_rewind_file(void);
int ndb_cif_rewind_datablock(void);
int ndb_cif_rewind_category(void);
int ndb_cif_rewind_row(void);
int ndb_cif_rewind_column(void);
int ndb_cif_reset_datablocks(void);
int ndb_cif_reset_datablock(void);
int ndb_cif_reset_datablock_by_id(const int datablockId);
int ndb_cif_reset_category(void);
int ndb_cif_reset_category_by_id(const int datablockId, const int categoryId);
int ndb_cif_remove_datablock(void);
int ndb_cif_remove_datablock_by_id(const int datablockId);
int ndb_cif_remove_datablock_by_name(const char *datablockName);
int ndb_cif_remove_category(void);
int ndb_cif_remove_category_by_id(const int datablockId, const int categoryId);
int ndb_cif_remove_category_by_name(const char *datablockName, const char *categoryName);
int ndb_cif_remove_row(void);
int ndb_cif_remove_row_by_id(const int datablockId, const int categoryId, const int rowId);
int ndb_cif_next_datablock(void);
int ndb_cif_next_category(void);
int ndb_cif_next_row(void);
int ndb_cif_next_column(void);
int ndb_cif_move_datablock(const char *datablockName);
int ndb_cif_move_datablock_by_id(const int datablockId);
int ndb_cif_move_category_by_name(const char *datablockName, const char *categoryName);
int ndb_cif_move_category_by_id(const int datablockId, const int categoryId);
int ndb_cif_move_category(const int categoryId);
int ndb_cif_move_row_by_name(const char *datablockName, const char *categoryName, const int rowId);
int ndb_cif_move_row_by_id(const int datablockId, const int categoryId, const int rowId);
int ndb_cif_move_row(const int rowId);
int ndb_cif_count_datablock(void);
int ndb_cif_count_category(void);
int ndb_cif_count_row(void);
int ndb_cif_count_column(void);
int ndb_cif_current_datablock(void);
int ndb_cif_current_datablock_name(char *datablockName);
int ndb_cif_current_category(void);
int ndb_cif_current_category_name(char *categoryName);
int ndb_cif_current_row(void);
int ndb_cif_current_col(void);
int ndb_cif_put_item_keyword(const char *itemKeyword);
int ndb_cif_remove_item_keyword(const char *itemKeyword);
int ndb_cif_get_item_shname(const int colId, char *itemName);
int ndb_cif_get_item_name(const int colId, char *itemName);
int ndb_cif_output_item(FILE *fp, const int colId);
char *ndb_cif_copy_item_value(const int colId);
int ndb_cif_get_item_value(const int colId, char *fieldValue, const int maxFieldLen);
int ndb_cif_get_item_value_length(const int colId);
int ndb_cif_item_value_strncmp(const int catId, const int colId, 
			       const int rowId, const int startPos,
			       char *fieldValue, const int maxFieldLen);
int ndb_cif_item_values_strncmp(const int catId1, const int colId1, 
				const int rowId1, const int startPos1, 
				const int catId2, const int colId2,  
				const int rowId2,  const int startPos2,
				const int maxFieldLen);
/*
   The following two functions return the row numbers where the given 
   strings are found (and sets the current row to this one), else return 0.
 */
int ndb_cif_item_row_1_key(const int colId, char *fieldValue);
int ndb_cif_item_row_2_keys(const int colId1, const char *fieldValue1,  
				    const int colId2, const char *fieldValue2);
int ndb_cif_item_row_3_keys(const int colId1, const char *fieldValue1, 
			    const int colId2, const char *fieldValue2,
			    const int colId3, const char *fieldValue3);
int ndb_cif_item_row_4_keys(const int colId1, const char *fieldValue1, 
			    const int colId2, const char *fieldValue2,
			    const int colId3, const char *fieldValue3,
			    const int colId4, const char *fieldValue4);
int ndb_cif_put_item_value(const int colId, const char *fieldValue);
int ndb_cif_get_category_name_from_item_name(char *categoryName, const char *itemName);
int ndb_cif_get_item_keyword_from_item_name(char *itemKeyword, const char *itemName);
int ndb_cif_get_datablock_id(const char *datablockName);
int ndb_cif_get_category_id(const char *datablockName, const char *categoryName);
int ndb_cif_get_column_id(const char *datablockName, const char *categoryName,
			  const char *itemKeyword);
void ndb_cif_print_datablock(FILE *fp);
void ndb_cif_pretty_print_datablock(FILE *fp);
void ndb_cif_print_datablocks(FILE *fp);
void ndb_cif_write_category(FILE *fp);
int ndb_cif_write_file_pr(FILE *fp);
void ndb_cif_encapsulate(char **a);
void ndb_cif_de_encapsulate(char ** a);

/**************************************************************************
*   The following functions are only local functions
*/

void ndb_cif_print_item_value(FILE *fp, const char *itemValue, int *linePos);
void ndb_cif_print_item_name(FILE *fp, const char *itemName, int *linePos);
void ndb_cif_print_formatted_item_value(FILE *outfile, int *offset, 
					char *value, int newline_end,
					int fieldLength);
char ndb_cif_set_null_char(char new_null);
int ndb_cif_print_item_value_pr(FILE *fp, const char *itemName, int *linePos);

#if 0
#ifdef __cplusplus
}
#endif
#endif

#endif
