/*$$FILE$$*/
/*$$VERSION$$*/
/*$$DATE$$*/
/*$$LICENSE$$*/
/*
  PURPOSE:      This module contains log file access routines ...
*/

#ifndef __NDB_MISC_UTIL_H__
#define __NDB_MISC_UTIL_H__

#ifdef __cplusplus
extern "C" {
#endif

#include	<sys/types.h>
#include	<fcntl.h>


#define NDB_MSG_ERR   1
#define NDB_MSG_ERROR 1
#define NDB_MSG_WARN  2
#define NDB_MSG_INFO  3
#define NDB_MSG_DEBUG4 4
#define NDB_MSG_DEBUG5 5
#define NDB_MSG_DEBUG6 6
#define NDB_MSG_DEBUG7 7
#define NDB_MSG_DEBUG8 8
#define NDB_MSG_DEBUG9 9
#define NDB_MSG_DEBUG  10

#define NDB_MSG_BUFFER_LEN    100000


#if !defined(HAVE_STRCASECMP)
int strcasecmp (const char *s1, const char *s2);
int strncasecmp (const char *s1, const char *s2, unsigned int n);
#endif 
/*
 *   ndb_misclib.c prototypes
 */

void ndb_set_debug_level(int level);
int  ndb_get_debug_level(void);

void ndb_set_show_stdout(int show);
int  ndb_get_show_stdout(void);

int  ndb_open_log(const char *prog_name, const char *fname);
void ndb_close_log(void);
void ndb_log_message(const int type, char *fmt, ...);  
void ndb_log_message_text(const int type, char *fmt, ...);  
void ndb_print_log_message(const int type, const char *message);
void ndb_print_log_message_text(const int type, const char *message);

/*
 *  ndb_string_util.c prototypes
 */ 
#define NDB_NONSENSE_STRING    "*"
#define NDB_NONSENSE_INTEGER   65530
#define NDB_NONSENSE_DOUBLE    1999999999.0
#define NDB_NONSENSE_ANGLE     "400.0"
#define NDB_APPLICATION        "NDB"

void ndb_set_string_to_upper(char *string);
void ndb_set_string_to_lower(char *string);
void ndb_clean_string(char *string);
void rcsb_clean_string(char *string);
void ndb_clean_string_dash(char *string);
void ndb_align_string(char *string); 
int ndb_strip_leading_blanks(char *string);
int ndb_strip_trailing_blanks(char *string);
void ndb_delete_newline_in_string(char *tmpstring);
int strsim(const char *wordI, const char *patternI, int dLimit, int ignoreCase);

/*
 *  ndb_sysutil.c prototypes
 */
void ndb_execute_command(char *cmdstring);

#define	FILE_MODE	(S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)
int lock_reg(int fd, int cmd, int type, off_t offset, int whence, off_t len);
pid_t lock_test(int fd, int type, off_t offset, int whence, off_t len);


#ifdef __cplusplus
}
#endif

#endif /* __NDB_MISC_UTIL_H__ */
