#include "mms_biol_unit.h"
#include "biol.h"

#include <iostream>
#include "sym_types.h"
#include "mol_symmetry_io.h"
#include "cell_constants.h"

namespace mms_biol_unit {
  using namespace mol_symmetry;

  int ctoi(char c) { return c - '0'; }
  
  MMS::BiolUnit biol_gen(MMS::AsymUnit asym_mol, MMS::StructBiolGenList biol_gens)
  {

    //   cerr << "entered biol_gen" << endl;
    //   cerr << "biol_gen: got asym unit, have " 
    //        << asym_mol.atom_site_list.size() << " atoms" 
    //        << endl;
    //   cerr << "biol_gen: got " 
    //        << biol_ops.size() << " biol-generators" 
    //        << endl;

    int i = 0;
    int j = 0;
    const char* space_name = asym_mol.sym_info.space_group;
    NDBSYMMETRY* sym;
    sym = ndb_get_sym_data(space_name, ' ', 0);
    //  cerr << "past ndb_get_sym_data" << endl;

    NDBSYMTRANSDATA* cur_op;
    NDBCELLCONST cell;

    MMS::BiolUnit biolUnit;

    biol::BiolOps b_op;
    biol::BiolOpsList biol_ops;
    bool op_match = false;

    /* Sort biol-unit symmetry operations so that
       each element in  biol_ops list contains
       one sym-op and corresponding list of chains
       it will be applied to.
    */

    b_op.biol_id = biol_gens[0].biol.index;
    b_op.asym_ids.push_back(string(biol_gens[0].asym.id));
    b_op.symop.name = biol_gens[0].symmetry;
    b_op.symop.op   = ctoi(biol_gens[0].symmetry[0L]);
    b_op.symop.trans_x = ctoi(biol_gens[0].symmetry[2L]);
    b_op.symop.trans_y = ctoi(biol_gens[0].symmetry[3L]);
    b_op.symop.trans_z = ctoi(biol_gens[0].symmetry[4L]);

    biol_ops.push_back(b_op);

    if (biol_gens.length() > 0) {

      for (i=1;i<biol_gens.length();i++) {
           
	op_match = false;

	for (j=0;j<biol_ops.size();j++) {
	
	  if (strcmp(biol_gens[i].symmetry, biol_gens[j].symmetry) == 0) {
	    biol_ops[j].asym_ids.push_back(string(biol_gens[i].asym.id));
	    op_match = true;
	  }
	}

	// cerr <<"Op check  " << op_match << endl;
	if (!op_match) {
	  b_op.asym_ids.clear();
	
	  b_op.biol_id = biol_gens[0].biol.index;
	  b_op.asym_ids.push_back(string(biol_gens[0].asym.id));
	  b_op.symop.name = biol_gens[0].symmetry;
	  b_op.symop.op   = ctoi(biol_gens[0].symmetry[0L]);
	  b_op.symop.trans_x = ctoi(biol_gens[0].symmetry[2L]);
	  b_op.symop.trans_y = ctoi(biol_gens[0].symmetry[3L]);
	  b_op.symop.trans_z = ctoi(biol_gens[0].symmetry[4L]);
	
	  biol_ops.push_back(b_op);
	  // cerr << "Added " << i << endl;
	
	}
      }
    }
    else
      return biolUnit;

 
    /*
     * Create cell 
     */

    if (asym_mol.atom_site_list.length() != 0) {
      cell.a = asym_mol.sym_info.acell.unit_cell.length_a;
      cell.b = asym_mol.sym_info.acell.unit_cell.length_b;
      cell.c = asym_mol.sym_info.acell.unit_cell.length_c;
      cell.alpha = asym_mol.sym_info.acell.unit_cell.angle_alpha;
      cell.beta  = asym_mol.sym_info.acell.unit_cell.angle_beta;
      cell.gamma  = asym_mol.sym_info.acell.unit_cell.angle_gamma;
    
      // -- can do at once? braver
      for (i = 0; i < 3; i++) {
	cell.o_to_f_vs[i] = asym_mol.sym_info.atransf.vector[i];
	for (j = 0; j < 3; j++)
	  cell.o_to_fs[i][j] = asym_mol.sym_info.atransf.matrix[i][j];
      }

      ndb_frac_to_ortho_matrix(&cell);  
      ndb_ortho_to_frac_matrix(&cell);
      inverse_mat3x3(cell.o_to_fs,cell.f_to_os);

      memcpy(cell.o_to_f,cell.o_to_fs,sizeof(cell.o_to_fs));
      memcpy(cell.f_to_o,cell.f_to_os,sizeof(cell.f_to_os));
      memcpy(cell.o_to_f_v,cell.o_to_f_vs,sizeof(cell.o_to_f_vs));

      /*
       * End create cell
       */
    
      //copy space group
      biolUnit.sym_info.space_group = CORBA::string_dup(asym_mol.sym_info.space_group);

      //copy cell
      biolUnit.sym_info.acell = asym_mol.sym_info.acell;

      /*
	cerr << "biol_unit: cell constants are: "
	<< biolUnit.sym_info.cell.unit_cell.length_a << ' '
	<< biolUnit.sym_info.cell.unit_cell.length_b << ' '
	<< biolUnit.sym_info.cell.unit_cell.length_c << ' '
	<< biolUnit.sym_info.cell.unit_cell.angle_alpha << ' '
	<< biolUnit.sym_info.cell.unit_cell.angle_beta  << ' '
	<< biolUnit.sym_info.cell.unit_cell.angle_gamma
	<< endl;
      */
      
      // copy matrix -- can do at once? braver
      // cerr << "biol_unit: matrix => " << endl;
      // should be able to use a single assignment?
      for (i = 0; i < 3; i++) {
	biolUnit.sym_info.atransf.vector[i] = asym_mol.sym_info.atransf.vector[i];
	for (j = 0; j < 3; j++) {
	  biolUnit.sym_info.atransf.matrix[i][j] = asym_mol.sym_info.atransf.matrix[i][j];
	  // cerr << "\t[" << i << "][" << j << "]: " << biolUnit.sym_info.transf.matrix[i][j];
	}
	// cerr << endl;
      }
      // cerr << endl;
    
      // cerr << "biol_unit: vector: (";
      // for (i = 0; i < 3; ++i) {cerr << biolUnit.sym_info.transf.vector[i]; if (i == 2) break; cerr << ", ";}
      // cerr << ')' << endl;

      int ops_num = biol_ops.size();
      int num_atoms = asym_mol.atom_site_list.length();
      bool chain_match = false;

      // iterate trough all symmetry operations
      int counter = 0;
      while(counter < ops_num) {

	biol::BiolOps* asym_op = &biol_ops[counter];

	/*
	  cerr << "biol_unit: asym_op: symop: "
	  << asym_op->symop.name		<< ' '
	  << asym_op->symop.op		<< ' '
	  << asym_op->symop.trans_x	<< ' '
	  << asym_op->symop.trans_y	<< ' '
	  << asym_op->symop.trans_z	<< ' '
	  << endl;
	*/
	  
	cur_op = (NDBSYMTRANSDATA *)malloc(sizeof(NDBSYMTRANSDATA));
	cur_op = (NDBSYMTRANSDATA *)memcpy(cur_op,&sym->op[asym_op->symop.op - 1], sizeof(NDBSYMTRANSDATA));
      
	ndb_sym_incr_trans(cur_op,asym_op->symop.trans_x,
			   asym_op->symop.trans_y,    
			   asym_op->symop.trans_z);
      

	MMS::AsymUnitReplica new_rep;
	// new_rep.asym_id = asym_op->asym_id;
	new_rep.symop = CORBA::string_dup(asym_op->symop.name.c_str());
      
	biolUnit.replicas.length(counter+1);
	biolUnit.replicas[counter] = new_rep;
          

	// iterate trough all atoms
	int last = 0;
	for(i = 0; i < num_atoms; i++) {

	  chain_match = false;

	  for(j = 0; j < biol_ops[counter].asym_ids.size(); j++)
	    if(strcmp(asym_mol.atom_site_list[i].label.asym.id, biol_ops[counter].asym_ids[j].c_str()) == 0)
	      chain_match = true;

	  if(chain_match) {
	    MMS::AtomSite atm;
	    atm.label.asym.id = CORBA::string_dup(asym_mol.atom_site_list[i].label.asym.id);
	    atm.label.comp.id = CORBA::string_dup(asym_mol.atom_site_list[i].label.comp.id); 
	    atm.label.seq.id  = CORBA::string_dup(asym_mol.atom_site_list[i].label.seq.id);
	    atm.label.atom.id = CORBA::string_dup(asym_mol.atom_site_list[i].label.atom.id);
	  
	    atm.cartn.x =  asym_mol.atom_site_list[i].cartn.x;
	    atm.cartn.y =  asym_mol.atom_site_list[i].cartn.y;
	    atm.cartn.z =  asym_mol.atom_site_list[i].cartn.z;

	    atm.id = CORBA::string_dup(asym_mol.atom_site_list[i].id);
	    atm.group_pdb = CORBA::string_dup(asym_mol.atom_site_list[i].group_pdb);
		  
	    sym_trans_op1(cur_op,atm,&cell);		

	    biolUnit.replicas[counter].atom_site_list.length(last+1);
	    biolUnit.replicas[counter].atom_site_list[last++] = atm; 
	  }
	}
   
	free(cur_op);
	counter++;
      }
    }

    return biolUnit; 
  }
} // namespace biol_unit

#if 0
int main(){
  biol::AsymUnit molecule;
  biol::BiolGenList ops;
  molecule.sym_info.space_group = "A 1 2 1";

  cout<<"Goran"<< endl;

  biol_unit::biol_gen(molecule, ops);
    
  return 0;
}
#endif
