/* 
   File:   mol_symmetry_io.c
   Date:   10-June-96 J. Westbrook
  
   Collection of routines that manage packing operations...

*/

#include <iostream>

//#include "dsp_global.h"
//#include "mol_global.h"
//#include "mol_proto.h"

#include "../../dir.h" // CS_DIR

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <ctype.h>
#include <stdarg.h>
#include <math.h>
#include <string.h>
#include "ndb_cifparse.h"
#include "sym_types.h"
#include "mol_symmetry_io.h"
#include "biol.h"
#include "cell_constants.h"

namespace mol_symmetry {
#define NDB_ERF_SPACE_GROUP_DATABLOCK             "NDB_SPACE_GROUP"
#define NDB_ERF_SPACE_GROUP_CATEGORY              "ndb_space_group"


int cif_init = 0; 

// the first is unused, the second must reside
// in the same directory where the top executable is!
static char gblsymdbname[] = CS_FULL_DIR "some/BiolGen/cif/space_group.cif";
static char lclsymdbname[] = "space_group.cif";

#define align(s) {\
		    int j=0,i=0;\
		    while(isspace(s[i])) i++;\
		    if(i > 0) {\
		      /* get rid of front-spaces */ \
		      while(s[i] != '\0') {\
			s[j] = s[i]; j++; s[j] = '\0'; i++; }\
		      if(i > 1) { \
			i-=2; \
			/* get rid of trailing spaces */ \
			while(i>=0 && s[i] != '\0' && (isspace(s[i]))) {\
			  s[i] = '\0'; i--; }\
		      }\
		    }\
		  }


static void ndb_name_sym_rot_op(double *v, char *rotname)
/* |ndb_name_sym_rot_op|.
 *  |ndb_name_sym_rot_op()| deduces the axis and sign of a symmetry operation 
 *  from the a vector in the rotation matrix for the symmetry operation.
 *
 */
{
  int i,irow, axis,sign;
  char buf[10];

  for (i=0; i < 3; i++) {
    if (fabs(v[i]) >= 0.001) {
      irow = i;
      sign = 43;
      if (v[irow] < 0.0) sign = 45;
      axis = irow + 120;
      memset(buf,NULL,10);
      sprintf(buf,"%c%c",sign,axis);
      strcat(rotname,buf);
    }
  }

}

static void ndb_name_sym_trans(char *name, 
			NDBSYMTRANSDATA *op, int xt, int yt, int zt)
// 
 //ndb_name_sym_trans()| construct a name for the combination
// of the input symmetry operation and the accompanying translation relative
// the |(+x,+y,+z)| base position.
 //
{
    int i,tn,td,ti[3],sign,axis;
    static int term[]  = { 44, 44, 32 };
    char buf[NDBMAXNAMELEN], rotname[10];
    ti[0]=xt; ti[1]=yt; ti[2]=zt;
    memset(name,NULL,NDBMAXNAMELEN);
    
    for (i=0; i < 3 ; i++) {
	memset(buf,NULL,NDBMAXNAMELEN);
	tn = op->tn[i];
	td = op->td[i];
	if (ti[i] != 0) {
	    if (td != 0) 
		tn += ti[i]*td;
	    else {
		tn = ti[i];
		td = 1;
	    }
	}
	memset(rotname,NULL,10);
	memset(buf,NULL,NDBMAXNAMELEN);
	ndb_name_sym_rot_op(op->rot[i],rotname);
	if (td  != 0) {
	    if ( abs(td) == abs(tn) || abs(td) == 1 ) 
		sprintf(buf,"  %d%s%c ",tn/td,rotname,term[i]);
	    else
		sprintf(buf,"%d/%d%s%c ",tn,td,rotname,term[i]);
	} else
	    sprintf(buf,"   %s%c ",rotname,term[i]);
	
	
	strcat(name,buf);
	
	
    }
}  

int get_spacegroup_info(const char *spgname, SPACE_GROUP *space_group)
{
    int found, iret, rowno, i, j, colno, icategory, irow, oldRows, opno;
    char value[YYLMAX];
    FILE *fp;

    if (!cif_init) 
	cif_init = ndb_cif_init();  /* Only do this once! */

    icategory = ndb_cif_move_category_by_name(NDB_ERF_SPACE_GROUP_DATABLOCK, 
					      NDB_ERF_SPACE_GROUP_CATEGORY);
    if (icategory == 0) {
/*
 * Open the space group database file and search for this space group.
 */
	if ((fp = fopen(lclsymdbname,"r")) == NULL) {
	    if  ((fp = fopen(gblsymdbname,"r")) == NULL) {
		return FALSE;
	    } else 
		printf("Searching for %s in space group database: %s\n",
		       spgname,gblsymdbname);	
	} 
	else { 
	    printf("Searching for %s in local space group database: %s\n",
		   spgname,lclsymdbname);	
	}


	if ((iret = ndb_cif_read_file(fp)) == FALSE) {
	    //dp_error_panel("Error reading space group database");
	    fclose(fp);
	    return FALSE;
	}
	fclose(fp);
    
	icategory = ndb_cif_move_category_by_name(NDB_ERF_SPACE_GROUP_DATABLOCK, 
						  NDB_ERF_SPACE_GROUP_CATEGORY);
    }

    if (icategory == 0) return FALSE;


    rowno = ndb_cif_count_row();
    colno = ndb_cif_count_column();
    ndb_cif_rewind_row();
    irow=0;
    space_group->SpaceDataNo =0;
    do {
	ndb_cif_get_item_value(1, value, YYLMAX);
	ndb_clean_string(value); 
	if (!strcmp(value, spgname)) {
	    strcpy(space_group->SpaceGroupName, spgname);
	    oldRows = space_group->SpaceDataNo;
	    do {
		ndb_cif_get_item_value(1, value, YYLMAX);
		if (!strcmp(value, spgname)) space_group->SpaceDataNo ++;
	    } while (!strcmp(value, spgname) && ndb_cif_next_row()) ;
	    ndb_cif_move_row(irow+1);
	    space_group->SpaceData = (SPACE_DATA *)
		realloc(space_group->SpaceData, 
			space_group->SpaceDataNo * sizeof(SPACE_DATA));

	    for (i=oldRows; i< space_group->SpaceDataNo; i++) {
		ndb_cif_get_item_value(2, value, YYLMAX);
		opno = atoi(value);
		if (opno > 0) {
		    if (opno > space_group->SpaceDataNo) { 
			space_group->SpaceDataNo = opno;
			space_group->SpaceData = (SPACE_DATA *)
			    realloc(space_group->SpaceData, 
				    space_group->SpaceDataNo * sizeof(SPACE_DATA));
		    }
		    for (j=2; j< colno; j++) {
			ndb_cif_get_item_value(j+1, value, YYLMAX);
			space_group->SpaceData[opno-1].Data[j-2] = atoi(value);
		    }
		    
		}
		ndb_cif_next_row();
	    }
	}
	irow++;
    } while (ndb_cif_next_row());
    if (space_group->SpaceDataNo > 0) return TRUE;
    else return FALSE;
}




NDBSYMMETRY *ndb_get_sym_data(const char* spg_name,char unique_axis, int debug)
/*****************************************************************************
 *   Purpose:  ndb_get_sym_data() read the space group operation database 
 *             and return the transformations for this input space group
 *             in a NDBSYMMETRY structure.
 *             
 *****************************************************************************/
{
      FILE *fp, *fpd;
    NDBSYMMETRY *sym = NULL;
    int  i, j, k, nop, check=0;
    int r11,r12,r13,r21,r22,r23,r31,r32,r33;
    int tn1,td1,tn2,td2,tn3,td3, found;
    char line[80];
    char spgname[80];
    SPACE_GROUP *spacegroup=NULL;
    
//    cerr << "in ndb_get_sym_data" << endl;

    if (unique_axis != ' ')
	sprintf(spgname, "%s(%c)\0", spg_name, unique_axis);
    else
	strcpy(spgname, spg_name);
    ndb_clean_string(spgname);
    align(spgname);
  
    spacegroup = (SPACE_GROUP *) malloc(sizeof(SPACE_GROUP));
    spacegroup->SpaceData = NULL;
    found = get_spacegroup_info(spgname, spacegroup);

    if (debug) {
	fpd = fopen("symops.info","w");
    }

    if (found) {
	sym = (NDBSYMMETRY *) malloc(sizeof(NDBSYMMETRY));
	nop = spacegroup->SpaceDataNo;
	sym->nop = nop;
	memset(sym->spgname,NULL,NDBMAXNAMELEN);
	strcpy(sym->spgname,spgname);
	sym->op = (NDBSYMTRANSDATA *) malloc(sizeof(NDBSYMTRANSDATA)*(sym->nop+6));
	if (debug) fprintf(fpd,"Number of operations is %d\n",sym->nop);	
	for (i=0; i < sym->nop; i++) {
	    if (debug) fprintf(fpd,"Reading operation %d\n",i);	
	    for (j=0; j < 3; j++) {
		for (k=0; k < 3; k++) {
		    sym->op[i].rot[j][k] = (double) spacegroup->SpaceData[i].Data[j*3+k];
		}
		sym->op[i].tn[j] = spacegroup->SpaceData[i].Data[9+j*2];
		sym->op[i].td[j] = spacegroup->SpaceData[i].Data[9+j*2+1];
		sym->op[i].trans[j] = 0.0;
		if ((double) sym->op[i].td[j] != 0) 
		    sym->op[i].trans[j] = 
			(double) sym->op[i].tn[j] / (double) sym->op[i].td[j];
	    }
	}
	free(spacegroup->SpaceData);
	free(spacegroup);

    }
    else {
	//dp_error_panel("No data for space group: %s",spgname);
	exit(1);
    }

    for (i=0; i < sym->nop; i++ ) {
	memset(sym->op[i].opname,NULL,NDBMAXNAMELEN);
	ndb_name_sym_trans(sym->op[i].opname,&sym->op[i],0,0,0);
    }
/*
 *  Reset the name of the identity operation 
 */


       if (debug) fclose(fpd);

    if (0) {
	if ( (fp = fopen("ndb_symmetry.info","a")) != NULL) {
	    fprintf(fp,"\n Symmetry data for spacegroup %s\n",sym->spgname);
	    fprintf(fp," Number of operations %d\n",sym->nop);
	    for (i=0; i < sym->nop; i++) {
		fprintf(fp,"\n Operation: %s\n",sym->op[i].opname);
		fprintf(fp,"\n Rotation matrix %d\n",i+1);
		for (j = 0; j < 3 ; j ++ ){
		    fprintf(fp," %10f %10f %10f \n",sym->op[i].rot[j][0],
			    sym->op[i].rot[j][1],
			    sym->op[i].rot[j][2]);
		}
		fprintf(fp,"\n Translation vector  %d\n",i+1);
		fprintf(fp," %10f %10f %10f \n",sym->op[i].trans[0],
			sym->op[i].trans[1],
			sym->op[i].trans[2]);
	    }
	    fclose(fp);
	}
    }
/*
 * Add the operations for translations.... 
 */

    for (i=sym->nop; i < sym->nop + 6; i++) { 
	sym->op[i].rot[0][0] = 1.0;
	sym->op[i].rot[1][1] = 1.0;
	sym->op[i].rot[2][2] = 1.0;
	sym->op[i].rot[1][0] = 0.0;
	sym->op[i].rot[0][1] = 0.0;
	sym->op[i].rot[2][0] = 0.0;
	sym->op[i].rot[0][2] = 0.0;
	sym->op[i].rot[2][1] = 0.0;
	sym->op[i].rot[1][2] = 0.0;
	sym->op[i].trans[0] = 0.0;
	sym->op[i].trans[1] = 0.0;
	sym->op[i].trans[2] = 0.0;
	sym->op[i].tn[0] = 0;
	sym->op[i].tn[1] = 0;
	sym->op[i].tn[2] = 0;
	sym->op[i].td[0] = 0;
	sym->op[i].td[1] = 0;
	sym->op[i].td[2] = 0;
    }

    j=sym->nop;
    sym->op[j].tn[0] = 1.0;
    sym->op[j].td[0] = 1.0;
    strcpy(sym->op[j].opname,"Translate 1 along a");
    j++;
    sym->op[j].tn[1] = 1.0;
    sym->op[j].td[1] = 1.0;
    memset(sym->op[j].opname,NULL,NDBMAXNAMELEN);
    strcpy(sym->op[j].opname,"Translate 1 along b");
    j++;
    sym->op[j].tn[2] = 1.0;
    sym->op[j].td[2] = 1.0;
    memset(sym->op[j].opname,NULL,NDBMAXNAMELEN);
    strcpy(sym->op[j].opname,"Translate 1 along c");
    j++;
    sym->op[j].tn[0] = -1.0;
    sym->op[j].td[0] =  1.0;
    memset(sym->op[j].opname,NULL,NDBMAXNAMELEN);
    strcpy(sym->op[j].opname,"Translate -1 along a");
    j++;
    sym->op[j].tn[1] = -1.0;
    sym->op[j].td[1] =  1.0;
    memset(sym->op[j].opname,NULL,NDBMAXNAMELEN);
    strcpy(sym->op[j].opname,"Translate -1 along b");
    
    j++;
    sym->op[j].tn[2] = -1.0;
    sym->op[j].td[2] =  1.0;
    memset(sym->op[j].opname,NULL,NDBMAXNAMELEN);
    strcpy(sym->op[j].opname,"Translate -1 along c");

    for (i=sym->nop; i < sym->nop + 6; i++) { 
	tn1 = sym->op[i].tn[0];
	tn2 = sym->op[i].tn[1];
	tn3 = sym->op[i].tn[2];
	td1 = sym->op[i].td[0];
	td2 = sym->op[i].td[1];
	td3 = sym->op[i].td[2];
	if (td1 != 0) sym->op[i].trans[0] = (double) tn1 / (double) td1;
	if (td2 != 0) sym->op[i].trans[1] = (double) tn2 / (double) td2;
	if (td3 != 0) sym->op[i].trans[2] = (double) tn3 / (double) td3;
	
	//memset(sym->op[i].opname,NULL,NDBMAXNAMELEN); 
	
	//ndb_name_sym_trans(sym->op[i].opname,&sym->op[i],0,0,0); 
	
    }
    return(sym);



}




void ndb_sym_incr_trans(NDBSYMTRANSDATA* cur_op,int x,int y, int z)
/* 
 * ndb_sym_incr_trans() set translations along x,y, and z directions 
 * according to the value of it.  There are 27 possible unit translations 
 * (+/-) 1 unit about (0,0,0).
 */
{

  cur_op->trans[0] = cur_op->trans[0] + 5 - x;
  cur_op->trans[1] = cur_op->trans[1] + 5 - y;  
  cur_op->trans[2] = cur_op->trans[2] + 5 - z;  
}


int sym_trans_op1( NDBSYMTRANSDATA* cur_op, MMS::AtomSite &atomSite, NDBCELLCONST* cell)
/* 
 * |sym_trans_op1()|  transform the molecular coordinates according 
 * to the contents of |*cur_op|  and the current value 
 * of |s_mode|. This routine supports an ADD mode and a MOVE mode. Both
 * operations are performed relative to the position of the base molecule.
 * |*bco| holds the current reference fractional coordinates.
 *
 */
{
  
 
       
	MMS::VectorXYZ* tuple = &atomSite.cartn;

	ndb_trans_coord_3by3( &tuple->x,
                              &tuple->y,
                              &tuple->z,
                             cell,NDB_TRANS_ORTHO_TO_FRAC);

	ndb_do_sym_op( tuple, cur_op);

	ndb_trans_coord_3by3(&tuple->x,
                             &tuple->y,
                             &tuple->z,cell,NDB_TRANS_FRAC_TO_ORTHO);

	
			     
   
    return 1;
}



void ndb_do_sym_op(MMS::VectorXYZ* tuple,  NDBSYMTRANSDATA *op)
/* 
 *
 * |ndb_do_sym_op()| transforms the coordinates in |fco| according
 * to the symmetry operation specified in |*op/
 *            
 *
 */
{

   
  int i,check = 0;
  double x,y,z;
 

 
  //  Rows of rot(3,3) x col(3,1) = col'(3,1)
   
  
 
    x=tuple->x*op->rot[0][0] + tuple->y*op->rot[0][1] + tuple->z*op->rot[0][2];
    y=tuple->x*op->rot[1][0] + tuple->y*op->rot[1][1] + tuple->z*op->rot[1][2];
    z=tuple->x*op->rot[2][0] + tuple->y*op->rot[2][1] + tuple->z*op->rot[2][2];
    tuple->x = x + op->trans[0];
    tuple->y = y + op->trans[1];
    tuple->z = z + op->trans[2];
 


 
}



void  ndb_trans_coord_3by3(float *x, float *y, float *z,
			   NDBCELLCONST *cell, int iflag)
/*  
 *   ndb_trans_coord_3by3()  multiplies the vector (x,y,z) by one of 
 *   the (3x3) transformation matrices in the cell constant structure.
 *   The matrix is specified by 'iflag'.
 */
{
  int i, j;
  float x0[3], t[3];

  x0[0] = *x;
  x0[1] = *y;
  x0[2] = *z;

  if (iflag == NDB_TRANS_ORTHO_TO_FRAC) {
    for (i=0; i < 3; i++) {
      t[i] = 0.0;
      for (j=0; j < 3; j++) {
	t[i] = t[i] + cell->o_to_f[i][j]*x0[j];
      }
      *x = t[0];
      *y = t[1];
      *z = t[2];
    }
    
  }

  if (iflag == NDB_TRANS_FRAC_TO_ORTHO) {
    for (i=0; i < 3; i++) {
      t[i] = 0.0;
      for (j=0; j < 3; j++) {
	t[i] = t[i] + cell->f_to_o[i][j]*x0[j];
      }
    }
    *x = t[0];
    *y = t[1];
    *z = t[2];
  }
}

void ndb_frac_to_ortho_matrix(NDBCELLCONST *cell)
/* 
 *  ndb_frac_to_ortho_matrix()  builds the transformation matrix between 
 *  fractional and orthogonal coordinates from the cell
 *  constants a,b,c,alpha,beta, and gamma and loads this
 *  into the cell constant structure.
 */
{
  int i;
  float rad, v, va, aa[3],ang[3];
  float alphr[3],cosa[3],sina[3];

  aa[0] = cell->a;
  aa[1] = cell->b;
  aa[2] = cell->c;
  
  ang[0] = cell->alpha;
  ang[1] = cell->beta;
  ang[2] = cell->gamma;

  rad = acos(-1.0)/180.0;
  for (i=0; i < 3; i++) {
    alphr[i] = ang[i]*rad;
    cosa[i]  = cos(alphr[i]);   
    sina[i]  = sin(alphr[i]);
    if ( fabs(ang[i]-90.0) < 0.001 ) {
      cosa[i]=0.0;
      sina[i]=1.0;  
    }
  }
  va = sqrt(1.0 + 2.0*cosa[0]*cosa[1]*cosa[2] - cosa[0]*cosa[0] 
	    - cosa[1]*cosa[1] - cosa[2]*cosa[2]);
  v = aa[0]*aa[1]*aa[2]*va;

  cell->f_to_oc[0][0] = aa[0];
  cell->f_to_oc[0][1] = aa[1]*cosa[2];  
  cell->f_to_oc[0][2] = aa[2]*cosa[1];  
  cell->f_to_oc[1][0] = 0.0; 
  cell->f_to_oc[1][1] = aa[1]*sina[2];  
  cell->f_to_oc[1][2] = aa[2]*(cosa[0]-cosa[1]*cosa[2])/sina[2];
  cell->f_to_oc[2][0] = 0.0; 
  cell->f_to_oc[2][1] = 0.0; 
  cell->f_to_oc[2][2] = v/(aa[0]*aa[1]*sina[2]);

}

void ndb_ortho_to_frac_matrix(NDBCELLCONST *cell)
/*  
 * ndb_ortho_to_frac_matrix()  builds the transformation matrix 
 * between fractional and orthogonal coordinates from the cell
 * constants a,b,c,alpha,beta, and gamma and loads this
 * into the cell constant structure.
 */
{
  int i;
  float rad, v, va, aa[3],ang[3],t[3][3];
  float alphr[3],cosa[3],sina[3];


  aa[0] = cell->a;
  aa[1] = cell->b;
  aa[2] = cell->c;
  
  ang[0] = cell->alpha;
  ang[1] = cell->beta;
  ang[2] = cell->gamma;

  rad = acos(-1.0)/180.0;
  for (i=0; i < 3; i++) {
    alphr[i] = ang[i]*rad;
    cosa[i]  = cos(alphr[i]);   
    sina[i]  = sin(alphr[i]);
    if ( fabs(ang[i]-90.0) < 0.001 ) {
      cosa[i]=0.0;
      sina[i]=1.0;  
    }
  }

  va = sqrt(1.00 + 2.0*cosa[0]*cosa[1]*cosa[2] - cosa[0]*cosa[0] 
	    - cosa[1]*cosa[1] - cosa[2]*cosa[2]);
  v = aa[0]*aa[1]*aa[2]*va;
/*  fprintf(stdout,"va = %f v = %f\n",va,v); */
  cell->o_to_fc[0][0] = 1.0/aa[0];
  cell->o_to_fc[0][1] = -cosa[2]/(aa[0]*sina[2]);
  cell->o_to_fc[0][2] = (cosa[2]*cosa[0]-cosa[1]) / (aa[0]*va*sina[2]);
  cell->o_to_fc[1][0] = 0.0 ;
  cell->o_to_fc[1][1] = 1.0/(aa[1]*sina[2]);
  cell->o_to_fc[1][2] = (cosa[1]*cosa[2]-cosa[0]) / (aa[1]*va*sina[2]);
  cell->o_to_fc[2][0] = 0.0; 
  cell->o_to_fc[2][1] = 0.0;
  cell->o_to_fc[2][2] = sina[2]/(aa[2]*va);
}


void inverse_mat3x3(float a[3][3], float b[3][3])
/*  
 *  inverse_mat3x3()  calculates the inverse of a 3 x 3 matrix,  B= A^{-1} 
 */
{
  float det,rdet,tol=0.000000001;
  int i,j;

  adjoint_mat3x3(a,b);

  det = det3x3(a);

  if ( fabs(det) < tol) {
    fprintf(stderr,"Matrix determinant is zero\n");
    exit(1);
  }
  rdet = 1.0/det;
  for (i=0; i < 3; i++) {
    for (j=0; j < 3; j++ ) {
      b[i][j] = b[i][j] * rdet;
    }
  }
}

float det3x3(float a[3][3])
/*  
 *   det3x3()  returns the determinant of matrix A.
 */
{
  float det;
  det = a[0][0]*det2x2(a[1][1],a[2][1],a[1][2],a[2][2]) 
      - a[0][1]*det2x2(a[1][0],a[2][0],a[1][2],a[2][2]) 
      + a[0][2]*det2x2(a[1][0],a[2][0],a[1][1],a[2][1]);
  return det;
}


void adjoint_mat3x3(float a[3][3], float b[3][3])
{
  b[0][0] =  det2x2(a[1][1],a[2][1],a[1][2],a[2][2]);
  b[0][1] = -det2x2(a[0][1],a[2][1],a[0][2],a[2][2]);
  b[0][2] =  det2x2(a[0][1],a[1][1],a[0][2],a[1][2]);
  b[1][0] = -det2x2(a[1][0],a[2][0],a[1][2],a[2][2]);
  b[1][1] =  det2x2(a[0][0],a[2][0],a[0][2],a[2][2]);
  b[1][2] = -det2x2(a[0][0],a[1][0],a[0][2],a[1][2]);
  b[2][0] =  det2x2(a[1][0],a[2][0],a[1][1],a[2][1]);
  b[2][1] = -det2x2(a[0][0],a[2][0],a[0][1],a[2][1]);
  b[2][2] =  det2x2(a[0][0],a[1][0],a[0][1],a[1][1]);
}


float det2x2(float a, float b, float c, float d)
/*  
 *  det2x2()  returns the determinant of a 2x2 matrix enumerated in 
 *         column order.
 */
{
  float  det;
  det = a*d - b*c;
  return det;
}

} // namespace mol_symmetry
