// Braver Entry Factory
// Factory body
// $Id: MMS_EntryFactoryImpl.C,v 1.7 2003/06/05 21:14:58 alexy Exp $

#include "MMS_EntryFactoryImpl.h"

#include "io.h"

// _create() -- create a new servant.
POA_MMS::EntryFactory*
MMS_EntryFactoryImpl::_create(PortableServer::POA_ptr the_poa)
  throw (MMS::DataAccessException)
{
    return new MMS_EntryFactoryImpl(the_poa);
}


// Constructor
MMS_EntryFactoryImpl::MMS_EntryFactoryImpl(
    PortableServer::POA_ptr the_poa
) throw (MMS::DataAccessException)
  :
  _poaV(PortableServer::POA::_duplicate(the_poa)),
  _nameV(CORBA::string_dup("A Braver Entry Factory")),
  _gotEntryIdList(false)
{
  cerr << "MMS_EntryFactoryImpl constructor called" << endl;
  load_entry_id_list();
}


void MMS_EntryFactoryImpl::load_entry_id_list()
{
  if (_gotEntryIdList) { 
    cerr << "got " << _entryIdListV->length() << " entry IDs" << endl; 
    return; 
  }

  cerr << "loading entries" << endl;

  _entryIdListV = MMS_EntryFactoryImpl::fetch_entry_id_list();

  _gotEntryIdList = true;
}


// ~MMS_EntryFactoryImpl destructor.
MMS_EntryFactoryImpl::~MMS_EntryFactoryImpl()
{
  // checking and throwing?
}


PortableServer::POA_ptr
MMS_EntryFactoryImpl::_default_POA()
{
    return PortableServer::POA::_duplicate(_poaV);
}


// name() -- Accessor
char* 
MMS_EntryFactoryImpl::name()
  throw (CORBA::SystemException)

{
    return CORBA::string_dup(_nameV);
}


// name() -- Modifier
void 
MMS_EntryFactoryImpl::name(const char* _new_value) 
  throw (CORBA::SystemException)
{
  _nameV = CORBA::string_dup(_new_value);
}


MMS::EntryIdList* MMS_EntryFactoryImpl::get_entry_id_list() 
  throw (MMS::DataAccessException)
{
  MMS::EntryIdList_var V;
  V = _entryIdListV; // deep copy
  return V._retn();
}    // MMS


CORBA::Long MMS_EntryFactoryImpl::get_entry_id_list_size() 
  throw (MMS::DataAccessException)
{
  return _entryIdListV->length();
}


// had to replace MMS::EntryId by const char * below:
MMS::Entry_ptr MMS_EntryFactoryImpl::get_entry_from_id(const char* entry_id) 
  throw (MMS::DataAccessException)
{
  // check whether the id is in the list, throw if not
  bool found = 0;
  for (long i = 0; i <  _entryIdListV->length(); ++i)
    if (found = !strcmp(_entryIdListV[i], entry_id)) break;
  if (!found) {
    cerr << "entry " << entry_id << " not found!" << endl;
    throw MMS::DataAccessException("MMS_EntryFactoryImpl::get_entry_from_id: invalid entry", entry_id);
  }
    
  PortableServer::ServantBase_var servantV;
  PortableServer::ObjectId_var oidV;
  CORBA::Object_var objV;
  MMS::Entry_var entryV;

  servantV = MMS_EntryImpl::_create(_poaV, entry_id);
  oidV = _poaV->activate_object(servantV.in());
  objV = _poaV->id_to_reference(oidV);
  entryV = MMS::Entry::_narrow(objV);
  if (CORBA::is_nil(entryV.in())) {
    cerr << "error: failed to narrow to MMS::Entry" << endl;
    exit(1);
  }

  return entryV._retn();
}
