// Braver Entry Factory
// Entry body
// $Id: MMS_EntryImpl.C,v 1.11 2003/03/09 17:59:28 alexy Exp $

#include "MMS_EntryImpl.h"
#include "Xray_SubentryImpl.h"

#include "cp.h" // global connpool::cp

#include "itoa.h"
#include "list_size.h"
#include "duration.h"

#include "db.h"

#include "io.h"

// _create() -- create a new servant.
POA_MMS::Entry*
MMS_EntryImpl::_create(PortableServer::POA_ptr the_poa,
		       const char *entry_id)
{
    return new MMS_EntryImpl(the_poa, entry_id);
}


// Constructor
MMS_EntryImpl::MMS_EntryImpl(PortableServer::POA_ptr the_poa, 
			     const char *entry_id) 
  : 
  _poaV(PortableServer::POA::_duplicate(the_poa)),
  _idV(CORBA::string_dup(entry_id)),
  _id(CORBA::string_dup(entry_id))
{
  if (strcmp((char*)_idV, "Entry1") == 0) {
    name("Abra");
    value(1917);
  }
  else {
    name("Cadabra");
    value(2002);
  }

  // Create a new subentry!
  PortableServer::ServantBase_var servantV;
  PortableServer::ObjectId_var oidV;
  CORBA::Object_var objV;
 
  servantV = Xray_SubentryImpl::_create(_poaV, "whatever, some subentry");
  oidV = _poaV->activate_object(servantV.in());
  objV = _poaV->id_to_reference(oidV);
  _subentryV = Xray::Subentry::_narrow(objV);
  if (CORBA::is_nil(_subentryV.in())) {
    cerr << "error: failed to narrow to Xray::Subentry" << endl;
    exit(1);
  }

#if 0 //  gcc 3.3 gives the following error, until 0=>(long)0,
      //  is there a signed long integer literal suffix?
MMS_EntryImpl.C:58: error: ISO C++ says that `typename T::ElementType 
   TSeqVar<T>::operator[](long unsigned int) [with T = 
   SequenceTmpl<PropertyService::Property, 0>]' and `operator[]' are ambiguous 
   even though the worst conversion for the former is better than the worst 
   conversion for the latter
  
  _propsV = new CosPropertyService::Properties(1);
  _propsV->length(1);
  _propsV[(long)0].property_name  = "xray";
  _propsV[(long)0].property_value <<= _subentryV;
#endif
}


// ~MMS_EntryImpl destructor.
MMS_EntryImpl::~MMS_EntryImpl()
{
}

PortableServer::POA_ptr
MMS_EntryImpl::_default_POA()
{
    return PortableServer::POA::_duplicate(_poaV);
}


char* 
MMS_EntryImpl::get_id()
  throw (CORBA::SystemException)

{
    return CORBA::string_dup(_idV);
}


// name() -- Accessor
char* 
MMS_EntryImpl::name()
  throw (CORBA::SystemException)

{
    return CORBA::string_dup(_nameV);
}


// name() -- Modifier
void 
MMS_EntryImpl::name(const char* _new_value) 
  throw (CORBA::SystemException)
{
  _nameV = CORBA::string_dup(_new_value);
}


// value() -- Accessor
CORBA::Long 
MMS_EntryImpl::value()
  throw (CORBA::SystemException)
{
    return _valueV;
}


// value() -- Modifier
void 
MMS_EntryImpl::value(const CORBA::Long _new_value) 
  throw (CORBA::SystemException)
{
  _valueV = _new_value;
}


// value -- get method
CORBA::Long
MMS_EntryImpl::get_value()  
  throw (CORBA::SystemException)
{
    return _valueV;
}


CORBA::Long list_size(const SQLHANDLE& hdbc, const string& category, const string& id)
  throw (CORBA::SystemException, MMS::DataAccessException);

#include "auto/mms.C"
// #include "atom.C"

MMS::Flags*
MMS_EntryImpl::get_presence_flags()
  throw (CORBA::SystemException, MMS::DataAccessException)
{
  SQLRETURN   sqlrc;
  SQLHANDLE   hdbc; // connection handle
  SQLHANDLE   hstmt ; // statement handle
  
  // schema/table can be a variable from a file
  string sstmt = "SELECT flags FROM DB2INST1.FLAGS WHERE structure_id = \'" + _id +'\'';
  SQLCHAR * stmt = (SQLCHAR *)sstmt.c_str();

  // BUFSIZ is defined in stdio.h!
  const unsigned long bufferSize = 1000L; // can be read from the database as BLOB(size)

  SQLCHAR         buffer[bufferSize];     
  SQLINTEGER      bufInd;

  try {
    hdbc = connpool::cp.getConnection();
  } catch (DBException) {
    throw MMS::DataAccessException("MMS_EntryImpl::get_presence_flags()", "no connections!");
  }

  sqlrc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
  if (sqlrc != SQL_SUCCESS)
    throw MMS::DataAccessException("MMS_EntryImpl::get_presence_flags()", "SQLAllocHandle");

  sqlrc = SQLPrepare( hstmt, stmt, SQL_NTS ) ;
  if (sqlrc != SQL_SUCCESS)
    throw MMS::DataAccessException("MMS_EntryImpl::get_presence_flags()", "SQLPrepare");

  sqlrc = SQLExecute(hstmt);
  if (sqlrc != SQL_SUCCESS)
    throw MMS::DataAccessException("MMS_EntryImpl::get_presence_flags()", "SQLExecute");
  
  sqlrc = SQLFetch(hstmt);
  if (sqlrc == SQL_NO_DATA)
    throw MMS::DataAccessException("MMS_EntryImpl::get_presence_flags()", "SQLFetch no data");
  else if (sqlrc != SQL_SUCCESS)
    throw MMS::DataAccessException("MMS_EntryImpl::get_presence_flags()", "SQLFetch");
  
  sqlrc = SQLGetData(hstmt, 1, SQL_C_BINARY,
		     (SQLPOINTER) buffer, BUFSIZ, &bufInd);
  if (sqlrc == SQL_NO_DATA)
    throw MMS::DataAccessException("MMS_EntryImpl::get_presence_flags()", "SQLFetch 2 no data");
  else if (sqlrc != SQL_SUCCESS)
    throw MMS::DataAccessException("MMS_EntryImpl::get_presence_flags()", "SQLGetData");

  sqlrc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt) ;
  if (sqlrc != SQL_SUCCESS)
    throw MMS::DataAccessException("MMS_EntryImpl::get_presence_flags()", "SQLFreeHandle");

  
  MMS::Flags *flagsP = new MMS::Flags(bufInd);
  flagsP->length(bufInd);

  memcpy(&(*flagsP)[0], buffer, bufInd);

  connpool::cp.returnConnection(hdbc);
  return flagsP;
}


CORBA::Long list_size(const SQLHANDLE& hdbc, const string& category, const string& id)
    throw (CORBA::SystemException, MMS::DataAccessException)
{
    SQLRETURN   sqlrc = SQL_SUCCESS;
    SQLHANDLE   hstmt ; // statement handle

    string sstmt = "SELECT count(*) FROM DB2INST1." + category
      + " WHERE structure_id = '" + id + '\'';
    SQLCHAR * stmt = ( SQLCHAR * ) sstmt.c_str();
        
    sqlrc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::list_size()", "SQLAllocHandle");

    sqlrc = SQLExecDirect(hstmt, stmt, SQL_NTS);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::list_size()", "SQLExecute");

    struct {
        SQLINTEGER ind;
        SQLINTEGER val;
    } total;

    sqlrc = SQLBindCol(hstmt, 1, SQL_C_LONG, &total.val, 0, &total.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::list_size()", "SQLBindCol");

    sqlrc = SQLFetch(hstmt);
    if (sqlrc != SQL_SUCCESS && sqlrc != SQL_NO_DATA)
        throw MMS::DataAccessException("MMS_EntryImpl::list_size()", "SQLFetch");

    if (sqlrc == SQL_NO_DATA_FOUND)
        throw MMS::DataAccessException("MMS_EntryImpl::list_size()", "no data");

    sqlrc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::list_size()", "SQLFreeHandle");

    return total.val;
}


CORBA::Any * MMS_EntryImpl::subentry()
{
  CORBA::Any_var a = new CORBA::Any();

  a <<= _subentryV;
  
  return a._retn();
}


CosPropertyService::Properties * MMS_EntryImpl::get_subentry_list()
{
  CosPropertyService::Properties_var propsV = _propsV;

  return propsV._retn();
}
