#include "MMS_EntryImpl.h"
#include "db.h"
#include "cp.h"
#include "itoa.h"
#include "list_size.h"

#include "BiolGen/include/mms_biol_unit.h"

#include <sstream>

MMS::AtomSiteList*
MMS_EntryImpl::get_atom_site_some_list(const char* beg_label_asym,
				       const long  beg_label_seq,
				       const long  end_label_seq)
    throw (CORBA::SystemException, MMS::DataAccessException)
{
    SQLRETURN   sqlrc = SQL_SUCCESS;
    SQLHANDLE   hdbc; // connection handle
    SQLHANDLE   hstmt ; // statement handle
    char        structure_id[10];

    MMS::AtomSiteList* listP = 0;
    CORBA::ULong count;
    SQLCHAR * stmt;

    try {
      connpool::Connection conn;
      hdbc = conn.hdbc; // use a reference/pointer, or it's already a pointer?
    } catch (DBException) {
      throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_list()", "no connections!");
    }

    ostringstream where;

    CORBA::ULong listSize = list_size(hdbc, "ATOM_SITE", _id);
    if (!listSize) goto finish;

    if (beg_label_asym && strlen(beg_label_asym)) { where << " AND LABEL_ASYM_ID = '" << beg_label_asym << "'"; }
    else throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "label_asym empty");
    where << " AND LABEL_SEQ_ID >= "
	  <<  beg_label_seq
	  << " AND LABEL_SEQ_ID <= "
	  << end_label_seq;
    
    stmt = ( SQLCHAR * )
      (string("SELECT B_ISO_OR_EQUIV,ID,OCCUPANCY,LABEL_ENTITY_ID,TYPE_SYMBOL,CARTN_X,CARTN_Y,CARTN_Z,LABEL_ALT_ID,LABEL_ASYM_ID,LABEL_ATOM_ID,LABEL_COMP_ID,LABEL_SEQ_ID FROM DB2INST1.ATOM_SITE WHERE structure_id = ?")
      + where.str()).c_str();
    // cerr << stmt << endl;
    
    sqlrc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "SQLAllocHandle");

    sqlrc = SQLPrepare(hstmt, stmt, SQL_NTS);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "SQLPrepare");

    sqlrc = SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, 10, 0, structure_id, 10, NULL);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "SQLBindParameter");

    strcpy( structure_id, (char*)_idV);
    sqlrc = SQLExecute(hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "SQLExecute");

    struct {
        SQLINTEGER ind;
        SQLDOUBLE val;
    } B_ISO_OR_EQUIV;

    struct {
        SQLINTEGER ind;
        SQLCHAR val[10];
    } ID;

    struct {
        SQLINTEGER ind;
        SQLDOUBLE val;
    } OCCUPANCY;

    struct {
        SQLINTEGER ind;
        SQLCHAR val[10];
    } LABEL_ENTITY_ID;

    struct {
        SQLINTEGER ind;
        SQLCHAR val[10];
    } TYPE_SYMBOL;

    struct {
        SQLINTEGER ind;
        SQLDOUBLE val;
    } CARTN_X;

    struct {
        SQLINTEGER ind;
        SQLDOUBLE val;
    } CARTN_Y;

    struct {
        SQLINTEGER ind;
        SQLDOUBLE val;
    } CARTN_Z;

    struct {
        SQLINTEGER ind;
        SQLCHAR val[10];
    } LABEL_ALT_ID;

    struct {
        SQLINTEGER ind;
        SQLCHAR val[10];
    } LABEL_ASYM_ID;

    struct {
        SQLINTEGER ind;
        SQLCHAR val[6];
    } LABEL_ATOM_ID;

    struct {
        SQLINTEGER ind;
        SQLCHAR val[10];
    } LABEL_COMP_ID;

    struct {
        SQLINTEGER ind;
        SQLINTEGER val;
    } LABEL_SEQ_ID;

    sqlrc = SQLBindCol( hstmt, 1, SQL_C_DOUBLE, &B_ISO_OR_EQUIV.val, 0, &B_ISO_OR_EQUIV.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "SQLBindCol 1");

    sqlrc = SQLBindCol( hstmt, 2, SQL_C_CHAR, &ID.val, 10, &ID.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "SQLBindCol 2");

    sqlrc = SQLBindCol( hstmt, 3, SQL_C_DOUBLE, &OCCUPANCY.val, 0, &OCCUPANCY.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "SQLBindCol 3");

    sqlrc = SQLBindCol( hstmt, 4, SQL_C_CHAR, &LABEL_ENTITY_ID.val, 10, &LABEL_ENTITY_ID.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "SQLBindCol 4");

    sqlrc = SQLBindCol( hstmt, 5, SQL_C_CHAR, &TYPE_SYMBOL.val, 10, &TYPE_SYMBOL.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "SQLBindCol 5");

    sqlrc = SQLBindCol( hstmt, 6, SQL_C_DOUBLE, &CARTN_X.val, 0, &CARTN_X.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "SQLBindCol 6");

    sqlrc = SQLBindCol( hstmt, 7, SQL_C_DOUBLE, &CARTN_Y.val, 0, &CARTN_Y.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "SQLBindCol 7");

    sqlrc = SQLBindCol( hstmt, 8, SQL_C_DOUBLE, &CARTN_Z.val, 0, &CARTN_Z.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "SQLBindCol 8");

    sqlrc = SQLBindCol( hstmt, 9, SQL_C_CHAR, &LABEL_ALT_ID.val, 10, &LABEL_ALT_ID.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "SQLBindCol 9");

    sqlrc = SQLBindCol( hstmt, 10, SQL_C_CHAR, &LABEL_ASYM_ID.val, 10, &LABEL_ASYM_ID.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "SQLBindCol 10");

    sqlrc = SQLBindCol( hstmt, 11, SQL_C_CHAR, &LABEL_ATOM_ID.val, 6, &LABEL_ATOM_ID.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "SQLBindCol 11");

    sqlrc = SQLBindCol( hstmt, 12, SQL_C_CHAR, &LABEL_COMP_ID.val, 10, &LABEL_COMP_ID.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "SQLBindCol 12");

    sqlrc = SQLBindCol( hstmt, 13, SQL_C_LONG, &LABEL_SEQ_ID.val, 0, &LABEL_SEQ_ID.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "SQLBindCol 13");

    sqlrc = SQLFetch(hstmt);
    if (sqlrc == SQL_NO_DATA_FOUND) goto finish;

    // we know the result set size here, but can just grow the list
    // as we iterate over it, below, with listP->length(count+1)
    listP = new MMS::AtomSiteList(0); // have to do it?
    listP->length(0);
    count = 0;

    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "SQLFetch first");

    while (sqlrc != SQL_NO_DATA_FOUND) {
      // this alone seems to work for elongating the list:
        listP->length(count+1);
        (*listP)[count].b_iso_or_equiv = B_ISO_OR_EQUIV.val;
        (*listP)[count].id = CORBA::string_dup((char*)ID.val);
        (*listP)[count].occupancy = OCCUPANCY.val;
        (*listP)[count].label_entity.id = CORBA::string_dup((char*)LABEL_ENTITY_ID.val);
        (*listP)[count].label_entity.index = atoi((char*)LABEL_ENTITY_ID.val);
        (*listP)[count].type_symbol.id = CORBA::string_dup((char*)TYPE_SYMBOL.val);
        (*listP)[count].type_symbol.index = atoi((char*)TYPE_SYMBOL.val);
        (*listP)[count].cartn.x = CARTN_X.val;
        (*listP)[count].cartn.y = CARTN_Y.val;
        (*listP)[count].cartn.z = CARTN_Z.val;
        (*listP)[count].label.atom.id = CORBA::string_dup((char*)LABEL_ATOM_ID.val);
        (*listP)[count].label.atom.index = atoi((char*)LABEL_ATOM_ID.val);
        (*listP)[count].label.alt.id = CORBA::string_dup((char*)LABEL_ALT_ID.val);
        (*listP)[count].label.alt.index = atoi((char*)LABEL_ALT_ID.val);
        (*listP)[count].label.asym.id = CORBA::string_dup((char*)LABEL_ASYM_ID.val);
        (*listP)[count].label.asym.index = atoi((char*)LABEL_ASYM_ID.val);
        (*listP)[count].label.comp.id = CORBA::string_dup((char*)LABEL_COMP_ID.val);
        (*listP)[count].label.comp.index = atoi((char*)LABEL_COMP_ID.val);
        (*listP)[count].label.seq.id = itoa(LABEL_SEQ_ID.val);
        (*listP)[count].label.seq.index = LABEL_SEQ_ID.val;

        ++count;

        sqlrc = SQLFetch(hstmt);
        if (sqlrc != SQL_SUCCESS && sqlrc != SQL_NO_DATA_FOUND)
            throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "SQLFetch loop");
    }
    sqlrc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_site_some_list()", "SQLFreeHandle");

finish:
    return listP;
}


// we'll assume single AtomSites per structure for now
#ifdef ATOM_SITES_LIST
CORBA::Long
MMS_EntryImpl::atom_sites_list_size()
    throw (CORBA::SystemException, MMS::DataAccessException)
{
    SQLRETURN   sqlrc = SQL_SUCCESS;
    SQLHANDLE   hdbc; // connection handle
    SQLHANDLE   hstmt ; // statement handle
    char        structure_id[10];

    SQLCHAR * stmt = ( SQLCHAR * )
        "SELECT count(*) FROM DB2INST1.ATOM_SITES WHERE structure_id = ?";

    try {
        hdbc = connpool::cp.getConnection();
        } catch (DBException) {
        throw MMS::DataAccessException("MMS_EntryImpl::atom_sites_list_size()", "no connections!");
    }

    sqlrc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::atom_sites_list_size()", "SQLAllocHandle");

    sqlrc = SQLPrepare(hstmt, stmt, SQL_NTS);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::atom_sites_list_size()", "SQLPrepare");

    sqlrc = SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, 10, 0, structure_id, 10, NULL);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::atom_sites_list_size()", "SQLBindParameter");

    strcpy( structure_id, (char*)_idV);
    sqlrc = SQLExecute(hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::atom_sites_list_size()", "SQLExecute");

    struct {
        SQLINTEGER ind;
        SQLINTEGER val;
    } total;

    sqlrc = SQLBindCol(hstmt, 1, SQL_C_LONG, &total.val, 0, &total.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::atom_sites_list_size()", "SQLBindCol");

    sqlrc = SQLFetch(hstmt);
    if (sqlrc != SQL_SUCCESS && sqlrc != SQL_NO_DATA)
        throw MMS::DataAccessException("MMS_EntryImpl::atom_sites_list_size()", "SQLFetch");

    if (sqlrc == SQL_NO_DATA_FOUND)
        throw MMS::DataAccessException("MMS_EntryImpl::atom_sites_list_size()", "no data");

    sqlrc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::atom_sites_list_size()", "SQLFreeHandle");

finish:
    connpool::cp.returnConnection(hdbc);
    return total.val;
}
#else // we'll assume single AtomSites per structure for now
MMS::AtomSites
MMS_EntryImpl::get_atom_sites()
    throw (CORBA::SystemException, MMS::DataAccessException)
{
    SQLRETURN   sqlrc = SQL_SUCCESS;
    SQLHANDLE   hdbc; // connection handle
    SQLHANDLE   hstmt ; // statement handle
    char        structure_id[10];

    SQLCHAR * stmt;
    MMS::AtomSites *atomSitesP=0; // not to cross goto finish
    
    try {
        hdbc = connpool::cp.getConnection();
        } catch (DBException) {
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_sites()", "no connections!");
    }

    stmt = ( SQLCHAR * )
        "SELECT "
        "FRACT_TRANSF_MATRIX11, FRACT_TRANSF_MATRIX12, FRACT_TRANSF_MATRIX13, "
        "FRACT_TRANSF_MATRIX21, FRACT_TRANSF_MATRIX22, FRACT_TRANSF_MATRIX23, "
        "FRACT_TRANSF_MATRIX31, FRACT_TRANSF_MATRIX32, FRACT_TRANSF_MATRIX33, "
        "FRACT_TRANSF_VECTOR1,  FRACT_TRANSF_VECTOR2,  FRACT_TRANSF_VECTOR3 "
        "FROM DB2INST1.ATOM_SITES WHERE structure_id = ?";

    sqlrc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_sites()", "SQLAllocHandle");

    sqlrc = SQLPrepare(hstmt, stmt, SQL_NTS);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_sites()", "SQLPrepare");

    sqlrc = SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, 10, 0, structure_id, 10, NULL);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_sites()", "SQLBindParameter");

    strcpy( structure_id, (char*)_idV);
    sqlrc = SQLExecute(hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_sites()", "SQLExecute");

    struct {
        SQLINTEGER ind;
        SQLDOUBLE  val;
    } FRACT_TRANSF_MATRIX11;

    struct {
        SQLINTEGER ind;
        SQLDOUBLE  val;
    } FRACT_TRANSF_MATRIX12;

    struct {
        SQLINTEGER ind;
        SQLDOUBLE  val;
    } FRACT_TRANSF_MATRIX13;

    struct {
        SQLINTEGER ind;
        SQLDOUBLE  val;
    } FRACT_TRANSF_MATRIX21;

    struct {
        SQLINTEGER ind;
        SQLDOUBLE  val;
    } FRACT_TRANSF_MATRIX22;

    struct {
        SQLINTEGER ind;
        SQLDOUBLE  val;
    } FRACT_TRANSF_MATRIX23;

    struct {
        SQLINTEGER ind;
        SQLDOUBLE  val;
    } FRACT_TRANSF_MATRIX31;

    struct {
        SQLINTEGER ind;
        SQLDOUBLE  val;
    } FRACT_TRANSF_MATRIX32;

    struct {
        SQLINTEGER ind;
        SQLDOUBLE  val;
    } FRACT_TRANSF_MATRIX33;

    struct {
        SQLINTEGER ind;
        SQLDOUBLE  val;
    } FRACT_TRANSF_VECTOR1;

    struct {
        SQLINTEGER ind;
        SQLDOUBLE  val;
    } FRACT_TRANSF_VECTOR2;

    struct {
        SQLINTEGER ind;
        SQLDOUBLE  val;
    } FRACT_TRANSF_VECTOR3;

    sqlrc = SQLBindCol( hstmt, 1, SQL_C_DOUBLE, &FRACT_TRANSF_MATRIX11.val, 0, &FRACT_TRANSF_MATRIX11.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_sites()", "SQLBindCol 1");

    sqlrc = SQLBindCol( hstmt, 2, SQL_C_DOUBLE, &FRACT_TRANSF_MATRIX12.val, 0, &FRACT_TRANSF_MATRIX12.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_sites()", "SQLBindCol 2");

    sqlrc = SQLBindCol( hstmt, 3, SQL_C_DOUBLE, &FRACT_TRANSF_MATRIX13.val, 0, &FRACT_TRANSF_MATRIX13.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_sites()", "SQLBindCol 3");

    sqlrc = SQLBindCol( hstmt, 4, SQL_C_DOUBLE, &FRACT_TRANSF_MATRIX21.val, 0, &FRACT_TRANSF_MATRIX21.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_sites()", "SQLBindCol 4");

    sqlrc = SQLBindCol( hstmt, 5, SQL_C_DOUBLE, &FRACT_TRANSF_MATRIX22.val, 0, &FRACT_TRANSF_MATRIX22.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_sites()", "SQLBindCol 5");

    sqlrc = SQLBindCol( hstmt, 6, SQL_C_DOUBLE, &FRACT_TRANSF_MATRIX23.val, 0, &FRACT_TRANSF_MATRIX23.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_sites()", "SQLBindCol 6");

    sqlrc = SQLBindCol( hstmt, 7, SQL_C_DOUBLE, &FRACT_TRANSF_MATRIX31.val, 0, &FRACT_TRANSF_MATRIX31.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_sites()", "SQLBindCol 7");

    sqlrc = SQLBindCol( hstmt, 8, SQL_C_DOUBLE, &FRACT_TRANSF_MATRIX32.val, 0, &FRACT_TRANSF_MATRIX32.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_sites()", "SQLBindCol 8");

    sqlrc = SQLBindCol( hstmt, 9, SQL_C_DOUBLE, &FRACT_TRANSF_MATRIX33.val, 0, &FRACT_TRANSF_MATRIX33.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_sites()", "SQLBindCol 9");

    sqlrc = SQLBindCol( hstmt, 10, SQL_C_DOUBLE, &FRACT_TRANSF_VECTOR1.val, 0, &FRACT_TRANSF_VECTOR1.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_sites()", "SQLBindCol 10");

    sqlrc = SQLBindCol( hstmt, 11, SQL_C_DOUBLE, &FRACT_TRANSF_VECTOR2.val, 0, &FRACT_TRANSF_VECTOR2.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_sites()", "SQLBindCol 11");

    sqlrc = SQLBindCol( hstmt, 12, SQL_C_DOUBLE, &FRACT_TRANSF_VECTOR3.val, 0, &FRACT_TRANSF_VECTOR3.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_sites()", "SQLBindCol 12");

    sqlrc = SQLFetch(hstmt);
    if (sqlrc == SQL_NO_DATA_FOUND) goto finish;

    atomSitesP = new MMS::AtomSites();

    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_sites()", "SQLFetch");

    atomSitesP->fract_transf_matrix[0][0] = FRACT_TRANSF_MATRIX11.val;
    atomSitesP->fract_transf_matrix[0][1] = FRACT_TRANSF_MATRIX12.val;
    atomSitesP->fract_transf_matrix[0][2] = FRACT_TRANSF_MATRIX13.val;
    atomSitesP->fract_transf_matrix[1][0] = FRACT_TRANSF_MATRIX21.val;
    atomSitesP->fract_transf_matrix[1][1] = FRACT_TRANSF_MATRIX22.val;
    atomSitesP->fract_transf_matrix[1][2] = FRACT_TRANSF_MATRIX23.val;
    atomSitesP->fract_transf_matrix[2][0] = FRACT_TRANSF_MATRIX31.val;
    atomSitesP->fract_transf_matrix[2][1] = FRACT_TRANSF_MATRIX32.val;
    atomSitesP->fract_transf_matrix[2][2] = FRACT_TRANSF_MATRIX33.val;
    atomSitesP->fract_transf_vector[0] = FRACT_TRANSF_VECTOR1.val;
    atomSitesP->fract_transf_vector[1] = FRACT_TRANSF_VECTOR2.val;
    atomSitesP->fract_transf_vector[2] = FRACT_TRANSF_VECTOR3.val;

#if 0
     cerr << "server: transf. matrix:" << endl;
     for (int i = 0; i < 3; ++i) {
       for (int j = 0; j < 3; ++j)
 	cerr << '\t' << atomSitesP->fract_transf_matrix[i][j];
       cerr << endl;
     }
#endif

    sqlrc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_atom_sites()", "SQLFreeHandle");

finish:
    connpool::cp.returnConnection(hdbc);
    return *atomSitesP;
}
#endif


// we'll assume one cell per structure for now
#ifdef CELL_LIST
CORBA::Long
MMS_EntryImpl::cell_list_size()
    throw (CORBA::SystemException, MMS::DataAccessException);
{
    SQLRETURN   sqlrc = SQL_SUCCESS;
    SQLHANDLE   hdbc; // connection handle
    SQLHANDLE   hstmt ; // statement handle
    char        structure_id[10];

    SQLCHAR * stmt = ( SQLCHAR * )
        "SELECT count(*) FROM DB2INST1.CELL1 WHERE structure_id = ?";

    try {
        hdbc = connpool::cp.getConnection();
        } catch (DBException) {
        throw MMS::DataAccessException("MMS_EntryImpl::cell_list_size()", "no connections!");
    }

    sqlrc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::cell_list_size()", "SQLAllocHandle");

    sqlrc = SQLPrepare(hstmt, stmt, SQL_NTS);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::cell_list_size()", "SQLPrepare");

    sqlrc = SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, 10, 0, structure_id, 10, NULL);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::cell_list_size()", "SQLBindParameter");

    strcpy( structure_id, (char*)_idV);
    sqlrc = SQLExecute(hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::cell_list_size()", "SQLExecute");

    struct {
        SQLINTEGER ind;
        SQLINTEGER val;
    } total;

    sqlrc = SQLBindCol(hstmt, 1, SQL_C_LONG, &total.val, 0, &total.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::cell_list_size()", "SQLBindCol");

    sqlrc = SQLFetch(hstmt);
    if (sqlrc != SQL_SUCCESS && sqlrc != SQL_NO_DATA)
        throw MMS::DataAccessException("MMS_EntryImpl::cell_list_size()", "SQLFetch");

    if (sqlrc == SQL_NO_DATA_FOUND)
        throw MMS::DataAccessException("MMS_EntryImpl::cell_list_size()", "no data");

    sqlrc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::cell_list_size()", "SQLFreeHandle");

finish:
    connpool::cp.returnConnection(hdbc);
    return total.val;
}
#else
// single cell per structure
MMS::Cell
MMS_EntryImpl::get_cell()
    throw (CORBA::SystemException, MMS::DataAccessException)
{
    SQLRETURN   sqlrc = SQL_SUCCESS;
    SQLHANDLE   hdbc; // connection handle
    SQLHANDLE   hstmt ; // statement handle
    char        structure_id[10];

    SQLCHAR * stmt;
    MMS::Cell *cellP=0; // not to cross goto finish
    
    try {
        hdbc = connpool::cp.getConnection();
        } catch (DBException) {
        throw MMS::DataAccessException("MMS_EntryImpl::get_cell()", "no connections!");
    }

    stmt = ( SQLCHAR * )
      "SELECT ANGLE_ALPHA, ANGLE_BETA, ANGLE_GAMMA, LENGTH_A, LENGTH_B, LENGTH_C " // trailing space important!
        "FROM DB2INST1.CELL1 WHERE structure_id = ?";

    sqlrc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_cell()", "SQLAllocHandle");

    sqlrc = SQLPrepare(hstmt, stmt, SQL_NTS);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_cell()", "SQLPrepare");

    sqlrc = SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, 10, 0, structure_id, 10, NULL);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_cell()", "SQLBindParameter");

    strcpy( structure_id, (char*)_idV);
    sqlrc = SQLExecute(hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_cell()", "SQLExecute");

    struct {
        SQLINTEGER ind;
        SQLDOUBLE  val;
    } ANGLE_ALPHA;

    struct {
        SQLINTEGER ind;
        SQLDOUBLE  val;
    } ANGLE_BETA;

    struct {
        SQLINTEGER ind;
        SQLDOUBLE  val;
    } ANGLE_GAMMA;

    struct {
        SQLINTEGER ind;
        SQLDOUBLE  val;
    } LENGTH_A;

    struct {
        SQLINTEGER ind;
        SQLDOUBLE  val;
    } LENGTH_B;

    struct {
        SQLINTEGER ind;
        SQLDOUBLE  val;
    } LENGTH_C;

    sqlrc = SQLBindCol( hstmt, 1, SQL_C_DOUBLE, &ANGLE_ALPHA.val, 0, &ANGLE_ALPHA.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_cell()", "SQLBindCol 1");

    sqlrc = SQLBindCol( hstmt, 2, SQL_C_DOUBLE, &ANGLE_BETA.val,  0, &ANGLE_BETA.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_cell()", "SQLBindCol 2");

    sqlrc = SQLBindCol( hstmt, 3, SQL_C_DOUBLE, &ANGLE_GAMMA.val, 0, &ANGLE_GAMMA.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_cell()", "SQLBindCol 3");

    sqlrc = SQLBindCol( hstmt, 4, SQL_C_DOUBLE, &LENGTH_A.val, 0, &LENGTH_A.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_cell()", "SQLBindCol 4");

    sqlrc = SQLBindCol( hstmt, 5, SQL_C_DOUBLE, &LENGTH_B.val, 0, &LENGTH_B.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_cell()", "SQLBindCol 5");

    sqlrc = SQLBindCol( hstmt, 6, SQL_C_DOUBLE, &LENGTH_C.val, 0, &LENGTH_C.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_cell()", "SQLBindCol 6");

    sqlrc = SQLFetch(hstmt);
    if (sqlrc == SQL_NO_DATA_FOUND) goto finish;

    cellP = new MMS::Cell();

    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_cell", "SQLFetch");

    cellP->unit_cell.angle_alpha	= ANGLE_ALPHA.val;
    cellP->unit_cell.angle_beta		= ANGLE_BETA.val;
    cellP->unit_cell.angle_gamma	= ANGLE_GAMMA.val;
    cellP->unit_cell.length_a		= LENGTH_A.val;
    cellP->unit_cell.length_b		= LENGTH_B.val;
    cellP->unit_cell.length_c		= LENGTH_C.val;

    sqlrc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_cell()", "SQLFreeHandle");

finish:
    connpool::cp.returnConnection(hdbc);
    return *cellP;
}
#endif /* cell_list v. just one cell */


MMS::Symmetry*
MMS_EntryImpl::get_symmetry()
    throw (CORBA::SystemException, MMS::DataAccessException)
{
    SQLRETURN   sqlrc = SQL_SUCCESS;
    SQLHANDLE   hdbc; // connection handle
    SQLHANDLE   hstmt ; // statement handle
    char        structure_id[10];

    SQLCHAR * stmt;
    MMS::Symmetry *symmetryP = 0; // not to cross goto finish
    
    try {
        hdbc = connpool::cp.getConnection();
        } catch (DBException) {
        throw MMS::DataAccessException("MMS_EntryImpl::get_symmetry()", "no connections!");
    }

    stmt = ( SQLCHAR * )
      "SELECT SPACE_GROUP_NAME_H_M " // trailing space important!
        "FROM DB2INST1.SYMMETRY WHERE structure_id = ?";

    sqlrc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_symmetry()", "SQLAllocHandle");

    sqlrc = SQLPrepare(hstmt, stmt, SQL_NTS);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_symmetry()", "SQLPrepare");

    sqlrc = SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, 10, 0, structure_id, 10, NULL);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_symmetry()", "SQLBindParameter");

    strcpy( structure_id, (char*)_idV);
    sqlrc = SQLExecute(hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_symmetry()", "SQLExecute");

    struct {
        SQLINTEGER ind;
        SQLCHAR    val[80];
    } SPACE_GROUP_NAME_H_M;

    sqlrc = SQLBindCol( hstmt, 1, SQL_C_CHAR, &SPACE_GROUP_NAME_H_M.val, 80, &SPACE_GROUP_NAME_H_M.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_symmetry()", "SQLBindCol 1");

    sqlrc = SQLFetch(hstmt);
    if (sqlrc == SQL_NO_DATA_FOUND) goto finish;

    symmetryP = new MMS::Symmetry();

    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_symmetry", "SQLFetch");

    symmetryP->space_group_name_h_m = CORBA::string_dup((char*)SPACE_GROUP_NAME_H_M.val);

    sqlrc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::get_symmetry()", "SQLFreeHandle");

finish:
    connpool::cp.returnConnection(hdbc);
    return symmetryP;
}


// Biological Assembly

CORBA::Long
MMS_EntryImpl::biol_unit_list_size()
    throw (CORBA::SystemException, MMS::DataAccessException)
{
    SQLRETURN   sqlrc = SQL_SUCCESS;
    SQLHANDLE   hdbc; // connection handle
    SQLHANDLE   hstmt ; // statement handle
    char        structure_id[10];

    SQLCHAR * stmt = ( SQLCHAR * )
      //        "SELECT count(distinct biol_id) FROM DB2INST1.STRUCT_BIOL_GEN WHERE structure_id = ?";
      "SELECT count(*) FROM DB2INST1.STRUCT_BIOL WHERE structure_id = ?";
    
    try {
        hdbc = connpool::cp.getConnection();
        } catch (DBException) {
        throw MMS::DataAccessException("MMS_EntryImpl::biol_unit_list_size()", "no connections!");
    }

    sqlrc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::biol_unit_list_size()", "SQLAllocHandle");

    sqlrc = SQLPrepare(hstmt, stmt, SQL_NTS);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::biol_unit_list_size()", "SQLPrepare");

    sqlrc = SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, 10, 0, structure_id, 10, NULL);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::biol_unit_list_size()", "SQLBindParameter");

    strcpy( structure_id, (char*)_idV);
    sqlrc = SQLExecute(hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::biol_unit_list_size()", "SQLExecute");

    struct {
        SQLINTEGER ind;
        SQLINTEGER val;
    } total;

    sqlrc = SQLBindCol(hstmt, 1, SQL_C_LONG, &total.val, 0, &total.ind);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::biol_unit_list_size()", "SQLBindCol");

    sqlrc = SQLFetch(hstmt);
    if (sqlrc != SQL_SUCCESS && sqlrc != SQL_NO_DATA)
        throw MMS::DataAccessException("MMS_EntryImpl::biol_unit_list_size()", "SQLFetch");

    if (sqlrc == SQL_NO_DATA_FOUND)
        throw MMS::DataAccessException("MMS_EntryImpl::biol_unit_list_size()", "no data");

    sqlrc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw MMS::DataAccessException("MMS_EntryImpl::biol_unit_list_size()", "SQLFreeHandle");

finish:
    connpool::cp.returnConnection(hdbc);
    return total.val;
}


#undef BIOL_MMS_CONV
#ifdef BIOL_MMS_CONV
#include "mms_biol.h"

MMS::BiolUnit*
MMS_EntryImpl::get_biol_unit(short biol_id)
    throw (CORBA::SystemException, MMS::DataAccessException)
{
  // let's create asymmetric unit as accepted by the biol_unit method
  
  biol::AsymUnit asym_unit;
  
  MMS::AtomSiteList_var atomSiteListV = get_atom_site_list();
  MMS::AtomSiteExtList_var atomSiteExtListV = get_atom_site_ext_list();
  biol::AtomSiteList atom_site_list(atomSiteListV->length());
  for (int i = 0; i < atomSiteListV->length(); ++i) {
    atom_site_list[i].group_PDB = atomSiteExtListV[i].group_pdb;
    atom_site_list[i].cartn.x= atomSiteListV[i].cartn.x;
    atom_site_list[i].cartn.y= atomSiteListV[i].cartn.y;
    atom_site_list[i].cartn.z= atomSiteListV[i].cartn.z;
    atom_site_list[i].label.asym = atomSiteListV[i].label.asym.id;
    atom_site_list[i].label.comp = atomSiteListV[i].label.comp.id;
    atom_site_list[i].label.seq  = atomSiteListV[i].label.seq.id;
    atom_site_list[i].label.atom = atomSiteListV[i].label.atom.id;
    atom_site_list[i].auth.asym = atomSiteExtListV[i].auth_asym_id;
    atom_site_list[i].auth.comp = atomSiteExtListV[i].auth_comp_id;
    atom_site_list[i].auth.seq  = atomSiteExtListV[i].auth_seq_id;
    atom_site_list[i].auth.atom = atomSiteExtListV[i].auth_atom_id;
 }

  asym_unit.atom_site_list = atom_site_list;

  biol::SymInfo sym_info;
  MMS::Cell_var		cellV = get_cell();
  MMS::AtomSites_var	atomSitesV = get_atom_sites();
  MMS::Symmetry_var	symmetryV  = get_symmetry();
  if (cellV) {
    sym_info.cell.alpha = cellV->unit_cell.angle_alpha;
    sym_info.cell.beta  = cellV->unit_cell.angle_beta;
    sym_info.cell.gamma = cellV->unit_cell.angle_gamma;
    sym_info.cell.a     = cellV->unit_cell.length_a;
    sym_info.cell.b     = cellV->unit_cell.length_b;
    sym_info.cell.c     = cellV->unit_cell.length_c;
  }
  if (atomSitesV) {
    sym_info.transf.matrix = atomSitesV->fract_transf_matrix;
    sym_info.transf.vector = atomSitesV->fract_transf_vector;
  }
  if (symmetryV) {
    sym_info.space_group = symmetryV->space_group_name_h_m;
  }

  asym_unit.sym_info       = sym_info;

  biol::BiolGenList biol_gen_list;
  MMS::StructBiolGenList_var structBiolGenV = get_struct_biol_gen_list();
  
  for (unsigned i = 0; i < structBiolGenV->length(); ++i)
    if (structBiolGenV[i].biol.index == biol_id)
      biol_gen_list.push_back(biol::BiolGen(structBiolGenV[i].biol.index,
					    structBiolGenV[i].asym.id,
					    structBiolGenV[i].symmetry));

  cerr << "let's see, sir" << endl;

  biol::BiolUnit biol_unit = biol_unit::biol_gen(asym_unit, biol_gen_list);

  MMS::BiolUnit *rp = new MMS::BiolUnit();

  cerr << "let's proceed, sir" << endl;

  *rp = mms_biol::conv(biol_unit);

  cerr << "let's return, sir" << endl;
  
  return rp;
}
#else // BIOL_MMS_CONV


MMS::SymInfo*
MMS_EntryImpl::get_sym_info()
    throw (CORBA::SystemException, MMS::DataAccessException)
{
  // cerr << "entering MMS_EntryImpl::get_sym_info()" << endl;

  MMS::SymInfo *r = 0;

  // Omni won't allow Cell_var here, no assignment from non-var:
  MMS::Cell		cell = get_cell();
  // no match for !_var under Omni
  // if (!cellV) 
  //   throw(MMS::DataAccessException("MMS_EntryImpl::get_sym_info()", "empty cell"));
  MMS::AtomSites	atomSites = get_atom_sites();
  // if (!atomSitesV) 
  //   throw(MMS::DataAccessException("MMS_EntryImpl::get_sym_info()", "empty atom sites"));
  MMS::Symmetry_var	symmetryV  = get_symmetry();
  // if (!symmetryV) 
  //   throw(MMS::DataAccessException("MMS_EntryImpl::get_sym_info()", "empty symmetry"));

  r = new MMS::SymInfo();
  
  r->acell = cell;

 /*
    r->cell.unit_cell.length_a = cellV->unit_cell.length_a;
    r->cell.unit_cell.length_b = cellV->unit_cell.length_b;
    r->cell.unit_cell.length_c = cellV->unit_cell.length_c;
    r->cell.unit_cell.angle_alpha = cellV->unit_cell.angle_alpha;
    r->cell.unit_cell.angle_beta  = cellV->unit_cell.angle_beta;
    r->cell.unit_cell.angle_gamma = cellV->unit_cell.angle_gamma;
    */

    /*
    cerr << "get_sym_info cell"
	 << ", a = " << r->cell.unit_cell.length_a 
	 << ", b = " << r->cell.unit_cell.length_b 
	 << ", c = " << r->cell.unit_cell.length_c 
	 << ", alpha = " << r->cell.unit_cell.angle_alpha 
	 << ", beta = "  << r->cell.unit_cell.angle_beta
	 << ", gamma = " << r->cell.unit_cell.angle_gamma
	 << endl;
*/

// gcc 3.3: ISO C++ forbids array assignment
  // r->atransf.matrix = atomSitesV->fract_transf_matrix;
  // r->atransf.vector = atomSitesV->fract_transf_vector;
  for (unsigned i=0; i<3; ++i) {
	r->atransf.vector[i] = atomSites.fract_transf_vector[i];
  	for (unsigned j=0; j<3; ++j)
		r->atransf.matrix[i][j] = atomSites.fract_transf_matrix[i][j];
  }

  r->space_group = CORBA::string_dup(symmetryV->space_group_name_h_m);
  
  // cerr << "MMS_EntryImpl::get_sym_info(): got sym_info" << endl;
  return r;
}


MMS::AsymUnit*
MMS_EntryImpl::get_asym_unit()
    throw (CORBA::SystemException, MMS::DataAccessException)
{
  // cerr << "entering MMS_EntryImpl::get_asym_unit()" << endl;
  
  MMS::AsymUnit *r = 0;

  MMS::SymInfo_var symInfoV = get_sym_info();
  // Omni has no use for !_var:
  // if (!symInfoV)
  //   throw (MMS::DataAccessException("MMS_EntryImpl::get_asym_unit(): empty sym_info", _idV));

  MMS::AtomSiteList_var atomSiteListV = get_atom_site_list();
  // if (!atomSiteListV)
  //   throw (MMS::DataAccessException("MMS_EntryImpl::get_asym_unit(): empty atom_site_list", _idV));

  r = new MMS::AsymUnit();

  r->sym_info = symInfoV;
  r->atom_site_list = atomSiteListV;

  return r;
}


MMS::BiolUnit*
MMS_EntryImpl::get_biol_unit(short biol_id)
    throw (CORBA::SystemException, MMS::DataAccessException)
{
  MMS::BiolUnit *r = 0;

  MMS::AsymUnit_var asymUnitV = get_asym_unit();

  MMS::StructBiolGenList_var structBiolGenListV = get_struct_biol_gen_list();

  // separate out the generators for the biol_id at hand
  MMS::StructBiolGenList bg;
  long len = 0;
  
  for (long i = 0; i < structBiolGenListV->length(); ++i)
    if (structBiolGenListV[i].biol.index == biol_id) {
      bg.length(len+1);
      bg[len++] = structBiolGenListV[i];
    }

  r = new MMS::BiolUnit();
  *r = mms_biol_unit::biol_gen(asymUnitV.in(), bg);

  return r;
}
#endif // BIOL_MMS_CONV
