// buil_unit interface
// $Id: biol.h,v 1.4 2003/05/04 18:27:35 alexy Exp $

#ifndef _BIOL_H_
#define _BIOL_H_

#include <vector>
#include <string>
using namespace std;

namespace biol {

  struct VectorXYZ {
    VectorXYZ();
    virtual ~VectorXYZ();
    float x;
    float y;
    float z;
    void show() const;
  };

  struct AtomIndex {
    AtomIndex();
    virtual ~AtomIndex();
    string asym;
    string comp;
    string seq;
    string atom;
//    string alt;
    void show() const;
  };
  
  struct AtomSite {
    long id;
    AtomSite();
    virtual ~AtomSite();
    VectorXYZ cartn;
    string group_PDB;
    AtomIndex auth;
    AtomIndex label;
    void show() const;
  };
  
  typedef vector<AtomSite> AtomSiteList;

  struct Symop {
    Symop();
    Symop(const string);
    virtual ~Symop();
    string name;
    int op;
    int trans_x;
    int trans_y;
    int trans_z;
  };

  struct BiolOps {
    BiolOps();
    virtual ~BiolOps();
    int            biol_id;
    vector<string> asym_ids;
    Symop          symop;
  };
  typedef vector<BiolOps> BiolOpsList;

  struct BiolGen {
    BiolGen(const int biol_id, const char* asym_id, const char* symop);
    virtual ~BiolGen();
    int		biol_id;
    string 	asym_id;
    Symop  	symop;
  };
  typedef vector<BiolGen> BiolGenList;

  struct AsymUnitReplica {
    AsymUnitReplica();
    virtual ~AsymUnitReplica();
    string		asym_id;
    string		symop;
    AtomSiteList	atom_site_list;
  };
  typedef vector<AsymUnitReplica> AsymUnitReplicaList;

  struct Cell {
    Cell();
    ~Cell();
    float alpha;
    float beta;
    float gamma;
    float a;
    float b;
    float c;
  };

  struct Transf {
    Transf();
    virtual ~Transf();
    float matrix[3][3];
    float vector[3];
  };

  struct SymInfo {
    SymInfo();
    virtual ~SymInfo();
    string		space_group;
    Cell		cell;
    Transf		transf;
  };

  struct AsymUnit {
    AsymUnit();
    ~AsymUnit();
    SymInfo		sym_info;
    AtomSiteList	atom_site_list;
  };
    
  struct BiolUnit {
    BiolUnit();
    virtual ~BiolUnit();
    SymInfo		sym_info;
    AsymUnitReplicaList	replicas;
  };
  typedef vector<BiolUnit> BiolUnitList;

  

/* --
   another way to do it is to have
   a single class Unit instead of
   both AsymUnit and BiolUnit:
   
  union UberCoords {
    AtomSiteList	atom_site_list;
    AsymUnitReplicas	replicas;
  }

  struct Unit {
    Spacial		spacial;
    UberCoords		coords;
  };
-- */
  
} // namespace biol
#endif
