// Braver Entry Factory
// Client Prologue
// $Id: client.C,v 1.11 2003/03/09 17:59:28 alexy Exp $

#include <iostream>
#include <iomanip>
#include <fstream>
#include <cstdlib>
#include <sstream>
#include <sys/time.h> // gettimeofday

#include "C_someC.h"
#include "C_subentryC.h"
#include "C_PropertyService.h"
#include "C_NamingService.h"
#include "C_ExceptionStream.h"

// #include "../Flags.h"

#include "show.h"

CORBA::ORB_var 
global_orb = CORBA::ORB::_nil();

double delta(struct timeval tv_beg, struct timeval tv_end);

int main(int argc, char **argv)
{
    try
    {
        CORBA::Object_var objV;  // For temporary object references.

        // Initialise the ORB.
        global_orb = CORBA::ORB_init(argc, argv);

	char *entry_name;

        // Parse command line arguments:
        if (argc > 2) {
	  cout << "Usage: " << argv[0] << " [<entry_id>]" << endl;
	  exit(0);
        }
	else if (argc == 2)
	  entry_name = argv[1];
	else
	  entry_name = (char*)"PD0264"; // pdb_id: 1k82, 4 biol. units.


        // Obtain Naming Service reference.
        CosNaming::NamingContext_var rootContextV;
        objV = global_orb->resolve_initial_references("NameService");
        rootContextV = CosNaming::NamingContext::_narrow(objV);
        if (CORBA::is_nil(rootContextV.in()) ) {
            cerr << "error: failed to narrow root naming context." << endl;
            exit(1);
        }

        // Obtain the collection object reference.
        CosNaming::Name objectName(2);
        objectName.length(2);
        objectName[0].id   = CORBA::string_dup("EntryFactory");
        objectName[0].kind = CORBA::string_dup("");
        objectName[1].id   = CORBA::string_dup("HandMade");
        objectName[1].kind = CORBA::string_dup("");

        objV = rootContextV->resolve(objectName);

	MMS::EntryFactory_var factoryV = 
	  MMS::EntryFactory::_narrow(objV);
	if (CORBA::is_nil(factoryV.in()))
	  {
	    cerr << "error: failed to narrow to MMS::EntryFactory." << endl;
	    exit(1);
	  }

        // Invoke 'MMS::EntryFactory' object.
        cout << "Details of entry factory:" << endl;
        CORBA::String_var name_strV;
        name_strV = factoryV->name();
        cout << "\tName of factory = \"" << name_strV.in() << "\"" << endl;

        long n_entries = factoryV->get_entry_id_list_size();
        cout << "\tNumber of entries = " << n_entries << endl;

	cout << "We have the following entries:" << endl;

	MMS::EntryIdList_var entryIdList = factoryV->get_entry_id_list();
	cout << ' ' << entryIdList->length() << " total" << endl;


	struct timeval  tv_zero, tv_beg, tv_end;
	struct timezone tz;

	gettimeofday(&tv_zero, &tz);
	
	for (unsigned i = 0; i < entryIdList->length(); ++i) {
	  cout << ' ' << entryIdList[i] << endl;

	  // Start timing
	  gettimeofday(&tv_beg, &tz);
	  
	  MMS::Entry_var entryV = factoryV->get_entry_from_id(entryIdList[i]); // (entry_name)

            // client call for category atom_site:
            CORBA::Long atomSiteListSize = entryV->atom_site_list_size();
            cout << "atom_site has " << atomSiteListSize << " elements (actually ";

            MMS::AtomSiteList_var atomSiteListV = entryV->get_atom_site_list();
            cout << atomSiteListV->length() << ')' << endl;


            // client call for category atom_site_anisotrop:
            CORBA::Long atomSiteAnisotropListSize = entryV->atom_site_anisotrop_list_size();
            cout << "atom_site_anisotrop has " << atomSiteAnisotropListSize << " elements (actually ";

            MMS::AtomSiteAnisotropList_var atomSiteAnisotropListV = entryV->get_atom_site_anisotrop_list();
            cout << atomSiteAnisotropListV->length() << ')' << endl;


            // client call for category atom_site_ext:
            CORBA::Long atomSiteExtListSize = entryV->atom_site_ext_list_size();
            cout << "atom_site_ext has " << atomSiteExtListSize << " elements (actually ";

            MMS::AtomSiteExtList_var atomSiteExtListV = entryV->get_atom_site_ext_list();
            cout << atomSiteExtListV->length() << ')' << endl;


            // client call for category atom_type:
            CORBA::Long atomTypeListSize = entryV->atom_type_list_size();
            cout << "atom_type has " << atomTypeListSize << " elements (actually ";

            MMS::AtomTypeList_var atomTypeListV = entryV->get_atom_type_list();
            cout << atomTypeListV->length() << ')' << endl;


            // client call for category chem_comp:
            CORBA::Long chemCompListSize = entryV->chem_comp_list_size();
            cout << "chem_comp has " << chemCompListSize << " elements (actually ";

            MMS::ChemCompList_var chemCompListV = entryV->get_chem_comp_list();
            cout << chemCompListV->length() << ')' << endl;


            // client call for category chem_comp_angle:
            CORBA::Long chemCompAngleListSize = entryV->chem_comp_angle_list_size();
            cout << "chem_comp_angle has " << chemCompAngleListSize << " elements (actually ";

            MMS::ChemCompAngleList_var chemCompAngleListV = entryV->get_chem_comp_angle_list();
            cout << chemCompAngleListV->length() << ')' << endl;


            // client call for category chem_comp_atom:
            CORBA::Long chemCompAtomListSize = entryV->chem_comp_atom_list_size();
            cout << "chem_comp_atom has " << chemCompAtomListSize << " elements (actually ";

            MMS::ChemCompAtomList_var chemCompAtomListV = entryV->get_chem_comp_atom_list();
            cout << chemCompAtomListV->length() << ')' << endl;


            // client call for category chem_comp_bond:
            CORBA::Long chemCompBondListSize = entryV->chem_comp_bond_list_size();
            cout << "chem_comp_bond has " << chemCompBondListSize << " elements (actually ";

            MMS::ChemCompBondList_var chemCompBondListV = entryV->get_chem_comp_bond_list();
            cout << chemCompBondListV->length() << ')' << endl;


            // client call for category chem_comp_chir:
            CORBA::Long chemCompChirListSize = entryV->chem_comp_chir_list_size();
            cout << "chem_comp_chir has " << chemCompChirListSize << " elements (actually ";

            MMS::ChemCompChirList_var chemCompChirListV = entryV->get_chem_comp_chir_list();
            cout << chemCompChirListV->length() << ')' << endl;


            // client call for category chem_comp_chir_atom:
            CORBA::Long chemCompChirAtomListSize = entryV->chem_comp_chir_atom_list_size();
            cout << "chem_comp_chir_atom has " << chemCompChirAtomListSize << " elements (actually ";

            MMS::ChemCompChirAtomList_var chemCompChirAtomListV = entryV->get_chem_comp_chir_atom_list();
            cout << chemCompChirAtomListV->length() << ')' << endl;


            // client call for category chem_comp_link:
            CORBA::Long chemCompLinkListSize = entryV->chem_comp_link_list_size();
            cout << "chem_comp_link has " << chemCompLinkListSize << " elements (actually ";

            MMS::ChemCompLinkList_var chemCompLinkListV = entryV->get_chem_comp_link_list();
            cout << chemCompLinkListV->length() << ')' << endl;


            // client call for category chem_comp_plane:
            CORBA::Long chemCompPlaneListSize = entryV->chem_comp_plane_list_size();
            cout << "chem_comp_plane has " << chemCompPlaneListSize << " elements (actually ";

            MMS::ChemCompPlaneList_var chemCompPlaneListV = entryV->get_chem_comp_plane_list();
            cout << chemCompPlaneListV->length() << ')' << endl;


            // client call for category chem_comp_plane_atom:
            CORBA::Long chemCompPlaneAtomListSize = entryV->chem_comp_plane_atom_list_size();
            cout << "chem_comp_plane_atom has " << chemCompPlaneAtomListSize << " elements (actually ";

            MMS::ChemCompPlaneAtomList_var chemCompPlaneAtomListV = entryV->get_chem_comp_plane_atom_list();
            cout << chemCompPlaneAtomListV->length() << ')' << endl;


            // client call for category chem_comp_tor:
            CORBA::Long chemCompTorListSize = entryV->chem_comp_tor_list_size();
            cout << "chem_comp_tor has " << chemCompTorListSize << " elements (actually ";

            MMS::ChemCompTorList_var chemCompTorListV = entryV->get_chem_comp_tor_list();
            cout << chemCompTorListV->length() << ')' << endl;


            // client call for category chem_comp_tor_value:
            CORBA::Long chemCompTorValueListSize = entryV->chem_comp_tor_value_list_size();
            cout << "chem_comp_tor_value has " << chemCompTorValueListSize << " elements (actually ";

            MMS::ChemCompTorValueList_var chemCompTorValueListV = entryV->get_chem_comp_tor_value_list();
            cout << chemCompTorValueListV->length() << ')' << endl;


            // client call for category chem_link:
            CORBA::Long chemLinkListSize = entryV->chem_link_list_size();
            cout << "chem_link has " << chemLinkListSize << " elements (actually ";

            MMS::ChemLinkList_var chemLinkListV = entryV->get_chem_link_list();
            cout << chemLinkListV->length() << ')' << endl;


            // client call for category chem_link_angle:
            CORBA::Long chemLinkAngleListSize = entryV->chem_link_angle_list_size();
            cout << "chem_link_angle has " << chemLinkAngleListSize << " elements (actually ";

            MMS::ChemLinkAngleList_var chemLinkAngleListV = entryV->get_chem_link_angle_list();
            cout << chemLinkAngleListV->length() << ')' << endl;


            // client call for category chem_link_bond:
            CORBA::Long chemLinkBondListSize = entryV->chem_link_bond_list_size();
            cout << "chem_link_bond has " << chemLinkBondListSize << " elements (actually ";

            MMS::ChemLinkBondList_var chemLinkBondListV = entryV->get_chem_link_bond_list();
            cout << chemLinkBondListV->length() << ')' << endl;


            // client call for category chem_link_chir:
            CORBA::Long chemLinkChirListSize = entryV->chem_link_chir_list_size();
            cout << "chem_link_chir has " << chemLinkChirListSize << " elements (actually ";

            MMS::ChemLinkChirList_var chemLinkChirListV = entryV->get_chem_link_chir_list();
            cout << chemLinkChirListV->length() << ')' << endl;


            // client call for category chem_link_chir_atom:
            CORBA::Long chemLinkChirAtomListSize = entryV->chem_link_chir_atom_list_size();
            cout << "chem_link_chir_atom has " << chemLinkChirAtomListSize << " elements (actually ";

            MMS::ChemLinkChirAtomList_var chemLinkChirAtomListV = entryV->get_chem_link_chir_atom_list();
            cout << chemLinkChirAtomListV->length() << ')' << endl;


            // client call for category chem_link_plane:
            CORBA::Long chemLinkPlaneListSize = entryV->chem_link_plane_list_size();
            cout << "chem_link_plane has " << chemLinkPlaneListSize << " elements (actually ";

            MMS::ChemLinkPlaneList_var chemLinkPlaneListV = entryV->get_chem_link_plane_list();
            cout << chemLinkPlaneListV->length() << ')' << endl;


            // client call for category chem_link_plane_atom:
            CORBA::Long chemLinkPlaneAtomListSize = entryV->chem_link_plane_atom_list_size();
            cout << "chem_link_plane_atom has " << chemLinkPlaneAtomListSize << " elements (actually ";

            MMS::ChemLinkPlaneAtomList_var chemLinkPlaneAtomListV = entryV->get_chem_link_plane_atom_list();
            cout << chemLinkPlaneAtomListV->length() << ')' << endl;


            // client call for category chem_link_tor:
            CORBA::Long chemLinkTorListSize = entryV->chem_link_tor_list_size();
            cout << "chem_link_tor has " << chemLinkTorListSize << " elements (actually ";

            MMS::ChemLinkTorList_var chemLinkTorListV = entryV->get_chem_link_tor_list();
            cout << chemLinkTorListV->length() << ')' << endl;


            // client call for category chem_link_tor_value:
            CORBA::Long chemLinkTorValueListSize = entryV->chem_link_tor_value_list_size();
            cout << "chem_link_tor_value has " << chemLinkTorValueListSize << " elements (actually ";

            MMS::ChemLinkTorValueList_var chemLinkTorValueListV = entryV->get_chem_link_tor_value_list();
            cout << chemLinkTorValueListV->length() << ')' << endl;


            // client call for category entity:
            CORBA::Long entityListSize = entryV->entity_list_size();
            cout << "entity has " << entityListSize << " elements (actually ";

            MMS::EntityList_var entityListV = entryV->get_entity_list();
            cout << entityListV->length() << ')' << endl;


            // client call for category entity_keywords:
            CORBA::Long entityKeywordsListSize = entryV->entity_keywords_list_size();
            cout << "entity_keywords has " << entityKeywordsListSize << " elements (actually ";

            MMS::EntityKeywordsList_var entityKeywordsListV = entryV->get_entity_keywords_list();
            cout << entityKeywordsListV->length() << ')' << endl;


            // client call for category entity_link:
            CORBA::Long entityLinkListSize = entryV->entity_link_list_size();
            cout << "entity_link has " << entityLinkListSize << " elements (actually ";

            MMS::EntityLinkList_var entityLinkListV = entryV->get_entity_link_list();
            cout << entityLinkListV->length() << ')' << endl;


            // client call for category entity_name_com:
            CORBA::Long entityNameComListSize = entryV->entity_name_com_list_size();
            cout << "entity_name_com has " << entityNameComListSize << " elements (actually ";

            MMS::EntityNameComList_var entityNameComListV = entryV->get_entity_name_com_list();
            cout << entityNameComListV->length() << ')' << endl;


            // client call for category entity_name_sys:
            CORBA::Long entityNameSysListSize = entryV->entity_name_sys_list_size();
            cout << "entity_name_sys has " << entityNameSysListSize << " elements (actually ";

            MMS::EntityNameSysList_var entityNameSysListV = entryV->get_entity_name_sys_list();
            cout << entityNameSysListV->length() << ')' << endl;


            // client call for category entity_poly:
            CORBA::Long entityPolyListSize = entryV->entity_poly_list_size();
            cout << "entity_poly has " << entityPolyListSize << " elements (actually ";

            MMS::EntityPolyList_var entityPolyListV = entryV->get_entity_poly_list();
            cout << entityPolyListV->length() << ')' << endl;


            // client call for category entity_src_gen:
            CORBA::Long entitySrcGenListSize = entryV->entity_src_gen_list_size();
            cout << "entity_src_gen has " << entitySrcGenListSize << " elements (actually ";

            MMS::EntitySrcGenList_var entitySrcGenListV = entryV->get_entity_src_gen_list();
            cout << entitySrcGenListV->length() << ')' << endl;


            // client call for category entity_src_nat:
            CORBA::Long entitySrcNatListSize = entryV->entity_src_nat_list_size();
            cout << "entity_src_nat has " << entitySrcNatListSize << " elements (actually ";

            MMS::EntitySrcNatList_var entitySrcNatListV = entryV->get_entity_src_nat_list();
            cout << entitySrcNatListV->length() << ')' << endl;


            // client call for category entry_link:
            CORBA::Long entryLinkListSize = entryV->entry_link_list_size();
            cout << "entry_link has " << entryLinkListSize << " elements (actually ";

            MMS::EntryLinkList_var entryLinkListV = entryV->get_entry_link_list();
            cout << entryLinkListV->length() << ')' << endl;


            // client call for category geom:
            CORBA::Long geomListSize = entryV->geom_list_size();
            cout << "geom has " << geomListSize << " elements (actually ";

            MMS::GeomList_var geomListV = entryV->get_geom_list();
            cout << geomListV->length() << ')' << endl;


            // client call for category geom_angle:
            CORBA::Long geomAngleListSize = entryV->geom_angle_list_size();
            cout << "geom_angle has " << geomAngleListSize << " elements (actually ";

            MMS::GeomAngleList_var geomAngleListV = entryV->get_geom_angle_list();
            cout << geomAngleListV->length() << ')' << endl;


            // client call for category geom_bond:
            CORBA::Long geomBondListSize = entryV->geom_bond_list_size();
            cout << "geom_bond has " << geomBondListSize << " elements (actually ";

            MMS::GeomBondList_var geomBondListV = entryV->get_geom_bond_list();
            cout << geomBondListV->length() << ')' << endl;


            // client call for category geom_contact:
            CORBA::Long geomContactListSize = entryV->geom_contact_list_size();
            cout << "geom_contact has " << geomContactListSize << " elements (actually ";

            MMS::GeomContactList_var geomContactListV = entryV->get_geom_contact_list();
            cout << geomContactListV->length() << ')' << endl;


            // client call for category geom_hbond:
            CORBA::Long geomHbondListSize = entryV->geom_hbond_list_size();
            cout << "geom_hbond has " << geomHbondListSize << " elements (actually ";

            MMS::GeomHbondList_var geomHbondListV = entryV->get_geom_hbond_list();
            cout << geomHbondListV->length() << ')' << endl;


            // client call for category geom_torsion:
            CORBA::Long geomTorsionListSize = entryV->geom_torsion_list_size();
            cout << "geom_torsion has " << geomTorsionListSize << " elements (actually ";

            MMS::GeomTorsionList_var geomTorsionListV = entryV->get_geom_torsion_list();
            cout << geomTorsionListV->length() << ')' << endl;


            // client call for category struct_asym:
            CORBA::Long structAsymListSize = entryV->struct_asym_list_size();
            cout << "struct_asym has " << structAsymListSize << " elements (actually ";

            MMS::StructAsymList_var structAsymListV = entryV->get_struct_asym_list();
            cout << structAsymListV->length() << ')' << endl;


            // client call for category struct_biol:
            CORBA::Long structBiolListSize = entryV->struct_biol_list_size();
            cout << "struct_biol has " << structBiolListSize << " elements (actually ";

            MMS::StructBiolList_var structBiolListV = entryV->get_struct_biol_list();
            cout << structBiolListV->length() << ')' << endl;


            // client call for category struct_biol_gen:
            CORBA::Long structBiolGenListSize = entryV->struct_biol_gen_list_size();
            cout << "struct_biol_gen has " << structBiolGenListSize << " elements (actually ";

            MMS::StructBiolGenList_var structBiolGenListV = entryV->get_struct_biol_gen_list();
            cout << structBiolGenListV->length() << ')' << endl;


            // client call for category struct_biol_keywords:
            CORBA::Long structBiolKeywordsListSize = entryV->struct_biol_keywords_list_size();
            cout << "struct_biol_keywords has " << structBiolKeywordsListSize << " elements (actually ";

            MMS::StructBiolKeywordsList_var structBiolKeywordsListV = entryV->get_struct_biol_keywords_list();
            cout << structBiolKeywordsListV->length() << ')' << endl;


            // client call for category struct_biol_view:
            CORBA::Long structBiolViewListSize = entryV->struct_biol_view_list_size();
            cout << "struct_biol_view has " << structBiolViewListSize << " elements (actually ";

            MMS::StructBiolViewList_var structBiolViewListV = entryV->get_struct_biol_view_list();
            cout << structBiolViewListV->length() << ')' << endl;


            // client call for category struct_conf:
            CORBA::Long structConfListSize = entryV->struct_conf_list_size();
            cout << "struct_conf has " << structConfListSize << " elements (actually ";

            MMS::StructConfList_var structConfListV = entryV->get_struct_conf_list();
            cout << structConfListV->length() << ')' << endl;


            // client call for category struct_conf_type:
            CORBA::Long structConfTypeListSize = entryV->struct_conf_type_list_size();
            cout << "struct_conf_type has " << structConfTypeListSize << " elements (actually ";

            MMS::StructConfTypeList_var structConfTypeListV = entryV->get_struct_conf_type_list();
            cout << structConfTypeListV->length() << ')' << endl;


            // client call for category struct_conn_type:
            CORBA::Long structConnTypeListSize = entryV->struct_conn_type_list_size();
            cout << "struct_conn_type has " << structConnTypeListSize << " elements (actually ";

            MMS::StructConnTypeList_var structConnTypeListV = entryV->get_struct_conn_type_list();
            cout << structConnTypeListV->length() << ')' << endl;


            // client call for category struct_keywords:
            CORBA::Long structKeywordsListSize = entryV->struct_keywords_list_size();
            cout << "struct_keywords has " << structKeywordsListSize << " elements (actually ";

            MMS::StructKeywordsList_var structKeywordsListV = entryV->get_struct_keywords_list();
            cout << structKeywordsListV->length() << ')' << endl;


            // client call for category struct_mon_details:
            CORBA::Long structMonDetailsListSize = entryV->struct_mon_details_list_size();
            cout << "struct_mon_details has " << structMonDetailsListSize << " elements (actually ";

            MMS::StructMonDetailsList_var structMonDetailsListV = entryV->get_struct_mon_details_list();
            cout << structMonDetailsListV->length() << ')' << endl;


            // client call for category struct_mon_nucl:
            CORBA::Long structMonNuclListSize = entryV->struct_mon_nucl_list_size();
            cout << "struct_mon_nucl has " << structMonNuclListSize << " elements (actually ";

            MMS::StructMonNuclList_var structMonNuclListV = entryV->get_struct_mon_nucl_list();
            cout << structMonNuclListV->length() << ')' << endl;


            // client call for category struct_mon_prot:
            CORBA::Long structMonProtListSize = entryV->struct_mon_prot_list_size();
            cout << "struct_mon_prot has " << structMonProtListSize << " elements (actually ";

            MMS::StructMonProtList_var structMonProtListV = entryV->get_struct_mon_prot_list();
            cout << structMonProtListV->length() << ')' << endl;


            // client call for category struct_mon_prot_cis:
            CORBA::Long structMonProtCisListSize = entryV->struct_mon_prot_cis_list_size();
            cout << "struct_mon_prot_cis has " << structMonProtCisListSize << " elements (actually ";

            MMS::StructMonProtCisList_var structMonProtCisListV = entryV->get_struct_mon_prot_cis_list();
            cout << structMonProtCisListV->length() << ')' << endl;


            // client call for category struct_ncs_dom:
            CORBA::Long structNcsDomListSize = entryV->struct_ncs_dom_list_size();
            cout << "struct_ncs_dom has " << structNcsDomListSize << " elements (actually ";

            MMS::StructNcsDomList_var structNcsDomListV = entryV->get_struct_ncs_dom_list();
            cout << structNcsDomListV->length() << ')' << endl;


            // client call for category struct_ncs_dom_lim:
            CORBA::Long structNcsDomLimListSize = entryV->struct_ncs_dom_lim_list_size();
            cout << "struct_ncs_dom_lim has " << structNcsDomLimListSize << " elements (actually ";

            MMS::StructNcsDomLimList_var structNcsDomLimListV = entryV->get_struct_ncs_dom_lim_list();
            cout << structNcsDomLimListV->length() << ')' << endl;


            // client call for category struct_ncs_ens:
            CORBA::Long structNcsEnsListSize = entryV->struct_ncs_ens_list_size();
            cout << "struct_ncs_ens has " << structNcsEnsListSize << " elements (actually ";

            MMS::StructNcsEnsList_var structNcsEnsListV = entryV->get_struct_ncs_ens_list();
            cout << structNcsEnsListV->length() << ')' << endl;


            // client call for category struct_ncs_ens_gen:
            CORBA::Long structNcsEnsGenListSize = entryV->struct_ncs_ens_gen_list_size();
            cout << "struct_ncs_ens_gen has " << structNcsEnsGenListSize << " elements (actually ";

            MMS::StructNcsEnsGenList_var structNcsEnsGenListV = entryV->get_struct_ncs_ens_gen_list();
            cout << structNcsEnsGenListV->length() << ')' << endl;


            // client call for category struct_ncs_oper:
            CORBA::Long structNcsOperListSize = entryV->struct_ncs_oper_list_size();
            cout << "struct_ncs_oper has " << structNcsOperListSize << " elements (actually ";

            MMS::StructNcsOperList_var structNcsOperListV = entryV->get_struct_ncs_oper_list();
            cout << structNcsOperListV->length() << ')' << endl;


            // client call for category struct_ref:
            CORBA::Long structRefListSize = entryV->struct_ref_list_size();
            cout << "struct_ref has " << structRefListSize << " elements (actually ";

            MMS::StructRefList_var structRefListV = entryV->get_struct_ref_list();
            cout << structRefListV->length() << ')' << endl;


            // client call for category struct_ref_seq:
            CORBA::Long structRefSeqListSize = entryV->struct_ref_seq_list_size();
            cout << "struct_ref_seq has " << structRefSeqListSize << " elements (actually ";

            MMS::StructRefSeqList_var structRefSeqListV = entryV->get_struct_ref_seq_list();
            cout << structRefSeqListV->length() << ')' << endl;


            // client call for category struct_ref_seq_dif:
            CORBA::Long structRefSeqDifListSize = entryV->struct_ref_seq_dif_list_size();
            cout << "struct_ref_seq_dif has " << structRefSeqDifListSize << " elements (actually ";

            MMS::StructRefSeqDifList_var structRefSeqDifListV = entryV->get_struct_ref_seq_dif_list();
            cout << structRefSeqDifListV->length() << ')' << endl;


            // client call for category struct_sheet:
            CORBA::Long structSheetListSize = entryV->struct_sheet_list_size();
            cout << "struct_sheet has " << structSheetListSize << " elements (actually ";

            MMS::StructSheetList_var structSheetListV = entryV->get_struct_sheet_list();
            cout << structSheetListV->length() << ')' << endl;


            // client call for category struct_sheet_hbond:
            CORBA::Long structSheetHbondListSize = entryV->struct_sheet_hbond_list_size();
            cout << "struct_sheet_hbond has " << structSheetHbondListSize << " elements (actually ";

            MMS::StructSheetHbondList_var structSheetHbondListV = entryV->get_struct_sheet_hbond_list();
            cout << structSheetHbondListV->length() << ')' << endl;


            // client call for category struct_sheet_order:
            CORBA::Long structSheetOrderListSize = entryV->struct_sheet_order_list_size();
            cout << "struct_sheet_order has " << structSheetOrderListSize << " elements (actually ";

            MMS::StructSheetOrderList_var structSheetOrderListV = entryV->get_struct_sheet_order_list();
            cout << structSheetOrderListV->length() << ')' << endl;


            // client call for category struct_sheet_range:
            CORBA::Long structSheetRangeListSize = entryV->struct_sheet_range_list_size();
            cout << "struct_sheet_range has " << structSheetRangeListSize << " elements (actually ";

            MMS::StructSheetRangeList_var structSheetRangeListV = entryV->get_struct_sheet_range_list();
            cout << structSheetRangeListV->length() << ')' << endl;


            // client call for category struct_sheet_topology:
            CORBA::Long structSheetTopologyListSize = entryV->struct_sheet_topology_list_size();
            cout << "struct_sheet_topology has " << structSheetTopologyListSize << " elements (actually ";

            MMS::StructSheetTopologyList_var structSheetTopologyListV = entryV->get_struct_sheet_topology_list();
            cout << structSheetTopologyListV->length() << ')' << endl;


            // client call for category struct_site:
            CORBA::Long structSiteListSize = entryV->struct_site_list_size();
            cout << "struct_site has " << structSiteListSize << " elements (actually ";

            MMS::StructSiteList_var structSiteListV = entryV->get_struct_site_list();
            cout << structSiteListV->length() << ')' << endl;


            // client call for category struct_site_gen:
            CORBA::Long structSiteGenListSize = entryV->struct_site_gen_list_size();
            cout << "struct_site_gen has " << structSiteGenListSize << " elements (actually ";

            MMS::StructSiteGenList_var structSiteGenListV = entryV->get_struct_site_gen_list();
            cout << structSiteGenListV->length() << ')' << endl;


            // client call for category struct_site_keywords:
            CORBA::Long structSiteKeywordsListSize = entryV->struct_site_keywords_list_size();
            cout << "struct_site_keywords has " << structSiteKeywordsListSize << " elements (actually ";

            MMS::StructSiteKeywordsList_var structSiteKeywordsListV = entryV->get_struct_site_keywords_list();
            cout << structSiteKeywordsListV->length() << ')' << endl;


            // client call for category struct_site_view:
            CORBA::Long structSiteViewListSize = entryV->struct_site_view_list_size();
            cout << "struct_site_view has " << structSiteViewListSize << " elements (actually ";

            MMS::StructSiteViewList_var structSiteViewListV = entryV->get_struct_site_view_list();
            cout << structSiteViewListV->length() << ')' << endl;

	    /// manually added biol_unit
	    try {
	      MMS::BiolUnit_var biolUnitV = entryV->get_biol_unit(1);
	      cout << "got the biol unit" << endl;
	    } catch (MMS::DataAccessException &da) {
	      cout << "CANNOT get the BIOL unit" << endl;
	    }
	    
	    gettimeofday(&tv_end, &tz);
	 double dt = delta(tv_beg, tv_end), tt = delta(tv_zero, tv_end);

	 cout << i << '\t' << entryIdList[i] << ' ' << dt << " sec ("
	 << tt << " sec )" << endl;

      } // for each entry
    } // try, or
	catch (MMS::DataAccessException &da)
      {
	cerr << "DataAccessException in " << da.method_name 
	     << ": " << da.description << endl;
      }
    catch(CORBA::Exception &ex) 
    {
      cerr << "Unexpected CORBA exception"
	   <<": " << ex
	   << endl;
    }

 finish:
    try 
    {
        global_orb->shutdown(1);
        global_orb->destroy();
    }
    catch (...) 
    { 
        // Do nothing.
    }        
    return 0;
}


double delta(struct timeval tv_beg, struct timeval tv_end)
{
  return tv_end.tv_sec - tv_beg.tv_sec
	 + (tv_end.tv_usec - tv_beg.tv_usec)/1000000.0;
}
