#!/usr/bin/env python
# A python client for the NDB CORBA server
# Dr
# Usage:
#    python client.py [omniORBpy options...]
# NDB CORBA Naming Service is used:
# either specify in in /etc/omniORB.cfg as
#    InitRef=NameService=corbaname::pdb-bioshark-7.rutgers.edu
# or supply as an option when invoking:
#    python client.py -ORBInitRef=corbaname::pdb-bioshark-7.rutgers.edu
# Contact:
# Dr. John Westbrook <jwest@rcsb.rutgers.edu>
# Alexy Khrabrov     <alexy@rcsb.rutgers.edu>

import sys

# Import the CORBA module

from omniORB import CORBA

# Import the stubs for the CosNaming

import CosNaming

# Import the MMS module

import MMS

# Initialise the ORB

orb = CORBA.ORB_init(sys.argv, CORBA.ORB_ID)

# Obtain a reference to the root naming context

obj         = orb.resolve_initial_references("NameService")
rootContext = obj._narrow(CosNaming.NamingContext)

if rootContext is None:
    print "Failed to narrow the root naming context"
    sys.exit(1)

# Name of the CORBA object representing the database
# The first component is the naming context
# For PDB, the naming context is "PDB", for NDB, "NDB"

name = [CosNaming.NameComponent("PDB", ""),
        CosNaming.NameComponent("EntryFactory", "")]

try:
    obj = rootContext.resolve(name)
except CosNaming.NamingContext.NotFound, ex:
    print "Name not found"

# Narrow the object to an MMS::EntryFactory

ef = obj._narrow(MMS.EntryFactory)

if (ef is None):
    print "Object reference is not an MMS::EntryFactory"
    sys.exit(1)

print "got the factory"

if len(sys.argv) > 1:
    sid = sys.argv[1]
else:
    sid = "1J59" # "PD0299" -- ID reflects the database

try:
    e = ef.get_entry_from_id(sid);
except:
    print "cannot get entry %, exiting!" % sid
    sys.exit(1)

print "got entry %s" % sid

# Get the atom site list

atoms = e.get_atom_site_list()

print "got %d atoms total" % (len(atoms))

print "A few atoms:"
for a in atoms[:10]:
    print "%s\t%.3f %.3f %.3f" % (a.id, a.cartn.x, a.cartn.y, a.cartn.z)

# Get the symmetry information

s = e.get_sym_info()

print "Symmetry:"
print "space group: %s" % s.space_group
print "cell constants: ",
c = s.acell.unit_cell
print "a=%.3f, b=%.3f, c=%.3f, alpha=%.3f, beta=%.3f, gamma=%.3f" % \
      (c.length_a, c.length_b, c.length_c, c.angle_alpha, c.angle_beta, c.angle_gamma)
print "Transformation vector (column 0) and matrix:"

for i in range(0,3):
    print s.atransf.vector[i], '\t',
    for j in range(0,3):
        print s.atransf.matrix[i][j], '\t',
    print


# Get the secondary structures

sconfs = e.get_struct_conf_list()

print "Secondary structures:"
for a in sconfs:
    print a.id, '\t', \
          a.beg_auth.asym.id, a.beg_auth.comp.id, a.beg_auth.seq.id, '\t-->', \
          a.end_auth.asym.id, a.end_auth.comp.id, a.end_auth.seq.id
          
# number of biological units

num_biol_units = e.biol_unit_list_size()

print "we have %d biological unit" % num_biol_units,
if num_biol_units > 1: print "\bs"
