#ifndef _CONNPOOL_H_
#define _CONNPOOL_H_

extern "C" {
#include <pthread.h>
#include <sqlcli1.h>
}
#include <vector>
using namespace std;

class DBException { };

class ConnectionPool
{
public:
  ConnectionPool();
  ConnectionPool(const char *dbAlias, 
		 const char *user, 
		 const char *pswd, 
		 const int max)
    throw (DBException);
  
  void init(const char *dbAlias, 
		 const char *user, 
		 const char *pswd, 
		 const int max)
    throw (DBException);


  virtual ~ConnectionPool();
  void disconnectAll();

  bool empty();

  SQLHANDLE getConnection()
    throw (DBException);

  void returnConnection(const SQLHANDLE);

private:
  bool                _connected;
  int                 _max;

  SQLHANDLE           _henv;
  vector<SQLHANDLE>   _pool;

  pthread_mutex_t     _mutex;
  void createConnections(const int num);

  char * _dbAlias;
  char * _user;
  char * _pswd;

  unsigned long _num;
};

#endif
