// external test for wrapper
// $Rev$ $Id: main.C,v 1.2 2003/06/05 21:14:58 alexy Exp $

#include "optionc.h"
#include "wrapper.h"

#include <iostream>

int main(int argc, char **argv)
{
  opt::parse(argc, argv);

  orb::Factory factory(argc, argv);

  char *structure_id = opt::sid;
  
  cerr << "getting " << structure_id << endl;

  try {
    orb::Entry e(structure_id);
    biol::AtomSiteList atoms = e.get_atom_site_list();

    cout << atoms.size() << " atoms total" << endl;

#if 0
    for (int i = 0; /*i < 10 &&*/ i < atoms.size(); ++i) {
      cout << i << ": ";
      atoms[i].show();
      // cout << atoms[i].id;
      cout << endl;
    }
#endif

#if 1
    biol::AsymUnit au;
    try {
      au = e.get_asym_unit();
    }
    catch(...) {
      cerr << "cannot get asym_unit, exiting" << endl;
      exit(1);
    }
    cerr << "got asym unit" << endl;

    for (int i = 0; i < 10 && i < atoms.size(); ++i) {
	au.atom_site_list[i].show();
	cout << endl;
    }

    cout << "The transformation matrix is:" << endl;

    for (int i = 0; i < 3; ++i) {
      for (int j = 0; j < 3; ++j)
	cout << '\t' << au.sym_info.transf.matrix[i][j];
      cout << endl;
    }
#endif

    long bn;
    biol::BiolUnit bu;

    try {
      bn = e.biol_unit_list_size();
    }
    catch(...) {
      cerr << "cannot get biol_unit_list_size, exiting" << flush << endl;
      exit(1);
    }
    cout << "there are " << bn << " biological units" << endl;
    
    try {
      bu = e.get_biol_unit(1);
    }
    catch(...) {
      cerr << "cannot get biol_unit, exiting" << flush << endl;
      exit(1);
    }
    cout << "our biol unit has " << bu.replicas.size() << " replicas" << endl;
    for (unsigned i = 0; i < bu.replicas.size(); ++i)
      cout << "replica #" << i << " has " << bu.replicas[i].atom_site_list.size() << " atoms"
	   << ", its symop is [" << bu.replicas[i].symop << ']'
	   << ", asym_id " << bu.replicas[i].asym_id
	   << endl;

  } catch (orb::DataAccessException &e) {
    cerr << "Data Access Exception: " << e.method_name << ' ' << e.description << endl;
  }

  return 0;
}
