#include "mms_biol.h"
#include "io.h"


/////////////////
// MMS <= biol //
/////////////////

MMS::Cell mms_biol::conv(const biol::Cell & o)
{
  MMS::Cell r;

  r.unit_cell.angle_alpha = o.alpha;
  r.unit_cell.angle_beta  = o.beta;
  r.unit_cell.angle_gamma = o.gamma;
  r.unit_cell.length_a = o.a;
  r.unit_cell.length_b = o.b;
  r.unit_cell.length_c = o.b;

  return r;
}


MMS::Transf mms_biol::conv(const biol::Transf & o)
{
  MMS::Transf r;

  for (int i = 0; i < 3; ++i) {
    r.vector[i] = o.vector[i];
    for (int j = 0; j < 3; ++j)
      r.matrix[i][j] = o.matrix[i][j];
  }

  return r;
}


MMS::SymInfo mms_biol::conv(const biol::SymInfo & o)
{
#if 0
  MMS::SymInfo *r = new MMS::SymInfo();
  r->space_group = CORBA::string_dup(o.space_group.c_str());
  r->cell 	 = conv(o.cell);
  r->transf	 = conv(o.transf);
  cerr << "conv SymInfo pointer done" << endl;
  return *r;
#else
  MMS::SymInfo r;
  r.space_group = CORBA::string_dup(o.space_group.c_str());
  r.acell 	 = conv(o.cell);
  r.atransf	 = conv(o.transf);
  cerr << "conv SymInfo plain done" << endl;
  return r;
#endif  
}


MMS::VectorXYZ mms_biol::conv(const biol::VectorXYZ & o)
{
  MMS::VectorXYZ r;

  r.x = o.x;
  r.y = o.y;
  r.z = o.z;

  return r;
}


MMS::AtomIndex mms_biol::conv(const biol::AtomIndex & o)
{
  MMS::AtomIndex r;

  r.atom.id = CORBA::string_dup(o.atom.c_str());
  r.seq.id  = CORBA::string_dup(o.seq.c_str());
  r.comp.id = CORBA::string_dup(o.comp.c_str());
  r.asym.id = CORBA::string_dup(o.asym.c_str());

  return r;
}


MMS::AtomSite mms_biol::conv(const biol::AtomSite & o)
{
  MMS::AtomSite r;

  r.cartn = conv(o.cartn);
  // r.id = o.id.c_str();
  r.label = conv(o.label);

  return r;
}


MMS::AtomSiteList mms_biol::conv(const biol::AtomSiteList & o)
{
  MMS::AtomSiteList r(o.size());
  r.length(o.size());

  cerr << "MMS::AtomSiteList = " << r.length() << endl;
  
  for (int i = 0; i < o.size(); ++i)
    r[i] = conv(o[i]);
  
  cerr << "MMS::AtomSiteList done " << endl;
  return r;  
}


MMS::AsymUnit mms_biol::conv(const biol::AsymUnit & o)
{
  MMS::AsymUnit r;

  r.sym_info = conv(o.sym_info);
  r.atom_site_list = conv(o.atom_site_list);

  return r;
}


MMS::AsymUnitReplica mms_biol::conv(const biol::AsymUnitReplica & o)
{
  MMS::AsymUnitReplica r;

  r.asym_id = CORBA::string_dup(o.asym_id.c_str());
  r.symop   = CORBA::string_dup(o.symop.c_str());
  r.atom_site_list = conv(o.atom_site_list);
  
  return r;
}


MMS::AsymUnitReplicaList mms_biol::conv(const biol::AsymUnitReplicaList & o)
{
  MMS::AsymUnitReplicaList r;
  r.length(o.size());

  for (int i = 0; i < o.size(); ++i)
    r[i] = conv(o[i]);

  return r;
}



MMS::BiolUnit mms_biol::conv(const biol::BiolUnit & o)
{
  MMS::BiolUnit r;

  r.sym_info = conv(o.sym_info);
  cerr << "conv BiolUnit: sym_info done" << endl;
  r.replicas = conv(o.replicas);

  return r;
}


/////////////////
// biol <= MMS //
/////////////////

biol::Cell mms_biol::conv(const MMS::Cell & o)
{
  biol::Cell r;

  r.alpha = o.unit_cell.angle_alpha;
  r.beta  = o.unit_cell.angle_beta;
  r.gamma = o.unit_cell.angle_gamma;
  r.a = o.unit_cell.length_a;
  r.b = o.unit_cell.length_b;
  r.c = o.unit_cell.length_c;

  return r;
}


biol::Transf mms_biol::conv(const MMS::Transf & o)
{
  biol::Transf r;

  for (int i = 0; i < 3; ++i) {
    r.vector[i] = o.vector[i];
    // cerr << ">>";
    for (int j = 0; j < 3; ++j) 
	{
      r.matrix[i][j] = o.matrix[i][j];
      // cerr << "\t" << r.matrix[i][j];
	}
    // cerr << endl;
  }

  return r;
}


biol::SymInfo mms_biol::conv(const MMS::SymInfo & o)
{
  biol::SymInfo r;

  r.space_group = string(o.space_group);
  r.cell 	= conv(o.acell);
  r.transf	= conv(o.atransf);

  return r;
}


biol::VectorXYZ mms_biol::conv(const MMS::VectorXYZ & o)
{
  biol::VectorXYZ r;

  r.x = o.x;
  r.y = o.y;
  r.z = o.z;

  return r;
}


biol::AtomIndex mms_biol::conv(const MMS::AtomIndex & o)
{
  biol::AtomIndex r;

  r.atom = string(o.atom.id);
  r.seq  = string(o.seq.id);
  r.comp = string(o.comp.id);
  r.asym = string(o.asym.id);

  return r;
}


biol::AtomSite mms_biol::conv(const MMS::AtomSite & o)
{
  biol::AtomSite r;

  r.id = atoi(o.id);
  r.group_PDB = string(o.group_pdb);
  r.cartn = conv(o.cartn);
  r.label = conv(o.label);

  return r;
}


biol::AtomSiteList mms_biol::conv(const MMS::AtomSiteList & o)
{
  biol::AtomSiteList r(o.length());

  for (int i = 0; i < o.length(); ++i)
    r[i] = conv(o[i]);

  return r;  
}


biol::AsymUnit mms_biol::conv(const MMS::AsymUnit & o)
{
  biol::AsymUnit r;

  r.sym_info = conv(o.sym_info);
  r.atom_site_list = conv(o.atom_site_list);

  return r;
}


biol::AsymUnitReplica mms_biol::conv(const MMS::AsymUnitReplica & o)
{
  biol::AsymUnitReplica r;

  r.asym_id = string(o.asym_id);
  r.symop   = string(o.symop);
  r.atom_site_list = conv(o.atom_site_list);
  
  return r;
}


biol::AsymUnitReplicaList mms_biol::conv(const MMS::AsymUnitReplicaList & o)
{
  biol::AsymUnitReplicaList r(o.length());

  for (int i = 0; i < o.length(); ++i)
    r[i] = conv(o[i]);

  return r;
}



biol::BiolUnit mms_biol::conv(const MMS::BiolUnit & o)
{
  biol::BiolUnit r;

  r.sym_info = conv(o.sym_info);
  r.replicas = conv(o.replicas);

  return r;
}
