// Braver Entry Factory
// Server
// $Id: server.C,v 1.13 2003/06/05 21:14:58 alexy Exp $

#include <iostream>
#include <fstream>
#include <string>
#include <cstdlib>
#include <signal.h>
#include <pthread.h>
// Could use an ORB-specific ThreadHandler
// for interrupt processing...
// #define SIGWAIT
#ifndef SIGWAIT
#include <semaphore.h>
#endif

#include "C_someS.h"
#include "C_NamingService.h"
#include "C_ExceptionStream.h"

#include "MMS_EntryFactoryImpl.h"

#include "options.h"
#include "dba.h" // corba_name
#define PNDB dba::dba->corba_name.c_str()
#include "cp.h"

CORBA::ORB_var 
global_orb = CORBA::ORB::_nil();

int got_int = 0;
#ifndef SIGWAIT
sem_t semaphore;
void catch_int(int sig_num) // sig_num unused
{
  if (got_int++) {
    cerr << "SIGINT call# " << got_int << endl;
    return;
  }
  sem_post(&semaphore);
}
#endif

pthread_t the_thread;

void* thread_waiting_for_sigint(void* unused)
{
#ifdef SIGWAIT
  sigset_t mask_set; int sig_num;
  sigaddset(&mask_set, SIGINT);
  sigwait(&mask_set, &sig_num);
  ++got_int;
#else
  sem_wait(&semaphore);
#endif

  /* shutdown everything */
    
  cerr << "interrupt cleanup" << endl;
  connpool::cp.disconnectAll();
  
  global_orb->shutdown(true);
  /* do not exit() here */
  return NULL;
}


// write_reference() -- export object reference to file.
void write_reference(CORBA::Object_ptr refP,
		     const char* objref_file)
{
    CORBA::String_var stringified_ref = global_orb->object_to_string(refP);
    cout << "Writing stringified object reference to " << objref_file << endl;
    
    ofstream os(objref_file);
    os << stringified_ref;
    if (!os.good())
    {
        cerr << "Failed to write to " << objref_file << endl;
    }
}


int main(int argc, char **argv)
{

  // initialize the semaphore and install signal handler,
  // set the INT (Ctrl-C) signal handler to 'catch_int',
  // _before_ starting the waiting thread!
#ifndef SIGWAIT
  sem_init(&semaphore,0,0);
  signal(SIGINT, catch_int);
#endif
  pthread_create(&the_thread,NULL,thread_waiting_for_sigint,NULL);

  // Variables to hold servants.
  PortableServer::ServantBase_var the_MMS_EntryFactory = 0;

  // check elaboration

  try
    {
      CORBA::Object_var objV;  // For temporary object references.

      // Initialise the ORB and Root POA.
      cout << "Initializing the ORB" << endl;
	
// parse the ORB options
      global_orb = CORBA::ORB_init(argc, argv);
// parse the non-ORB options after the ORB ones are removed...
      opt::parse(argc, argv);

      // Options initialize dba::dba->corba_name, used to define
      // the naming context below.
      // In case the Naming Service is not running, opening
      // this database connection would be in vain,
      // so we could separate out corba_name init.
      
      connpool::dbcp dbcp; // connection pool manager
      
      const char rootPOA[] = "RootPOA";
      objV = global_orb->resolve_initial_references(rootPOA);
      PortableServer::POA_var root_poaV
	= PortableServer::POA::_narrow(objV);
      if (CORBA::is_nil(root_poaV.in()) ) {
	cerr << "error: failed to narrow root POA." << endl;
	exit(1);
      }
      PortableServer::POAManager_var root_poa_managerV
	= root_poaV->the_POAManager();
      if (CORBA::is_nil(root_poa_managerV.in()) ) {
	cerr << "error: failed to narrow root POA manager." << endl;
	exit(1);
      }

      // NB:: braver moved it up here from just before global_orb->run()
      // so that internal Entry objects can be already initialized
      // inside the EntryFactory constructor.
      // An alternative is to check for initialization at each request
      // there.

      // Activate the POA Manager and let the ORB process requests.
      root_poa_managerV->activate();


      // Obtain Naming Service reference.
      CosNaming::NamingContext_var rootContextV;
      objV = global_orb->resolve_initial_references("NameService");
      rootContextV = CosNaming::NamingContext::_narrow(objV);
      if (CORBA::is_nil(rootContextV.in()) ) {
	cerr << "error: failed to narrow root naming context." << endl;
	exit(1);
      }

      cerr << "making sure the EntryFactory naming context exists..." << endl;
      // Make sure that a 'EntryFactory' naming context exists.
      CosNaming::Name objectName(2);
      objectName.length(1);
      objectName[0].id   = CORBA::string_dup(PNDB); // PNDB define'd above via dba.h
      objectName[0].kind = CORBA::string_dup("");
      try {
	rootContextV->bind_new_context(objectName);
      }
      catch (CORBA::Exception& ex) { 
	cerr << "caught exception while binding EntryFactory: " << ex
	     << endl ;
      }


      // Create servants and export object references.
      PortableServer::ObjectId_var oidV;

      // Create a servant for interface MMS::EntryFactory.
      the_MMS_EntryFactory =
	MMS_EntryFactoryImpl::_create(root_poaV);
      oidV = root_poaV->activate_object(the_MMS_EntryFactory.in() );
      cerr << "activated the factory..." << endl;
      objV = root_poaV->id_to_reference(oidV);
      // write_reference(objV, "MMS_EntryFactory.ref");
      objectName.length(2);
      objectName[0].id   = CORBA::string_dup(PNDB);
      objectName[0].kind = CORBA::string_dup("");
      objectName[1].id   = CORBA::string_dup("EntryFactory");
      objectName[1].kind = CORBA::string_dup("");
      rootContextV->rebind(objectName, objV);

      cout << "Waiting for requests..." << endl;
      global_orb->run();
    }
  catch (CORBA::Exception& e)
    {
      cout << "Unexpected CORBA exception: " << e 
	   << endl;
    }

  // Servants are automatically deleted by '_var' types.

  try 
    {
      if (!got_int) global_orb->shutdown(1); // or how do we check orb's status?
      global_orb->destroy();
    }
  catch (...) 
    { 
      cerr << "cleanup" << endl;
      // connpool::cp.disconnectAll();
    }        
  return 0;
}
