// wrap CORBA client for external use
// $Rev$ $Id: wrapper.C,v 1.4 2003/06/05 21:14:58 alexy Exp $

#include "optionc.h"
#include "wrapper.h"
#include "mms_biol.h"

#include "C_someC.h"
#include "C_subentryC.h"
#include "C_PropertyService.h"
#include "C_NamingService.h"
#include <cstdlib> // exit()
// #include "biol_unit.h" // BiolGen

#include "io.h"

CORBA::ORB_var 
global_orb = CORBA::ORB::_nil();

class orb::FactoryBody
{
public:
  FactoryBody(MMS::EntryFactory_var);
  virtual ~FactoryBody();
  MMS::EntryFactory_var var;
};

orb::FactoryBody::FactoryBody(MMS::EntryFactory_var var) : var(var) {};
orb::FactoryBody::~FactoryBody() {};

class orb::EntryBody
{
public:
  EntryBody(MMS::Entry_var);
  virtual ~EntryBody();
  MMS::Entry_var var;
};

orb::EntryBody::EntryBody(MMS::Entry_var var) : var(var) {};
orb::EntryBody::~EntryBody() {};


orb::Factory * orb::Entry::factory = 0;

bool orbRuns = false;

orb::Factory::Factory(int & argc, char ** &argv)
{
  if (orbRuns) {
    cerr << "MMS Orb runs already, cannot start another -- exiting" << endl;
    exit(1);
  }
  try
    {
      CORBA::Object_var objV;  // For temporary object references.
      
      // Initialise the ORB.
      global_orb = CORBA::ORB_init(argc, argv);
      
      char *entry_name;
      
      // Obtain Naming Service reference.
      CosNaming::NamingContext_var rootContextV;
      objV = global_orb->resolve_initial_references("NameService");
      rootContextV = CosNaming::NamingContext::_narrow(objV);
      if (CORBA::is_nil(rootContextV.in()) ) {
	cerr << "error: failed to narrow root naming context." << endl;
	exit(1);
      }

      // Obtain 'BookRepository::Collection' object reference.
      CosNaming::Name objectName(2);
      objectName.length(2);
      objectName[0].id   = CORBA::string_dup(opt::xdb);
      objectName[0].kind = CORBA::string_dup("");
      objectName[1].id   = CORBA::string_dup("EntryFactory");
      objectName[1].kind = CORBA::string_dup("");

      objV = rootContextV->resolve(objectName);

      // _var, or * both work here!
      MMS::EntryFactory_var factoryV = MMS::EntryFactory::_narrow(objV);
      if (CORBA::is_nil(factoryV))
	{
	  cerr << "error: failed to narrow to MMS::EntryFactory." << endl;
	  exit(1);
	}

      // Invoke 'MMS::EntryFactory' object.
      cout << "Details of entry factory:" << endl;
      CORBA::String_var name_strV;
      name_strV = factoryV->name();
      cout << "\tName of factory = \"" << name_strV.in() << "\"" << endl;

      long n_entries = factoryV->get_entry_id_list_size();
      cout << "\tNumber of entries = " << n_entries << endl;

      cout << "We have the following entries:" << endl;

      MMS::EntryIdList_var entryIdList = factoryV->get_entry_id_list();

      cout << ' ' << entryIdList->length() << " total" << endl;


      for (unsigned i = 0; i < entryIdList->length(); ++i)
	cout << ' ' << entryIdList[i]; // .in() ?
      cout << endl;

      ptr = new FactoryBody(factoryV);
      orb::Entry::factory = this;
      orbRuns = true;
      cerr << "ORB Running..." << endl;
    }
  catch(MMS::DataAccessException &da)
    {
      cerr << "DataAccessException in " << da.method_name 
	   << ": " << da.description << endl;
    }
  catch(CORBA::Exception &ex) 
    {
      cerr << "Unexpected CORBA exception"
#ifdef _MICO_
	   <<": " << ex
#endif
	   << endl;
    }
}


orb::Factory::~Factory()
{
  try 
    {
      delete ptr;
      global_orb->shutdown(1);
      global_orb->destroy();
    }
  catch (...) 
    { 
      // Do nothing.
    }
  orbRuns = false;
  cerr << "Destroyed Factory" << endl;
}


orb::Entry::Entry(const char *const entry_id) throw (orb::DataAccessException)
{
  if (!orbRuns) {
    cerr << "MMS Orb does NOT run, exiting" << endl;
    exit(1);
  }
  try {
    ptr = new EntryBody(factory->ptr->var->get_entry_from_id((const char*)entry_id));
  }
  catch (MMS::DataAccessException &e) {
    throw DataAccessException(e.method_name, e.description);
  }
}


orb::Entry::~Entry()
{
  delete ptr;
}


long orb::Entry::atom_list_size()
{
  return ptr->var->atom_site_list_size();
}


biol::AtomSiteList orb::Entry::get_atom_site_list()
{
#define STEPS
  biol::AtomSiteList r
#ifndef STEPS
    (atomSiteListV->length())
#else
    (0)
#endif
    ;
  
  MMS::AtomSiteList_var atomSiteListV = ptr->var->get_atom_site_list();
  MMS::AtomSiteExtList_var atomSiteExtListV = ptr->var->get_atom_site_ext_list();

  //  if (!atomSiteListV) cerr << "orb::Entry::get_atom_site_list(): empty atomSiteListV!" << endl;
  //  else cerr << "orb::Entry::get_atom_site_list(): atomSiteListV->length() = " << atomSiteListV->length()<< endl;

  for (unsigned i = 0; i < atomSiteListV->length(); ++i) {
#ifdef STEPS
    // gcc 3.3 insists on an argument to push_back()!
    biol::AtomSite bas;
    r.push_back(bas);
#endif
    r[i].id = atoi(atomSiteListV[i].id); // NB!
    r[i].group_PDB = string(atomSiteListV[i].group_pdb); // was atomSiteExtListV
    r[i].cartn.x= atomSiteListV[i].cartn.x;
    r[i].cartn.y= atomSiteListV[i].cartn.y;
    r[i].cartn.z= atomSiteListV[i].cartn.z;
    r[i].label.asym = string(atomSiteListV[i].label.asym.id);
    r[i].label.comp = string(atomSiteListV[i].label.comp.id);
    r[i].label.seq  = string(atomSiteListV[i].label.seq.id);
    r[i].label.atom = string(atomSiteListV[i].label.atom.id);
    r[i].auth.asym = string(atomSiteExtListV[i].auth_asym_id);
    r[i].auth.comp = string(atomSiteExtListV[i].auth_comp_id);
    r[i].auth.seq  = string(atomSiteExtListV[i].auth_seq_id);
    r[i].auth.atom = string(atomSiteExtListV[i].auth_atom_id);
  }
  return r;
}


// first, we tested a local, client-end biol_gen;
// now we use the server's version directly...
#undef BIOL_LOCAL
#ifdef BIOL_LOCAL
biol::SymInfo orb::Entry::get_sym_info()
  // throw (orb::DataAccessException)
{
  biol::SymInfo r;
  MMS::Cell_var		cellV = ptr->var->get_cell();
  MMS::AtomSites_var	atomSitesV = ptr->var->get_atom_sites();
  MMS::Symmetry_var	symmetryV  = ptr->var->get_symmetry();

  if (cellV) {
    cerr << "HAS cell for " << ptr->var->get_id() << endl; 
    r.cell.alpha = cellV->unit_cell.angle_alpha;
    r.cell.beta  = cellV->unit_cell.angle_beta;
    r.cell.gamma = cellV->unit_cell.angle_gamma;
    r.cell.a     = cellV->unit_cell.length_a;
    r.cell.b     = cellV->unit_cell.length_b;
    r.cell.c     = cellV->unit_cell.length_c;
  }
  else {
    cerr << "Empty CELL for " << ptr->var->get_id() << endl;
    r.cell.alpha = -1.0;
    r.cell.beta  = -1.0; 
    r.cell.gamma = -1.0; 
    r.cell.a     = -1.0; 
    r.cell.b     = -1.0; 
    r.cell.c     = -1.0;
  }

  if (atomSitesV) {
    cerr << "HAS atom_sites for " << ptr->var->get_id() << endl; 
    r.transf.matrix = atomSitesV->fract_transf_matrix;
    r.transf.vector = atomSitesV->fract_transf_vector;
  }
  else {
    cerr << "Empty ATOM_SITES for " << ptr->var->get_id() << endl;
    for (int i = 0; i < 3; ++i) {
      r.transf.vector[i] = -1.0;
      for (int j = 0; j < 3; ++j)
	r.transf.matrix[i][j] = -1.0;
    }
  }

  if (symmetryV) {
    cerr << "HAS symmetry for " << ptr->var->get_id() << endl; 
    r.space_group = symmetryV->space_group_name_h_m;
  }
  else {
    cerr << "Empty SYMMETRY for " << ptr->var->get_id() << endl;
    r.space_group = "";
  }
  
  return r;
}


biol::AsymUnit orb::Entry::get_asym_unit()
{
  biol::AsymUnit r;

  r.atom_site_list = get_atom_site_list();
  cerr << "orb::Entry::get_asym_unit(): atom_site_list.size() = "
       << r.atom_site_list.size() << endl;
  r.sym_info = get_sym_info();
  cerr << "orb::Entry::get_asym_unit(): got sym_info" << endl;
  return r;
}
#else  // not BIOL_LOCAL
biol::AsymUnit orb::Entry::get_asym_unit()
{
  MMS::AsymUnit_var asymUnitV = ptr->var->get_asym_unit();

  // MICO has *_var, Omni has not => _var.in():
  return mms_biol::conv(asymUnitV.in());
}
#endif // BIOL_LOCAL


biol::BiolGenList orb::Entry::get_biol_gen(int biol_id)
{
  biol::BiolGenList r;
  MMS::StructBiolGenList_var structBiolGenV = ptr->var->get_struct_biol_gen_list();
  
  for (unsigned i = 0; i < structBiolGenV->length(); ++i)
    if (structBiolGenV[i].biol.index == biol_id)
      r.push_back(biol::BiolGen(structBiolGenV[i].biol.index,
				structBiolGenV[i].asym.id,
				structBiolGenV[i].symmetry));
  return r;
}



#ifdef BIOL_LOCAL
biol::BiolUnit orb::Entry::get_biol_unit(int biol_id)
{
  // return biol_unit::biol_gen(get_asym_unit(), get_biol_gen(biol_id));
  
  
  biol::AsymUnit au = get_asym_unit();
  cerr << "got asym unit, have " << au.atom_site_list.size() << " atoms" 
       << endl;
  biol::BiolGenList bgl = get_biol_gen(biol_id);
  cerr << "got " << bgl.size() << " biol-generators" << endl;
  biol::BiolUnit bu;
  bu = biol_unit::biol_gen(au, bgl);
  return bu;
}

#else  // BIOL_LOCAL

biol::BiolUnit orb::Entry::get_biol_unit(int biol_id)
{
  MMS::BiolUnit_var biol_unitV = ptr->var->get_biol_unit(biol_id);

  return mms_biol::conv(biol_unitV.in());
}
#endif // BIOL_LOCAL

  
void biol::VectorXYZ::show() const
{
  cout << '<' << x << ',' << y << ',' << z << '>';
}


long orb::Entry::biol_unit_list_size()
{
  CORBA::Long r = ptr->var->biol_unit_list_size();

  return r;
}


void biol::AtomIndex::show() const
{
  cout << "asym= " << asym
       << ", comp= " << comp
       << ", seq= " << seq
       << ", atom= " << atom;
}


void biol::AtomSite::show() const
{
  cout << id << '\t';
  cout << "cartn: ";
  cartn.show();
  cout << "label: ";
  label.show();
//   cout << '\t';
//   cout << "auth: ";
//   auth.show();
}


orb::DataAccessException::DataAccessException(char *method_name,
					      char *description)
  : method_name(method_name), description(description)
{}
