// Testing SQLTables access to ndb4

#include <db2/sqlcli1.h>
#include <iostream>
#include <string>

struct DBException {
    DBException(const string method_name, const string description);
    
    string method_name;
    string description;
};


int main(int argc, char *argv[])
{
    SQLRETURN   sqlrc;
    SQLHANDLE   henv;  // environment handle
    SQLHANDLE   hdbc;  // connection handle
    SQLHANDLE   hstmt; // statement handle
    
    // can come from opt::
#define MAX_UID_LENGTH 18
#define MAX_PWD_LENGTH 30
    char       dbAlias [SQL_MAX_DSN_LENGTH + 1] ;
    char       user[MAX_UID_LENGTH + 1] ;
    char       pswd[MAX_PWD_LENGTH + 1] ;

    if (argc != 4) {
      cerr << "usage: " << argv[0] << " database user password" << endl;
      return 1;
    }

    strcpy(dbAlias,  argv[1]);
    strcpy(user,     argv[2]);
    strcpy(pswd,     argv[3]);

    char structure_id[10] = "1J59";
      
    SQLCHAR * stmt = ( SQLCHAR * )
        "SELECT count(*) FROM DB2INST1.ATOM_SITE WHERE structure_id = ?";

    struct {
        SQLINTEGER ind;
        SQLINTEGER val;
    } total;

    try {

    /* initialize the CLI application */
    sqlrc = SQLAllocHandle( SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::DB()", "SQLAllocHandle henv");
    
    sqlrc = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc);
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::DB()", "SQLAllocHandle hdbc");
    
    cerr << "Connecting to " << dbAlias << "... ";
    sqlrc = SQLConnect( hdbc,
			(SQLCHAR *)dbAlias, SQL_NTS,
			(SQLCHAR *)user, SQL_NTS,
			(SQLCHAR *)pswd, SQL_NTS
			) ;
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::DB()", "SQLConnect");
    
    cerr << "done." << endl;

    sqlrc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw DBException("MMS_EntryImpl::atom_site_list_size()", "SQLAllocHandle");

    sqlrc = SQLPrepare(hstmt, stmt, SQL_NTS);
    if (sqlrc != SQL_SUCCESS)
        throw DBException("MMS_EntryImpl::atom_site_list_size()", "SQLPrepare");

    sqlrc = SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_CHAR, 10, 0, structure_id, 10, NULL);
    if (sqlrc != SQL_SUCCESS)
        throw DBException("MMS_EntryImpl::atom_site_list_size()", "SQLBindParameter");

    sqlrc = SQLExecute(hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw DBException("MMS_EntryImpl::atom_site_list_size()", "SQLExecute");

    sqlrc = SQLBindCol(hstmt, 1, SQL_C_LONG, &total.val, 0, &total.ind);
    if (sqlrc != SQL_SUCCESS)
        throw DBException("MMS_EntryImpl::atom_site_list_size()", "SQLBindCol");

    sqlrc = SQLFetch(hstmt);
    if (sqlrc != SQL_SUCCESS && sqlrc != SQL_NO_DATA)
        throw DBException("MMS_EntryImpl::atom_site_list_size()", "SQLFetch");

    if (sqlrc == SQL_NO_DATA_FOUND)
        throw DBException("MMS_EntryImpl::atom_site_list_size()", "no data");

    sqlrc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
    if (sqlrc != SQL_SUCCESS)
        throw DBException("MMS_EntryImpl::atom_site_list_size()", "SQLFreeHandle");
    }
    catch (...) {
      cerr << "(caught exception)" << endl;
    }

finish:
    cerr << "Disconnecting from " << dbAlias << "... ";
    sqlrc = SQLDisconnect(hdbc) ;
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::~DB()", "SQLDisconnect");
    
    cerr << "done." << endl;
    
    sqlrc = SQLFreeHandle( SQL_HANDLE_DBC, hdbc) ;
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::~DB()", "SQLFreeHandle hdbc");
    
    sqlrc = SQLFreeHandle( SQL_HANDLE_ENV, henv) ;
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::~DB()", "SQLFreeHandle henv");

    cerr << structure_id << " has " << total.val << " atom sites" << endl;
    return 0;
}


DBException::DBException(const string method_name, const string description)
{
  this->method_name = method_name;
  this->description = description;
}
