// Testing SQLTables access to ndb4

#include <db2/sqlcli1.h>
#include <iostream>
#include <string>

struct DBException {
    DBException(const string method_name, const string description);
    
    string method_name;
    string description;
};


int main(int argc, char *argv[])
{
    SQLRETURN   sqlrc;
    SQLHANDLE   henv;  // environment handle
    SQLHANDLE   hdbc;  // connection handle
    SQLHANDLE   hstmt; // statement handle
    
    // can come from opt::
#define MAX_UID_LENGTH 18
#define MAX_PWD_LENGTH 30
    char       dbAlias [SQL_MAX_DSN_LENGTH + 1] ;
    char       user[MAX_UID_LENGTH + 1] ;
    char       pswd[MAX_PWD_LENGTH + 1] ;

    if (argc != 4) {
      cerr << "usage: " << argv[0] << " database user password" << endl;
      return 1;
    }

    strcpy(dbAlias,  argv[1]);
    strcpy(user,     argv[2]);
    strcpy(pswd,     argv[3]);

    try {
    /* initialize the CLI application */
    sqlrc = SQLAllocHandle( SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::DB()", "SQLAllocHandle henv");
    
    sqlrc = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc);
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::DB()", "SQLAllocHandle hdbc");
    
    cerr << "Connecting to " << dbAlias << "... ";
    sqlrc = SQLConnect( hdbc,
			(SQLCHAR *)dbAlias, SQL_NTS,
			(SQLCHAR *)user, SQL_NTS,
			(SQLCHAR *)pswd, SQL_NTS
			) ;
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::DB()", "SQLConnect");
    
    cerr << "done." << endl;

  sqlrc = SQLAllocHandle(SQL_HANDLE_STMT, hdbc, &hstmt);
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::DB::tablePresent()", "SQLAllocHandle hstmt");

  string schema = "DB2INST1";
  string tableUpper  = "GEOM_ANGLE";
  sqlrc = SQLTables(hstmt, NULL, 0,
		    (SQLCHAR*) schema.c_str(), SQL_NTS,
		    (SQLCHAR*) tableUpper.c_str(),   SQL_NTS,
		    NULL, 0);
  if (sqlrc != SQL_SUCCESS) {
    cerr << "db::DB::tablePresent(): sqlrc = " << sqlrc << ", table = " << tableUpper << ", schema = " << schema << endl;
    throw DBException("db::DB::tablePresent()", "SQLTables");
  }
  
  sqlrc = SQLFetch( hstmt);
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::DB::tablePresent()", "SQLFetch");


  sqlrc = SQLFreeHandle(SQL_HANDLE_STMT, hstmt) ;
  if (sqlrc != SQL_SUCCESS)
    throw DBException("db::DB::tablePresent()", "SQLFreeHandle");
    
    }
    catch (...) {
      cerr << "(caught exception)" << endl;
    }

    cerr << "Disconnecting from " << dbAlias << "... ";
    sqlrc = SQLDisconnect(hdbc) ;
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::~DB()", "SQLDisconnect");
    
    cerr << "done." << endl;
    
    sqlrc = SQLFreeHandle( SQL_HANDLE_DBC, hdbc) ;
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::~DB()", "SQLFreeHandle hdbc");
    
    sqlrc = SQLFreeHandle( SQL_HANDLE_ENV, henv) ;
    if (sqlrc != SQL_SUCCESS)
      throw DBException("db::~DB()", "SQLFreeHandle henv");
}


DBException::DBException(const string method_name, const string description)
{
  this->method_name = method_name;
  this->description = description;
}
